/*
 * NFARun.h
 *
 *  Created on: 14. 2. 2014
 *      Author: tomas
 */

#ifndef NFARUN_H_
#define NFARUN_H_

#include <automaton/FSM/FSM.h>
#include <alphabet/Symbol.h>
#include <AlibException.h>

#include <climits>
#include <list>
#include <map>
#include <stdexcept>

/**
 * @brief Simulates NFA run
 */
class NFARun
{
public:
    NFARun( const automaton::FSM & fsm );

    /**
     * Simulates run of FSM with input word
     * @param word Input word
     * @return bool boolean value whether word was accepted
     */
    bool simulate( const std::list<alphabet::Symbol> & string ) const;

private:
    /**
     * Create unique state in automaton. If state with desired name is
     *  already present, tries to create state with same name and integer suffix
     * @param desiredName desired name of state
     * @return new unique state
     * @throws AutomatonException if was unable to create unique state (counter
     *  reached INT_MAX)
     */
    automaton::State createUniqueState( const std::string & desiredName ) const;

    /**
     * @return true if all symbols are in the automata's input alphabet
     */
    bool checkAlphabet( const std::list<alphabet::Symbol> & string ) const;

    /*
     * Puts automaton into configuration Q x T* and checks if
     * @param state state of automata
     * @param string unreaded substring
     * @return true if in configuration ( q, \eps ), q is accepting state
     */
    bool configuration( const automaton::State & state, const std::list<alphabet::Symbol> & string ) const;

    automaton::FSM m_fsm;
};

#endif /* NFARUN_H_ */
