/*
 * GrammarDiff.cpp
 *
 *  Created on: Apr 1, 2013
 *      Author: honza
 */

#include "GrammarDiff.h"

#include <set>
#include <map>
#include <list>
#include <utility>
#include <vector>
#include <typeinfo>
#include <iostream>
#include <algorithm>

#include "std/set.hpp"

#include "grammar/Regular/LeftLG.h"
#include "grammar/Regular/LeftRG.h"
#include "grammar/Regular/RightLG.h"
#include "grammar/Regular/RightRG.h"
#include "grammar/ContextFree/LG.h"
#include "grammar/ContextFree/CFG.h"
#include "grammar/ContextFree/EpsilonFreeCFG.h"
#include "grammar/ContextFree/CNF.h"
#include "grammar/ContextFree/GNF.h"
#include "grammar/ContextSensitive/CSG.h"
#include "grammar/ContextSensitive/NonContractingGrammar.h"
#include "grammar/Unrestricted/ContextPreservingUnrestrictedGrammar.h"
#include "grammar/Unrestricted/UnrestrictedGrammar.h"

#include "exception/AlibException.h"

bool GrammarDiff::testDiff(const grammar::LeftLG& a, const grammar::LeftLG& b) {
	return  	a.getNonterminalAlphabet() == b.getNonterminalAlphabet() &&
			a.getRules()               == b.getRules()               &&
			a.getInitialSymbol()       == b.getInitialSymbol()       &&
			a.getTerminalAlphabet()    == b.getTerminalAlphabet()    ;
}

bool GrammarDiff::testDiff(const grammar::LeftRG& a, const grammar::LeftRG& b) {
	return  	a.getNonterminalAlphabet() == b.getNonterminalAlphabet() &&
			a.getRules()               == b.getRules()               &&
			a.getInitialSymbol()       == b.getInitialSymbol()       &&
			a.getTerminalAlphabet()    == b.getTerminalAlphabet()    ;
}

bool GrammarDiff::testDiff(const grammar::RightLG& a, const grammar::RightLG& b) {
	return  	a.getNonterminalAlphabet() == b.getNonterminalAlphabet() &&
			a.getRules()               == b.getRules()               &&
			a.getInitialSymbol()       == b.getInitialSymbol()       &&
			a.getTerminalAlphabet()    == b.getTerminalAlphabet()    ;
}

bool GrammarDiff::testDiff(const grammar::RightRG& a, const grammar::RightRG& b) {
	return  	a.getNonterminalAlphabet() == b.getNonterminalAlphabet() &&
			a.getRules()               == b.getRules()               &&
			a.getInitialSymbol()       == b.getInitialSymbol()       &&
			a.getTerminalAlphabet()    == b.getTerminalAlphabet()    ;
}

bool GrammarDiff::testDiff(const grammar::LG& a, const grammar::LG& b) {
	return  	a.getNonterminalAlphabet() == b.getNonterminalAlphabet() &&
			a.getRules()               == b.getRules()               &&
			a.getInitialSymbol()       == b.getInitialSymbol()       &&
			a.getTerminalAlphabet()    == b.getTerminalAlphabet()    ;
}

bool GrammarDiff::testDiff(const grammar::CFG& a, const grammar::CFG& b) {
	return  	a.getNonterminalAlphabet() == b.getNonterminalAlphabet() &&
			a.getRules()               == b.getRules()               &&
			a.getInitialSymbol()       == b.getInitialSymbol()       &&
			a.getTerminalAlphabet()    == b.getTerminalAlphabet()    ;
}

bool GrammarDiff::testDiff(const grammar::EpsilonFreeCFG& a, const grammar::EpsilonFreeCFG& b) {
	return  	a.getNonterminalAlphabet() == b.getNonterminalAlphabet() &&
			a.getRules()               == b.getRules()               &&
			a.getInitialSymbol()       == b.getInitialSymbol()       &&
			a.getTerminalAlphabet()    == b.getTerminalAlphabet()    ;
}

bool GrammarDiff::testDiff(const grammar::CNF& a, const grammar::CNF& b) {
	return  	a.getNonterminalAlphabet() == b.getNonterminalAlphabet() &&
			a.getRules()               == b.getRules()               &&
			a.getInitialSymbol()       == b.getInitialSymbol()       &&
			a.getTerminalAlphabet()    == b.getTerminalAlphabet()    ;
}

bool GrammarDiff::testDiff(const grammar::GNF& a, const grammar::GNF& b) {
	return  	a.getNonterminalAlphabet() == b.getNonterminalAlphabet() &&
			a.getRules()               == b.getRules()               &&
			a.getInitialSymbol()       == b.getInitialSymbol()       &&
			a.getTerminalAlphabet()    == b.getTerminalAlphabet()    ;
}

bool GrammarDiff::testDiff(const grammar::CSG& a, const grammar::CSG& b) {
	return  	a.getNonterminalAlphabet() == b.getNonterminalAlphabet() &&
			a.getRules()               == b.getRules()               &&
			a.getInitialSymbol()       == b.getInitialSymbol()       &&
			a.getTerminalAlphabet()    == b.getTerminalAlphabet()    ;
}

bool GrammarDiff::testDiff(const grammar::NonContractingGrammar& a, const grammar::NonContractingGrammar& b) {
	return  	a.getNonterminalAlphabet() == b.getNonterminalAlphabet() &&
			a.getRules()               == b.getRules()               &&
			a.getInitialSymbol()       == b.getInitialSymbol()       &&
			a.getTerminalAlphabet()    == b.getTerminalAlphabet()    ;
}

bool GrammarDiff::testDiff(const grammar::ContextPreservingUnrestrictedGrammar& a, const grammar::ContextPreservingUnrestrictedGrammar& b) {
	return  	a.getNonterminalAlphabet() == b.getNonterminalAlphabet() &&
			a.getRules()               == b.getRules()               &&
			a.getInitialSymbol()       == b.getInitialSymbol()       &&
			a.getTerminalAlphabet()    == b.getTerminalAlphabet()    ;
}

bool GrammarDiff::testDiff(const grammar::UnrestrictedGrammar& a, const grammar::UnrestrictedGrammar& b) {
	return  	a.getNonterminalAlphabet() == b.getNonterminalAlphabet() &&
			a.getRules()               == b.getRules()               &&
			a.getInitialSymbol()       == b.getInitialSymbol()       &&
			a.getTerminalAlphabet()    == b.getTerminalAlphabet()    ;
}

template <class T> void GrammarDiff::setDiff(const std::set<T> a, const std::set<T> b) {
	std::set<T> aMinusB;
	std::set_difference(a.begin(), a.end(), b.begin(), b.end(), std::inserter(aMinusB, aMinusB.begin()));

	std::set<T> bMinusA;
	std::set_difference(b.begin(), b.end(), a.begin(), a.end(), std::inserter(bMinusA, bMinusA.begin()));

	for(typename std::set<T>::const_iterator iter = aMinusB.begin(); iter != aMinusB.end(); iter++) {
		std::cout << "< " << *iter << std::endl;
	}
	std::cout << "---" << std::endl;
	for(typename std::set<T>::const_iterator iter = bMinusA.begin(); iter != bMinusA.end(); iter++) {
		std::cout << "> " << *iter << std::endl;
	}
}

template <class T> void GrammarDiff::listDiff(const std::list<T> a, const std::list<T> b) {
	std::list<T> aMinusB;
	std::set_difference(a.begin(), a.end(), b.begin(), b.end(), std::inserter(aMinusB, aMinusB.begin()));

	std::list<T> bMinusA;
	std::set_difference(b.begin(), b.end(), a.begin(), a.end(), std::inserter(bMinusA, bMinusA.begin()));

	for(typename std::list<T>::const_iterator iter = aMinusB.begin(); iter != aMinusB.end(); iter++) {
		std::cout << "< " << *iter << std::endl;
	}
	std::cout << "---" << std::endl;
	for(typename std::list<T>::const_iterator iter = bMinusA.begin(); iter != bMinusA.end(); iter++) {
		std::cout << "> " << *iter << std::endl;
	}
}

template <class T, class R> void GrammarDiff::mapDiff(const std::map<T, R> a, const std::map<T, R> b) {
	std::map<T, R> aMinusB;
	std::set_difference(a.begin(), a.end(), b.begin(), b.end(), std::inserter(aMinusB, aMinusB.begin()));

	std::map<T, R> bMinusA;
	std::set_difference(b.begin(), b.end(), a.begin(), a.end(), std::inserter(bMinusA, bMinusA.begin()));

	for(typename std::map<T, R>::const_iterator iter = aMinusB.begin(); iter != aMinusB.end(); iter++) {
		std::cout << "< " << iter->first << ", " << iter->second << std::endl;
	}
	std::cout << "---" << std::endl;
	for(typename std::map<T, R>::const_iterator iter = bMinusA.begin(); iter != bMinusA.end(); iter++) {
		std::cout << "< " << iter->first << ", " << iter->second << std::endl;
	}
}

void GrammarDiff::printDiff(const grammar::LeftLG& a, const grammar::LeftLG& b) {
	std::cout << "GrammarDiffer" << std::endl;

	if(a.getNonterminalAlphabet() != b.getNonterminalAlphabet()) {
		std::cout << "Nonterminal alphabet" << std::endl;

		GrammarDiff::setDiff(a.getNonterminalAlphabet(), b.getNonterminalAlphabet());
	}

	if(a.getRules() != b.getRules()){
		std::cout << "Rules" << std::endl;

		GrammarDiff::mapDiff(a.getRules(), b.getRules());
	}

	if(a.getInitialSymbol() != b.getInitialSymbol()){
		std::cout << "Initial symbol" << std::endl;

		std::cout << "< " << a.getInitialSymbol() << std::endl;
		std::cout << "---" << std::endl;
		std::cout << "> " << b.getInitialSymbol() << std::endl;
	}

	if(a.getTerminalAlphabet() != b.getTerminalAlphabet()) {
		std::cout << "Terminal alphabet" << std::endl;

		GrammarDiff::setDiff(a.getTerminalAlphabet(), b.getTerminalAlphabet());
	}
}

void GrammarDiff::printDiff(const grammar::LeftRG& a, const grammar::LeftRG& b) {
	std::cout << "GrammarDiffer" << std::endl;

	if(a.getNonterminalAlphabet() != b.getNonterminalAlphabet()) {
		std::cout << "Nonterminal alphabet" << std::endl;

		GrammarDiff::setDiff(a.getNonterminalAlphabet(), b.getNonterminalAlphabet());
	}

	if(a.getRules() != b.getRules()){
		std::cout << "Rules" << std::endl;

		GrammarDiff::mapDiff(a.getRules(), b.getRules());
	}

	if(a.getInitialSymbol() != b.getInitialSymbol()){
		std::cout << "Initial symbol" << std::endl;

		std::cout << "< " << a.getInitialSymbol() << std::endl;
		std::cout << "---" << std::endl;
		std::cout << "> " << b.getInitialSymbol() << std::endl;
	}

	if(a.getTerminalAlphabet() != b.getTerminalAlphabet()) {
		std::cout << "Terminal alphabet" << std::endl;

		GrammarDiff::setDiff(a.getTerminalAlphabet(), b.getTerminalAlphabet());
	}
}

void GrammarDiff::printDiff(const grammar::RightLG& a, const grammar::RightLG& b) {
	std::cout << "GrammarDiffer" << std::endl;

	if(a.getNonterminalAlphabet() != b.getNonterminalAlphabet()) {
		std::cout << "Nonterminal alphabet" << std::endl;

		GrammarDiff::setDiff(a.getNonterminalAlphabet(), b.getNonterminalAlphabet());
	}

	if(a.getRules() != b.getRules()){
		std::cout << "Rules" << std::endl;

		GrammarDiff::mapDiff(a.getRules(), b.getRules());
	}

	if(a.getInitialSymbol() != b.getInitialSymbol()){
		std::cout << "Initial symbol" << std::endl;

		std::cout << "< " << a.getInitialSymbol() << std::endl;
		std::cout << "---" << std::endl;
		std::cout << "> " << b.getInitialSymbol() << std::endl;
	}

	if(a.getTerminalAlphabet() != b.getTerminalAlphabet()) {
		std::cout << "Terminal alphabet" << std::endl;

		GrammarDiff::setDiff(a.getTerminalAlphabet(), b.getTerminalAlphabet());
	}
}

void GrammarDiff::printDiff(const grammar::RightRG& a, const grammar::RightRG& b) {
	std::cout << "GrammarDiffer" << std::endl;

	if(a.getNonterminalAlphabet() != b.getNonterminalAlphabet()) {
		std::cout << "Nonterminal alphabet" << std::endl;

		GrammarDiff::setDiff(a.getNonterminalAlphabet(), b.getNonterminalAlphabet());
	}

	if(a.getRules() != b.getRules()){
		std::cout << "Rules" << std::endl;

		GrammarDiff::mapDiff(a.getRules(), b.getRules());
	}

	if(a.getInitialSymbol() != b.getInitialSymbol()){
		std::cout << "Initial symbol" << std::endl;

		std::cout << "< " << a.getInitialSymbol() << std::endl;
		std::cout << "---" << std::endl;
		std::cout << "> " << b.getInitialSymbol() << std::endl;
	}

	if(a.getTerminalAlphabet() != b.getTerminalAlphabet()) {
		std::cout << "Terminal alphabet" << std::endl;

		GrammarDiff::setDiff(a.getTerminalAlphabet(), b.getTerminalAlphabet());
	}
}

void GrammarDiff::printDiff(const grammar::LG& a, const grammar::LG& b) {
	std::cout << "GrammarDiffer" << std::endl;

	if(a.getNonterminalAlphabet() != b.getNonterminalAlphabet()) {
		std::cout << "Nonterminal alphabet" << std::endl;

		GrammarDiff::setDiff(a.getNonterminalAlphabet(), b.getNonterminalAlphabet());
	}

	if(a.getRules() != b.getRules()){
		std::cout << "Rules" << std::endl;

		GrammarDiff::mapDiff(a.getRules(), b.getRules());
	}

	if(a.getInitialSymbol() != b.getInitialSymbol()){
		std::cout << "Initial symbol" << std::endl;

		std::cout << "< " << a.getInitialSymbol() << std::endl;
		std::cout << "---" << std::endl;
		std::cout << "> " << b.getInitialSymbol() << std::endl;
	}

	if(a.getTerminalAlphabet() != b.getTerminalAlphabet()) {
		std::cout << "Terminal alphabet" << std::endl;

		GrammarDiff::setDiff(a.getTerminalAlphabet(), b.getTerminalAlphabet());
	}
}

void GrammarDiff::printDiff(const grammar::CFG& a, const grammar::CFG& b) {
	std::cout << "GrammarDiffer" << std::endl;

	if(a.getNonterminalAlphabet() != b.getNonterminalAlphabet()) {
		std::cout << "Nonterminal alphabet" << std::endl;

		GrammarDiff::setDiff(a.getNonterminalAlphabet(), b.getNonterminalAlphabet());
	}

	if(a.getRules() != b.getRules()){
		std::cout << "Rules" << std::endl;

		GrammarDiff::mapDiff(a.getRules(), b.getRules());
	}

	if(a.getInitialSymbol() != b.getInitialSymbol()){
		std::cout << "Initial symbol" << std::endl;

		std::cout << "< " << a.getInitialSymbol() << std::endl;
		std::cout << "---" << std::endl;
		std::cout << "> " << b.getInitialSymbol() << std::endl;
	}

	if(a.getTerminalAlphabet() != b.getTerminalAlphabet()) {
		std::cout << "Terminal alphabet" << std::endl;

		GrammarDiff::setDiff(a.getTerminalAlphabet(), b.getTerminalAlphabet());
	}
}

void GrammarDiff::printDiff(const grammar::EpsilonFreeCFG& a, const grammar::EpsilonFreeCFG& b) {
	std::cout << "GrammarDiffer" << std::endl;

	if(a.getNonterminalAlphabet() != b.getNonterminalAlphabet()) {
		std::cout << "Nonterminal alphabet" << std::endl;

		GrammarDiff::setDiff(a.getNonterminalAlphabet(), b.getNonterminalAlphabet());
	}

	if(a.getRules() != b.getRules()){
		std::cout << "Rules" << std::endl;

		GrammarDiff::mapDiff(a.getRules(), b.getRules());
	}

	if(a.getInitialSymbol() != b.getInitialSymbol()){
		std::cout << "Initial symbol" << std::endl;

		std::cout << "< " << a.getInitialSymbol() << std::endl;
		std::cout << "---" << std::endl;
		std::cout << "> " << b.getInitialSymbol() << std::endl;
	}

	if(a.getTerminalAlphabet() != b.getTerminalAlphabet()) {
		std::cout << "Terminal alphabet" << std::endl;

		GrammarDiff::setDiff(a.getTerminalAlphabet(), b.getTerminalAlphabet());
	}
}

void GrammarDiff::printDiff(const grammar::CNF& a, const grammar::CNF& b) {
	std::cout << "GrammarDiffer" << std::endl;

	if(a.getNonterminalAlphabet() != b.getNonterminalAlphabet()) {
		std::cout << "Nonterminal alphabet" << std::endl;

		GrammarDiff::setDiff(a.getNonterminalAlphabet(), b.getNonterminalAlphabet());
	}

	if(a.getRules() != b.getRules()){
		std::cout << "Rules" << std::endl;

		GrammarDiff::mapDiff(a.getRules(), b.getRules());
	}

	if(a.getInitialSymbol() != b.getInitialSymbol()){
		std::cout << "Initial symbol" << std::endl;

		std::cout << "< " << a.getInitialSymbol() << std::endl;
		std::cout << "---" << std::endl;
		std::cout << "> " << b.getInitialSymbol() << std::endl;
	}

	if(a.getTerminalAlphabet() != b.getTerminalAlphabet()) {
		std::cout << "Terminal alphabet" << std::endl;

		GrammarDiff::setDiff(a.getTerminalAlphabet(), b.getTerminalAlphabet());
	}
}

void GrammarDiff::printDiff(const grammar::GNF& a, const grammar::GNF& b) {
	std::cout << "GrammarDiffer" << std::endl;

	if(a.getNonterminalAlphabet() != b.getNonterminalAlphabet()) {
		std::cout << "Nonterminal alphabet" << std::endl;

		GrammarDiff::setDiff(a.getNonterminalAlphabet(), b.getNonterminalAlphabet());
	}

	if(a.getRules() != b.getRules()){
		std::cout << "Rules" << std::endl;

		GrammarDiff::mapDiff(a.getRules(), b.getRules());
	}

	if(a.getInitialSymbol() != b.getInitialSymbol()){
		std::cout << "Initial symbol" << std::endl;

		std::cout << "< " << a.getInitialSymbol() << std::endl;
		std::cout << "---" << std::endl;
		std::cout << "> " << b.getInitialSymbol() << std::endl;
	}

	if(a.getTerminalAlphabet() != b.getTerminalAlphabet()) {
		std::cout << "Terminal alphabet" << std::endl;

		GrammarDiff::setDiff(a.getTerminalAlphabet(), b.getTerminalAlphabet());
	}
}

void GrammarDiff::printDiff(const grammar::CSG& a, const grammar::CSG& b) {
	std::cout << "GrammarDiffer" << std::endl;

	if(a.getNonterminalAlphabet() != b.getNonterminalAlphabet()) {
		std::cout << "Nonterminal alphabet" << std::endl;

		GrammarDiff::setDiff(a.getNonterminalAlphabet(), b.getNonterminalAlphabet());
	}

	if(a.getRules() != b.getRules()){
		std::cout << "Rules" << std::endl;

		GrammarDiff::mapDiff(a.getRules(), b.getRules());
	}

	if(a.getInitialSymbol() != b.getInitialSymbol()){
		std::cout << "Initial symbol" << std::endl;

		std::cout << "< " << a.getInitialSymbol() << std::endl;
		std::cout << "---" << std::endl;
		std::cout << "> " << b.getInitialSymbol() << std::endl;
	}

	if(a.getTerminalAlphabet() != b.getTerminalAlphabet()) {
		std::cout << "Terminal alphabet" << std::endl;

		GrammarDiff::setDiff(a.getTerminalAlphabet(), b.getTerminalAlphabet());
	}
}

void GrammarDiff::printDiff(const grammar::NonContractingGrammar& a, const grammar::NonContractingGrammar& b) {
	std::cout << "GrammarDiffer" << std::endl;

	if(a.getNonterminalAlphabet() != b.getNonterminalAlphabet()) {
		std::cout << "Nonterminal alphabet" << std::endl;

		GrammarDiff::setDiff(a.getNonterminalAlphabet(), b.getNonterminalAlphabet());
	}

	if(a.getRules() != b.getRules()){
		std::cout << "Rules" << std::endl;

		GrammarDiff::mapDiff(a.getRules(), b.getRules());
	}

	if(a.getInitialSymbol() != b.getInitialSymbol()){
		std::cout << "Initial symbol" << std::endl;

		std::cout << "< " << a.getInitialSymbol() << std::endl;
		std::cout << "---" << std::endl;
		std::cout << "> " << b.getInitialSymbol() << std::endl;
	}

	if(a.getTerminalAlphabet() != b.getTerminalAlphabet()) {
		std::cout << "Terminal alphabet" << std::endl;

		GrammarDiff::setDiff(a.getTerminalAlphabet(), b.getTerminalAlphabet());
	}
}

void GrammarDiff::printDiff(const grammar::ContextPreservingUnrestrictedGrammar& a, const grammar::ContextPreservingUnrestrictedGrammar& b) {
	std::cout << "GrammarDiffer" << std::endl;

	if(a.getNonterminalAlphabet() != b.getNonterminalAlphabet()) {
		std::cout << "Nonterminal alphabet" << std::endl;

		GrammarDiff::setDiff(a.getNonterminalAlphabet(), b.getNonterminalAlphabet());
	}

	if(a.getRules() != b.getRules()){
		std::cout << "Rules" << std::endl;

		GrammarDiff::mapDiff(a.getRules(), b.getRules());
	}

	if(a.getInitialSymbol() != b.getInitialSymbol()){
		std::cout << "Initial symbol" << std::endl;

		std::cout << "< " << a.getInitialSymbol() << std::endl;
		std::cout << "---" << std::endl;
		std::cout << "> " << b.getInitialSymbol() << std::endl;
	}

	if(a.getTerminalAlphabet() != b.getTerminalAlphabet()) {
		std::cout << "Terminal alphabet" << std::endl;

		GrammarDiff::setDiff(a.getTerminalAlphabet(), b.getTerminalAlphabet());
	}
}

void GrammarDiff::printDiff(const grammar::UnrestrictedGrammar& a, const grammar::UnrestrictedGrammar& b) {
	std::cout << "GrammarDiffer" << std::endl;

	if(a.getNonterminalAlphabet() != b.getNonterminalAlphabet()) {
		std::cout << "Nonterminal alphabet" << std::endl;

		GrammarDiff::setDiff(a.getNonterminalAlphabet(), b.getNonterminalAlphabet());
	}

	if(a.getRules() != b.getRules()){
		std::cout << "Rules" << std::endl;

		GrammarDiff::mapDiff(a.getRules(), b.getRules());
	}

	if(a.getInitialSymbol() != b.getInitialSymbol()){
		std::cout << "Initial symbol" << std::endl;

		std::cout << "< " << a.getInitialSymbol() << std::endl;
		std::cout << "---" << std::endl;
		std::cout << "> " << b.getInitialSymbol() << std::endl;
	}

	if(a.getTerminalAlphabet() != b.getTerminalAlphabet()) {
		std::cout << "Terminal alphabet" << std::endl;

		GrammarDiff::setDiff(a.getTerminalAlphabet(), b.getTerminalAlphabet());
	}
}

int GrammarDiff::diff(const grammar::LeftLG& a, const grammar::LeftLG& b) {
	if(!GrammarDiff::testDiff(a, b)) {
	  GrammarDiff::printDiff(a, b);
	  return 1;
	} else {
	  return 0;
	}
}

int GrammarDiff::diff(const grammar::LeftRG& a, const grammar::LeftRG& b) {
	if(!GrammarDiff::testDiff(a, b)) {
	  GrammarDiff::printDiff(a, b);
	  return 1;
	} else {
	  return 0;
	}
}

int GrammarDiff::diff(const grammar::RightLG& a, const grammar::RightLG& b) {
	if(!GrammarDiff::testDiff(a, b)) {
	  GrammarDiff::printDiff(a, b);
	  return 1;
	} else {
	  return 0;
	}
}

int GrammarDiff::diff(const grammar::RightRG& a, const grammar::RightRG& b) {
	if(!GrammarDiff::testDiff(a, b)) {
	  GrammarDiff::printDiff(a, b);
	  return 1;
	} else {
	  return 0;
	}
}

int GrammarDiff::diff(const grammar::LG& a, const grammar::LG& b) {
	if(!GrammarDiff::testDiff(a, b)) {
	  GrammarDiff::printDiff(a, b);
	  return 1;
	} else {
	  return 0;
	}
}

int GrammarDiff::diff(const grammar::CFG& a, const grammar::CFG& b) {
	if(!GrammarDiff::testDiff(a, b)) {
	  GrammarDiff::printDiff(a, b);
	  return 1;
	} else {
	  return 0;
	}
}

int GrammarDiff::diff(const grammar::EpsilonFreeCFG& a, const grammar::EpsilonFreeCFG& b) {
	if(!GrammarDiff::testDiff(a, b)) {
	  GrammarDiff::printDiff(a, b);
	  return 1;
	} else {
	  return 0;
	}
}

int GrammarDiff::diff(const grammar::CNF& a, const grammar::CNF& b) {
	if(!GrammarDiff::testDiff(a, b)) {
	  GrammarDiff::printDiff(a, b);
	  return 1;
	} else {
	  return 0;
	}
}

int GrammarDiff::diff(const grammar::GNF& a, const grammar::GNF& b) {
	if(!GrammarDiff::testDiff(a, b)) {
	  GrammarDiff::printDiff(a, b);
	  return 1;
	} else {
	  return 0;
	}
}

int GrammarDiff::diff(const grammar::CSG& a, const grammar::CSG& b) {
	if(!GrammarDiff::testDiff(a, b)) {
	  GrammarDiff::printDiff(a, b);
	  return 1;
	} else {
	  return 0;
	}
}

int GrammarDiff::diff(const grammar::NonContractingGrammar& a, const grammar::NonContractingGrammar& b) {
	if(!GrammarDiff::testDiff(a, b)) {
	  GrammarDiff::printDiff(a, b);
	  return 1;
	} else {
	  return 0;
	}
}

int GrammarDiff::diff(const grammar::ContextPreservingUnrestrictedGrammar& a, const grammar::ContextPreservingUnrestrictedGrammar& b) {
	if(!GrammarDiff::testDiff(a, b)) {
	  GrammarDiff::printDiff(a, b);
	  return 1;
	} else {
	  return 0;
	}
}

int GrammarDiff::diff(const grammar::UnrestrictedGrammar& a, const grammar::UnrestrictedGrammar& b) {
	if(!GrammarDiff::testDiff(a, b)) {
	  GrammarDiff::printDiff(a, b);
	  return 1;
	} else {
	  return 0;
	}
}

template<typename T>
int GrammarDiff::dynamicDiff(const grammar::Grammar& a, const grammar::Grammar& b) {
	const T* ap = dynamic_cast<const T*>(&a.getData());
	const T* bp = dynamic_cast<const T*>(&b.getData());

	if(ap && bp) {
		return diff(*ap, *bp);
	} else if(ap || bp) {
		std::cout << "GrammarsDifferByType" << std::endl;
		return 1;
	} else {
		return -1;
	}

}

int GrammarDiff::diff(const grammar::Grammar& a, const grammar::Grammar& b) {
	int res;

	res = dynamicDiff<grammar::LeftLG>(a, b);
	if(res == 0) return 0;
	if(res == 1) return 1;

	res = dynamicDiff<grammar::LeftRG>(a, b);
	if(res == 0) return 0;
	if(res == 1) return 1;

	res = dynamicDiff<grammar::RightLG>(a, b);
	if(res == 0) return 0;
	if(res == 1) return 1;

	res = dynamicDiff<grammar::RightRG>(a, b);
	if(res == 0) return 0;
	if(res == 1) return 1;

	res = dynamicDiff<grammar::LG>(a, b);
	if(res == 0) return 0;
	if(res == 1) return 1;

	res = dynamicDiff<grammar::CFG>(a, b);
	if(res == 0) return 0;
	if(res == 1) return 1;

	res = dynamicDiff<grammar::EpsilonFreeCFG>(a, b);
	if(res == 0) return 0;
	if(res == 1) return 1;

	res = dynamicDiff<grammar::CNF>(a, b);
	if(res == 0) return 0;
	if(res == 1) return 1;

	res = dynamicDiff<grammar::GNF>(a, b);
	if(res == 0) return 0;
	if(res == 1) return 1;

	res = dynamicDiff<grammar::CSG>(a, b);
	if(res == 0) return 0;
	if(res == 1) return 1;

	res = dynamicDiff<grammar::NonContractingGrammar>(a, b);
	if(res == 0) return 0;
	if(res == 1) return 1;

	res = dynamicDiff<grammar::ContextPreservingUnrestrictedGrammar>(a, b);
	if(res == 0) return 0;
	if(res == 1) return 1;

	res = dynamicDiff<grammar::UnrestrictedGrammar>(a, b);
	if(res == 0) return 0;
	if(res == 1) return 1;

	throw exception::AlibException("Unknown grammar type");
}
