/*
 * GrammarDiff.h
 *
 *  Created on: Apr 1, 2013
 *      Author: honza
 */

#ifndef GRAMMAR_DIFF_H_
#define GRAMMAR_DIFF_H_

#include <ostream>

#include <grammar/Grammar.h>
#include <set>
#include <list>
#include <map>
#include <utility>

class GrammarDiff {
	static bool testDiff(const grammar::LeftLG& a, const grammar::LeftLG& b);
	static void printDiff(const grammar::LeftLG& a, const grammar::LeftLG& b);

	static bool testDiff(const grammar::LeftRG& a, const grammar::LeftRG& b);
	static void printDiff(const grammar::LeftRG& a, const grammar::LeftRG& b);

	static bool testDiff(const grammar::RightLG& a, const grammar::RightLG& b);
	static void printDiff(const grammar::RightLG& a, const grammar::RightLG& b);

	static bool testDiff(const grammar::RightRG& a, const grammar::RightRG& b);
	static void printDiff(const grammar::RightRG& a, const grammar::RightRG& b);

	static bool testDiff(const grammar::LG& a, const grammar::LG& b);
	static void printDiff(const grammar::LG& a, const grammar::LG& b);

	static bool testDiff(const grammar::CFG& a, const grammar::CFG& b);
	static void printDiff(const grammar::CFG& a, const grammar::CFG& b);

	static bool testDiff(const grammar::EpsilonFreeCFG& a, const grammar::EpsilonFreeCFG& b);
	static void printDiff(const grammar::EpsilonFreeCFG& a, const grammar::EpsilonFreeCFG& b);

	static bool testDiff(const grammar::CNF& a, const grammar::CNF& b);
	static void printDiff(const grammar::CNF& a, const grammar::CNF& b);

	static bool testDiff(const grammar::GNF& a, const grammar::GNF& b);
	static void printDiff(const grammar::GNF& a, const grammar::GNF& b);

	static bool testDiff(const grammar::CSG& a, const grammar::CSG& b);
	static void printDiff(const grammar::CSG& a, const grammar::CSG& b);

	static bool testDiff(const grammar::NonContractingGrammar& a, const grammar::NonContractingGrammar& b);
	static void printDiff(const grammar::NonContractingGrammar& a, const grammar::NonContractingGrammar& b);

	static bool testDiff(const grammar::ContextPreservingUnrestrictedGrammar& a, const grammar::ContextPreservingUnrestrictedGrammar& b);
	static void printDiff(const grammar::ContextPreservingUnrestrictedGrammar& a, const grammar::ContextPreservingUnrestrictedGrammar& b);

	static bool testDiff(const grammar::UnrestrictedGrammar& a, const grammar::UnrestrictedGrammar& b);
	static void printDiff(const grammar::UnrestrictedGrammar& a, const grammar::UnrestrictedGrammar& b);

	template <class T> static void setDiff(const std::set<T> a, const std::set<T> b);
	template <class T> static void listDiff(const std::list<T> a, const std::list<T> b);
	template <class T, class R> static void mapDiff(const std::map<T, R> a, const std::map<T, R> b);
public:
	static int diff(const grammar::LeftLG& a, const grammar::LeftLG& b);
	static int diff(const grammar::LeftRG& a, const grammar::LeftRG& b);
	static int diff(const grammar::RightLG& a, const grammar::RightLG& b);
	static int diff(const grammar::RightRG& a, const grammar::RightRG& b);
	static int diff(const grammar::LG& a, const grammar::LG& b);
	static int diff(const grammar::CFG& a, const grammar::CFG& b);
	static int diff(const grammar::EpsilonFreeCFG& a, const grammar::EpsilonFreeCFG& b);
	static int diff(const grammar::CNF& a, const grammar::CNF& b);
	static int diff(const grammar::GNF& a, const grammar::GNF& b);
	static int diff(const grammar::CSG& a, const grammar::CSG& b);
	static int diff(const grammar::NonContractingGrammar& a, const grammar::NonContractingGrammar& b);
	static int diff(const grammar::ContextPreservingUnrestrictedGrammar& a, const grammar::ContextPreservingUnrestrictedGrammar& b);
	static int diff(const grammar::UnrestrictedGrammar& a, const grammar::UnrestrictedGrammar& b);
	
	template<typename T>
	static int dynamicDiff(const grammar::Grammar& a, const grammar::Grammar& b);

	static int diff(const grammar::Grammar& a, const grammar::Grammar& b);
};

#endif /* GRAMMAR_DIFF_H_ */
