//============================================================================
// Name		: aepislon.cpp
// Author	: Tomas Pecka
//============================================================================

#include <iostream>

#include <factory/DataFactory.hpp>
#include <exception/AlibException.h>

#include "epsilon/fsm/FSMEpsilonRemover.h"

int main(int argc, char** argv) {

	try {
		if (argc == 2 && std::string("-h").compare(argv[1]) == 0) {
			std::cout << "Remove epsilon transitions from automaton." << std::endl << "Usage: aepsilon [automaton.xml]" << std::endl;
			return -1;
		} else if (argc == 1 || (argc == 2 && std::string("--").compare(argv[1]) == 0)) {
			alib::DataFactory::toStdout(epsilon::FSMEpsilonRemover::remove(alib::DataFactory::fromStdin<automaton::Automaton>()));
		} else if (argc == 2) {
			alib::DataFactory::toStdout(epsilon::FSMEpsilonRemover::remove(alib::DataFactory::fromStdin<automaton::Automaton>()));
		} else {
			std::cout << "Automaton minimize require deterministic finite automaton" << std::endl;
			return 1;
		}

		return 0;

	} catch (const exception::AlibException& exception) {
		alib::DataFactory::toStdout(exception);
		return 1;
	} catch(...) {
		return 127;
	}
}
