/*
 * FSMSingleInitialState.cpp
 *
 *  Created on: 20. 9. 2014
 *	  Author: Tomas Pecka
 */

#include "FSMSingleInitialState.h"

#include <algorithm>
#include <set>

#include <exception/AlibException.h>
#include <automaton/FSM/DFA.h>
#include <automaton/FSM/NFA.h>
#include <automaton/FSM/MultiInitialStateNFA.h>
#include <automaton/FSM/ExtendedNFA.h>
#include <automaton/FSM/EpsilonNFA.h>
#include <automaton/FSM/CompactNFA.h>

namespace automaton {

automaton::Automaton FSMSingleInitialState::convert(const Automaton& automaton) {
	automaton::Automaton* out = NULL;
	automaton.getData().Accept((void*) &out, FSMSingleInitialState::FSM_SINGLE_INITIAL_STATE);
	automaton::Automaton res = std::move(*out);
	delete out;
	return res;
}

automaton::NFA FSMSingleInitialState::convert(const automaton::MultiInitialStateNFA& automaton) {
	// step 3
	automaton::State q0 = automaton::createUniqueState(automaton::State("q0"), automaton.getStates());
	automaton::NFA res(q0);

	//TODO introduce addStates
	std::set<automaton::State> states = automaton.getStates();
	states.insert(q0);
	res.setStates(states);

	res.setInputSymbols(automaton.getInputAlphabet());

	// step 2
	for(const auto& q : automaton.getInitialStates()) {
		for(const auto& kv: automaton.getTransitionsFromState(q)) {
			for(const auto& to : kv.second) {
				res.addTransition(q0, kv.first.second, to);
			}
		}
	}

	// step 4, 5
	std::set<automaton::State> intersection;
	std::set_intersection(res.getFinalStates().begin(), res.getFinalStates().end(), automaton.getInitialStates().begin(), automaton.getInitialStates().end(), std::inserter(intersection, intersection.begin()));

	res.setFinalStates(automaton.getFinalStates());
	if(intersection.size() != 0) {
		res.addFinalState(q0);
	}

	return res;
}

automaton::DFA FSMSingleInitialState::convert(const automaton::DFA& automaton) {
	return automaton;
}

automaton::EpsilonNFA FSMSingleInitialState::convert(const automaton::EpsilonNFA& automaton) {
	return automaton;
}

automaton::NFA FSMSingleInitialState::convert(const automaton::NFA& automaton) {
	return automaton;
}

automaton::CompactNFA FSMSingleInitialState::convert(const automaton::CompactNFA& automaton) {
	return automaton;
}

automaton::ExtendedNFA FSMSingleInitialState::convert(const automaton::ExtendedNFA& automaton) {
	return automaton;
}

void FSMSingleInitialState::Visit(void* data, const EpsilonNFA& automaton) const {
	automaton::Automaton*& out = *((automaton::Automaton**) data);
	out = new automaton::Automaton(this->convert(automaton));
}

void FSMSingleInitialState::Visit(void* data, const MultiInitialStateNFA& automaton) const {
	automaton::Automaton*& out = *((automaton::Automaton**) data);
	out = new automaton::Automaton(this->convert(automaton));
}

void FSMSingleInitialState::Visit(void* data, const NFA& automaton) const {
	automaton::Automaton*& out = *((automaton::Automaton**) data);
	out = new automaton::Automaton(this->convert(automaton));
}

void FSMSingleInitialState::Visit(void* data, const DFA& automaton) const {
	automaton::Automaton*& out = *((automaton::Automaton**) data);
	out = new automaton::Automaton(this->convert(automaton));
}

void FSMSingleInitialState::Visit(void* data, const ExtendedNFA& automaton) const {
	automaton::Automaton*& out = *((automaton::Automaton**) data);
	out = new automaton::Automaton(this->convert(automaton));
}

void FSMSingleInitialState::Visit(void* data, const CompactNFA& automaton) const {
	automaton::Automaton*& out = *((automaton::Automaton**) data);
	out = new automaton::Automaton(this->convert(automaton));
}

void FSMSingleInitialState::Visit(void*, const DPDA&) const {
	throw exception::AlibException("Unsupported automaton type DPDA");
}

void FSMSingleInitialState::Visit(void*, const SinglePopDPDA&) const {
	throw exception::AlibException("Unsupported automaton type SinglePopDPDA");
}

void FSMSingleInitialState::Visit(void*, const InputDrivenNPDA&) const {
	throw exception::AlibException("Unsupported automaton type InputDrivenNPDA");
}

void FSMSingleInitialState::Visit(void*, const VisiblyPushdownDPDA&) const {
	throw exception::AlibException("Unsupported automaton type VisiblyPushdownDPDA");
}

void FSMSingleInitialState::Visit(void*, const VisiblyPushdownNPDA&) const {
	throw exception::AlibException("Unsupported automaton type VisiblyPushdownNPDA");
}

void FSMSingleInitialState::Visit(void*, const RealTimeHeightDeterministicDPDA&) const {
	throw exception::AlibException("Unsupported automaton type RealTimeHeightDeterministicDPDA");
}

void FSMSingleInitialState::Visit(void*, const RealTimeHeightDeterministicNPDA&) const {
	throw exception::AlibException("Unsupported automaton type RealTimeHeightDeterministicNPDA");
}

void FSMSingleInitialState::Visit(void*, const NPDA&) const {
	throw exception::AlibException("Unsupported automaton type NPDA");
}

void FSMSingleInitialState::Visit(void*, const SinglePopNPDA&) const {
	throw exception::AlibException("Unsupported automaton type SinglePopNPDA");
}

void FSMSingleInitialState::Visit(void*, const OneTapeDTM&) const {
	throw exception::AlibException("Unsupported automaton type OneTapeDTM");
}

const FSMSingleInitialState FSMSingleInitialState::FSM_SINGLE_INITIAL_STATE;

}

