/*
 * FSMTotal.cpp
 *
 *  Created on: 20. 9. 2014
 *	  Author: Tomas Pecka
 */

#include "FSMTotal.h"

#include <exception/AlibException.h>

#include <automaton/FSM/NFA.h>
#include <automaton/FSM/DFA.h>

namespace automaton {

automaton::Automaton FSMTotal::total(const Automaton& automaton) {
	automaton::Automaton* out = NULL;
	automaton.getData().Accept((void*) &out, FSMTotal::FSM_TOTAL);
	automaton::Automaton res = std::move(*out);
	delete out;
	return res;
}

automaton::NFA FSMTotal::total(const automaton::NFA& automaton) {
	if(! automaton.isDeterministic()) {
		throw exception::AlibException("Automaton must be deterministic to make its transition function total");
	}

	automaton::NFA res(automaton);
	automaton::State nullState = automaton::createUniqueState(automaton::State("q0"), automaton.getStates());
	res.addState(nullState);

	for(const auto& q : res.getStates()) {
		for(const auto& a : res.getInputAlphabet()) {
			if(res.getTransitions().find(std::make_pair(q, a)) == res.getTransitions().end()) {
				res.addTransition(q, a, nullState);
			}
		}
	}

	return res;
}

automaton::DFA FSMTotal::total(const automaton::DFA& automaton) {
	automaton::DFA res(automaton);
	automaton::State nullState = automaton::createUniqueState(automaton::State("q0"), automaton.getStates());
	res.addState(nullState);

	for(const auto& q : res.getStates()) {
		for(const auto& a : res.getInputAlphabet()) {
			if(res.getTransitions().find(std::make_pair(q, a)) == res.getTransitions().end()) {
				res.addTransition(q, a, nullState);
			}
		}
	}

	return res;
}

void FSMTotal::Visit(void*, const EpsilonNFA&) const {
	throw exception::AlibException("Unsupported automaton type EpsilonNFA");
}

void FSMTotal::Visit(void*, const MultiInitialStateNFA&) const {
	throw exception::AlibException("Unsupported automaton type MultiInitialStateNFA");
}

void FSMTotal::Visit(void* data, const NFA& automaton) const {
	automaton::Automaton*& out = *((automaton::Automaton**) data);
	out = new automaton::Automaton(this->total(automaton));
}

void FSMTotal::Visit(void* data, const DFA& automaton) const {
	automaton::Automaton*& out = *((automaton::Automaton**) data);
	out = new automaton::Automaton(this->total(automaton));
}

void FSMTotal::Visit(void*, const ExtendedNFA&) const {
	throw exception::AlibException("Unsupported automaton type ExtendedNFA");
}

void FSMTotal::Visit(void*, const CompactNFA&) const {
	throw exception::AlibException("Unsupported automaton type CompactNFA");
}

const FSMTotal FSMTotal::FSM_TOTAL;

}

