/*
 * PDAToRHPDA.h
 *
 *  Created on: 23. 3. 2014
 *	  Author: Jan Travnicek
 */

#ifndef PDA_TO_RHPDA_H_
#define PDA_TO_RHPDA_H_

#include <algorithm>
#include <deque>
#include <set>

#include "automaton/Automaton.h"
#include <automaton/common/State.h>

namespace automaton {

class PDAToRHPDA : public VisitableConstPDABase {
public:
	static automaton::RealTimeHeightDeterministicDPDA convert( const automaton::RealTimeHeightDeterministicDPDA & pda);
	static automaton::RealTimeHeightDeterministicDPDA convert( const automaton::DPDA & pda);
	static automaton::RealTimeHeightDeterministicNPDA convert( const automaton::RealTimeHeightDeterministicNPDA & pda);
	static automaton::RealTimeHeightDeterministicNPDA convert( const automaton::NPDA & pda);

	/**
	 * Computes epsilon closure of a state in epsilon nonfree automaton
	 */
	static automaton::Automaton convert( const automaton::Automaton & automaton );
private:
	void Visit(void*, const DPDA& automaton) const;
	void Visit(void*, const SinglePopDPDA& automaton) const;
	void Visit(void*, const InputDrivenNPDA& automaton) const;
	void Visit(void*, const VisiblyPushdownDPDA& automaton) const;
	void Visit(void*, const VisiblyPushdownNPDA& automaton) const;
	void Visit(void*, const RealTimeHeightDeterministicDPDA& automaton) const;
	void Visit(void*, const RealTimeHeightDeterministicNPDA& automaton) const;
	void Visit(void*, const NPDA& automaton) const;
	void Visit(void*, const SinglePopNPDA& automaton) const;

	static const PDAToRHPDA PDA_TO_RHPDA;
};

}

#endif /* PDA_TO_RHPDA_H_ */
