/*
 * UsefullStates.cpp
 *
 *  Created on: 23. 3. 2014
 *	  Author: Tomas Pecka
 */

#include "UsefullStates.h"

#include <exception/AlibException.h>
#include <automaton/FSM/ExtendedNFA.h>
#include <automaton/FSM/CompactNFA.h>
#include <automaton/FSM/EpsilonNFA.h>
#include <automaton/FSM/NFA.h>
#include <automaton/FSM/DFA.h>

#include <set>
#include <map>
#include <queue>

namespace automaton {

std::set<automaton::State> UsefullStates::usefullStates(const Automaton& automaton) {
	std::set<automaton::State> out;
	automaton.getData().Accept((void*) &out, UsefullStates::USEFULL_STATES);
	return out;
}

template<class T>
std::set<automaton::State> UsefullStates::usefullStates( const T & fsm ) {
	// 1a
	std::deque<std::set<automaton::State>> Qi;
	Qi.push_back( std::set<automaton::State>( ) );
	Qi.at( 0 ) = fsm.getFinalStates( );

	int i = 1;

	// 1bc
	while( true ) {
		Qi.push_back( Qi.at( i - 1 ) ); // copy Qi-1 into Qi
		for( const auto & p : Qi.at( i - 1 ) )
			for( const auto & t : fsm.getTransitionsToState( p ) )
				Qi.at( i ).insert( t.first.first );

		if( Qi.at( i ) == Qi.at( i - 1 ) )
			break;

		i = i + 1;
	}
	return Qi.at( i );
}

template std::set<automaton::State> UsefullStates::usefullStates( const automaton::EpsilonNFA & fsm );
template std::set<automaton::State> UsefullStates::usefullStates( const automaton::NFA & fsm );
template std::set<automaton::State> UsefullStates::usefullStates( const automaton::CompactNFA & fsm );
template std::set<automaton::State> UsefullStates::usefullStates( const automaton::ExtendedNFA & fsm );

template<>
std::set<automaton::State> UsefullStates::usefullStates( const automaton::DFA & fsm ) {
	// 1a
	std::deque<std::set<automaton::State>> Qi;
	Qi.push_back( std::set<automaton::State>( ) );
	Qi.at( 0 ) = fsm.getFinalStates( );

	int i = 1;

	// 1bc
	while( true ) {
		Qi.push_back( Qi.at( i - 1 ) ); // copy Qi-1 into Qi
		for( const auto & p : Qi.at( i - 1 ) )
			for( const auto & t : fsm.getTransitionsToState( p ) )
				Qi.at( i ).insert( t.first.first );

		if( Qi.at( i ) == Qi.at( i - 1 ) )
			break;

		i = i + 1;
	}
	return Qi.at( i );
}

void UsefullStates::Visit(void* data, const EpsilonNFA& automaton) const {
	std::set<automaton::State> & out = *((std::set<automaton::State>*) data);
	out = std::move(this->usefullStates(automaton));
}

void UsefullStates::Visit(void* data, const MultiInitialStateNFA& automaton) const {
	std::set<automaton::State> & out = *((std::set<automaton::State>*) data);
	out = std::move(this->usefullStates(automaton));
}

void UsefullStates::Visit(void* data, const NFA& automaton) const {
	std::set<automaton::State> & out = *((std::set<automaton::State>*) data);
	out = std::move(this->usefullStates(automaton));
}

void UsefullStates::Visit(void* data, const DFA& automaton) const {
	std::set<automaton::State> & out = *((std::set<automaton::State>*) data);
	out = std::move(this->usefullStates(automaton));
}

void UsefullStates::Visit(void* data, const ExtendedNFA& automaton) const {
	std::set<automaton::State> & out = *((std::set<automaton::State>*) data);
	out = std::move(this->usefullStates(automaton));
}

void UsefullStates::Visit(void* data, const CompactNFA& automaton) const {
	std::set<automaton::State> & out = *((std::set<automaton::State>*) data);
	out = std::move(this->usefullStates(automaton));
}

void UsefullStates::Visit(void*, const DPDA&) const {
	throw exception::AlibException("Unsupported automaton type DPDA");
}

void UsefullStates::Visit(void*, const SinglePopDPDA&) const {
	throw exception::AlibException("Unsupported automaton type SinglePopDPDA");
}

void UsefullStates::Visit(void*, const InputDrivenNPDA&) const {
	throw exception::AlibException("Unsupported automaton type InputDrivenNPDA");
}

void UsefullStates::Visit(void*, const VisiblyPushdownDPDA&) const {
	throw exception::AlibException("Unsupported automaton type VisiblyPushdownDPDA");
}

void UsefullStates::Visit(void*, const VisiblyPushdownNPDA&) const {
	throw exception::AlibException("Unsupported automaton type VisiblyPushdownNPDA");
}

void UsefullStates::Visit(void*, const RealTimeHeightDeterministicDPDA&) const {
	throw exception::AlibException("Unsupported automaton type RealTimeHeightDeterministicDPDA");
}

void UsefullStates::Visit(void*, const RealTimeHeightDeterministicNPDA&) const {
	throw exception::AlibException("Unsupported automaton type RealTimeHeightDeterministicNPDA");
}

void UsefullStates::Visit(void*, const NPDA&) const {
	throw exception::AlibException("Unsupported automaton type NPDA");
}

void UsefullStates::Visit(void*, const SinglePopNPDA&) const {
	throw exception::AlibException("Unsupported automaton type SinglePopNPDA");
}

void UsefullStates::Visit(void*, const OneTapeDTM&) const {
	throw exception::AlibException("Unsupported automaton type OneTapeDTM");
}

const UsefullStates UsefullStates::USEFULL_STATES;

}

