#ifndef __FATORRGCONVERTER_H__
#define __FATORRGCONVERTER_H__

#include <grammar/Regular/RightRG.h>
#include <automaton/FSM/NFA.h>
#include <automaton/FSM/DFA.h>

#include <grammar/Grammar.h>
#include <automaton/Automaton.h>

namespace conversions
{

namespace fa2rg
{

/**
 * Finite automaton to right regular grammar converter.
 * Source: Melichar 2.104
 */
class FAtoRRGConverter : public automaton::VisitableConstFSMBase
{
public:
	/**
	 * Performs conversion.
	 * @return left regular grammar equivalent to source automaton.
	 */
	static grammar::Grammar convert(const automaton::Automaton& automaton);

	static grammar::RightRG convert(const automaton::NFA& automaton);
	static grammar::RightRG convert(const automaton::DFA& automaton);

private:
	void Visit(void*, const automaton::EpsilonNFA& automaton) const;
	void Visit(void*, const automaton::MultiInitialStateNFA& automaton) const;
	void Visit(void*, const automaton::NFA& automaton) const;
	void Visit(void*, const automaton::DFA& automaton) const;
	void Visit(void*, const automaton::ExtendedNFA& automaton) const;
	void Visit(void*, const automaton::CompactNFA& automaton) const;

	static const FAtoRRGConverter FA_TO_RRG_CONVERTER;
};

} /* namespace fa2rg */

} /* namespace conversions */

#endif /* __FATORRGCONVERTER_H__ */
