/*
 * GlushkovNFA.h
 *
 *  Created on: 11. 1. 2014
 *      Author: Tomas Pecka
 */

#ifndef FA2RE_GLUSHKOVNFA_H_
#define FA2RE_GLUSHKOVNFA_H_

#include <map>

#include <automaton/FSM/NFA.h>
#include <regexp/RegExp.h>
#include <regexp/formal/FormalRegExp.h>
#include <regexp/unbounded/UnboundedRegExp.h>

namespace conversions {

namespace re2fa {

/**
 * Converts regular expression to finite automaton using Glushkov's NFA construction algorithm.
 * Source: Melichar 2.107
 */
class GlushkovNFA : public regexp::VisitableRegExpBase::const_visitor_type
{
public:
    /**
     * Performs conversion.
     * @param re Original regular expression.
     * @return NFA equivalent to original regular expression.
     */
	static automaton::NFA convert(const regexp::RegExp& re);

    static automaton::NFA convert(const regexp::UnboundedRegExp& re);
    static automaton::NFA convert(const regexp::FormalRegExp& re);

private:
    void Visit(void*, const regexp::FormalRegExp& regexp) const;
    void Visit(void*, const regexp::UnboundedRegExp& regexp) const;

	static const GlushkovNFA GLUSHKOV_NFA;
};

} /* namespace re2fa */

} /* namespace conversions */

#endif /* FA2RE_GLUSHKOV_H_ */
