/*
 * RGtoFA.h
 *
 * Created on: 1. 11. 2013
 * Author: Tomas Pecka
 */

#ifndef __RGTOFA_H__
#define __RGTOFA_H__

#include <grammar/Grammar.h>
#include <grammar/Regular/LeftRG.h>
#include <grammar/Regular/RightRG.h>

#include <automaton/Automaton.h>
#include <automaton/FSM/NFA.h>

namespace conversions
{

namespace rg2fa
{

/**
 * Converts regular grammar to nondeterministic finite automaton.
 * Sources: Melichar 2.98 (RightRG -> NFA) and 2.102 (LeftRG -> NFA).
 */
class RGtoFA : public grammar::VisitableConstRGBase
{
public:
    /**
     * Performs conversion.
     * @param grammar Regular grammar to convert.
     * @return FSM equivalent to source grammar.
     */
    static automaton::Automaton convert(const grammar::Grammar& grammar);

    static automaton::NFA convert(const grammar::LeftRG& grammar);
    static automaton::NFA convert(const grammar::RightRG& grammar);

private:
    void Visit(void*, const grammar::RightRG& grammar) const;
    void Visit(void*, const grammar::LeftRG& grammar) const;
    void Visit(void*, const grammar::RightLG& grammar) const;
    void Visit(void*, const grammar::LeftLG& grammar) const;

    static const RGtoFA RG_TO_FA_CONVERSION;
};

} /* namespace rg2fa */

} /* namespace conversions */

#endif /* __RGTOFA_H__ */
