/*
 * LeftToRightRegularGrammar.cpp
 *
 *  Created on: 8. 3. 2014
 *      Author: Tomas Pecka
 */

#include "LeftToRightRegularGrammar.h"

#include <exception/AlibException.h>

namespace conversions {

namespace rg2rg {

grammar::RightRG LeftToRightRegularGrammar::convert(const grammar::Grammar& grammar)
{
	grammar::RightRG* out = NULL;
	grammar.getData().Accept((void*) &out, LeftToRightRegularGrammar::LEFT_TO_RIGHT_REGULAR_GRAMMAR);
	grammar::RightRG res = std::move(*out);
	delete out;
	return res;
}

grammar::RightRG LeftToRightRegularGrammar::convert(const grammar::LeftRG& grammar)
{
	// 1.
	alphabet::Symbol s = alphabet::createUniqueSymbol( grammar.getInitialSymbol( ), grammar.getNonterminalAlphabet(), grammar.getTerminalAlphabet() );

	grammar::RightRG rrg( s );

	for(const auto & nonterminalSymbol : grammar.getNonterminalAlphabet() ) {
		rrg.addNonterminalSymbol( nonterminalSymbol );
	}

	rrg.setTerminalAlphabet( grammar.getTerminalAlphabet( ) );
	rrg.setGeneratesEpsilon( grammar.getGeneratesEpsilon( ) );

	// 2
	for( const auto & rule : grammar.getRules( ) ) {
		const alphabet::Symbol& lhs = rule.first;

		for(const auto & ruleRHS : rule.second ) {
		        if( ruleRHS.is<std::pair<alphabet::Symbol, alphabet::Symbol>>( ) ) {
				const std::pair<alphabet::Symbol, alphabet::Symbol>& rhs = ruleRHS.get<std::pair<alphabet::Symbol, alphabet::Symbol>>();

				alphabet::Symbol leftSide = ( rhs.first );
				std::pair<alphabet::Symbol, alphabet::Symbol> rightSide = std::make_pair( rhs.second, lhs );
				rrg.addRule( leftSide, rightSide );

				if( lhs == grammar.getInitialSymbol( ) ) {
					alphabet::Symbol leftSide = rhs.first;
					alphabet::Symbol rightSide = rhs.second;
					rrg.addRule( leftSide, rightSide );
				}
			} else {
				const alphabet::Symbol& rhs = ruleRHS.get<alphabet::Symbol>();

				alphabet::Symbol leftSide = rrg.getInitialSymbol( );
				std::pair<alphabet::Symbol, alphabet::Symbol> rightSide = std::make_pair( rhs, lhs );
				rrg.addRule( leftSide, rightSide );

				if( lhs == grammar.getInitialSymbol( ) ) {
					alphabet::Symbol leftSide = rrg.getInitialSymbol( );
					alphabet::Symbol rightSide = rhs;
					rrg.addRule( leftSide, rightSide );
				}
			}
		}
	}
	return rrg;
}

void LeftToRightRegularGrammar::Visit(void*, const grammar::RightRG&) const
{
	throw exception::AlibException("Unsupported grammar type RightRG");
}

void LeftToRightRegularGrammar::Visit(void* userData, const grammar::LeftRG& grammar) const
{
	grammar::RightRG* & out = *((grammar::RightRG**) userData);
	out = new grammar::RightRG(this->convert(grammar));
}

void LeftToRightRegularGrammar::Visit(void*, const grammar::RightLG&) const
{
	throw exception::AlibException("Unsupported grammar type RightLG");
}

void LeftToRightRegularGrammar::Visit(void*, const grammar::LeftLG&) const
{
	throw exception::AlibException("Unsupported grammar type LeftLG");
}

const LeftToRightRegularGrammar LeftToRightRegularGrammar::LEFT_TO_RIGHT_REGULAR_GRAMMAR;

} /* namespace rg2rg */

} /* namespace conversions */
