/*
 * LeftToRightRegularGrammar.h
 *
 *  Created on: 8. 3. 2014
 *      Author: Tomas Pecka
 */

#ifndef LEFT_TORIGHT_REGULAR_GRAMMAR_H_
#define LEFT_TORIGHT_REGULAR_GRAMMAR_H_

#include <grammar/Grammar.h>
#include <grammar/Regular/LeftRG.h>
#include <grammar/Regular/RightRG.h>

namespace conversions{

namespace rg2rg
{

/**
 * Converts left regular grammar to right regular grammar.
 */
class LeftToRightRegularGrammar : public grammar::VisitableConstRGBase
{
public:
   /**
     * Performs conversion.
     * @param grammar Left regular grammar to convert
     * @return right regular grammar which is equivalent to source left regular grammar.
     */
    static grammar::RightRG convert(const grammar::Grammar& grammar);
    static grammar::RightRG convert(const grammar::LeftRG& grammar);

private:
    void Visit(void*, const grammar::RightRG& grammar) const;
    void Visit(void*, const grammar::LeftRG& grammar) const;
    void Visit(void*, const grammar::RightLG& grammar) const;
    void Visit(void*, const grammar::LeftLG& grammar) const;

    static const LeftToRightRegularGrammar LEFT_TO_RIGHT_REGULAR_GRAMMAR;
};

} /* namespace rg2rg */

} /* namespace conversions */

#endif /* LEFT_TO_RIGHT_REGULAR_GRAMMAR_H_ */
