/*
 * RightToLeftRegularGrammar.cpp
 *
 *  Created on: 8. 3. 2014
 *      Author: Tomas Pecka
 */

#include "RightToLeftRegularGrammar.h"

#include <exception/AlibException.h>

namespace conversions
{

namespace rg2rg
{

grammar::LeftRG RightToLeftRegularGrammar::convert(const grammar::Grammar& grammar)
{
	grammar::LeftRG* out = NULL;
	grammar.getData().Accept((void*) &out, RightToLeftRegularGrammar::RIGHT_TO_LEFT_REGULAR_GRAMMAR);
	grammar::LeftRG res = std::move(*out);
	delete out;
	return res;
}

grammar::LeftRG RightToLeftRegularGrammar::convert(const grammar::RightRG& grammar)
{
	// 1.
	alphabet::Symbol s = alphabet::createUniqueSymbol( grammar.getInitialSymbol( ), grammar.getNonterminalAlphabet(), grammar.getTerminalAlphabet() );

	grammar::LeftRG lrg(s);

	for(const auto & nonterminalSymbol : grammar.getNonterminalAlphabet()) {
		lrg.addNonterminalSymbol( nonterminalSymbol );
	}

	lrg.setTerminalAlphabet( grammar.getTerminalAlphabet( ) );
	lrg.setGeneratesEpsilon( grammar.getGeneratesEpsilon( ) );

	// 2.
	for( const auto & rule : grammar.getRules( ) ) {
		const alphabet::Symbol& lhs = rule.first;

		for(const auto & ruleRHS : rule.second ) {
		        if( ruleRHS.is<std::pair<alphabet::Symbol, alphabet::Symbol>>( ) ) {
				const std::pair<alphabet::Symbol, alphabet::Symbol>& rhs = ruleRHS.get<std::pair<alphabet::Symbol, alphabet::Symbol>>();

				alphabet::Symbol leftSide = rhs.second;
				std::pair<alphabet::Symbol, alphabet::Symbol> rightSide = std::make_pair( lhs, rhs.first );
				lrg.addRule( leftSide, rightSide );

				if( lhs == grammar.getInitialSymbol( ) ) {
					alphabet::Symbol leftSide = rhs.second;
					alphabet::Symbol rightSide = rhs.first;
					lrg.addRule( leftSide, rightSide );
				}
			} else {
				const alphabet::Symbol& rhs = ruleRHS.get<alphabet::Symbol>();

				alphabet::Symbol leftSide = lrg.getInitialSymbol( );
				std::pair<alphabet::Symbol, alphabet::Symbol> rightSide = std::make_pair ( lhs, rhs );
				lrg.addRule( leftSide, rightSide );

				if( lhs == grammar.getInitialSymbol( ) ) {
					alphabet::Symbol leftSide = lrg.getInitialSymbol( );
					alphabet::Symbol rightSide = rhs;
					lrg.addRule( leftSide, rightSide );
				}
			}
		}
	}
	return lrg;
}

void RightToLeftRegularGrammar::Visit(void* userData, const grammar::RightRG& grammar) const
{
	grammar::LeftRG* & out = *((grammar::LeftRG**) userData);
	out = new grammar::LeftRG(this->convert(grammar));
}

void RightToLeftRegularGrammar::Visit(void*, const grammar::LeftRG&) const
{
	throw exception::AlibException("Unsupported grammar type RightRG");
}

void RightToLeftRegularGrammar::Visit(void*, const grammar::RightLG&) const
{
	throw exception::AlibException("Unsupported grammar type RightLG");
}

void RightToLeftRegularGrammar::Visit(void*, const grammar::LeftLG&) const
{
	throw exception::AlibException("Unsupported grammar type LeftLG");
}

const RightToLeftRegularGrammar RightToLeftRegularGrammar::RIGHT_TO_LEFT_REGULAR_GRAMMAR;


} /* namespace rg2rg */

} /* namespace conversions */
