/*
 * RegExpAlternate.cpp
 *
 *  Created on: 17. 10. 2014
 *	  Author: Jan Travnicek
 */

#include "RegExpAlternate.h"

namespace regexp
{

regexp::RegExp RegExpAlternate::alternate(const regexp::RegExp& first, const regexp::RegExp& second)
{
	RegExpBase* out;
	Accept((void*) &out, first.getData(), second.getData(), RegExpAlternate::REG_EXP_ALTERNATE);
	regexp::RegExp res(*out);
	delete out;
	return res;
}

regexp::FormalRegExp RegExpAlternate::alternate(const regexp::FormalRegExp& first, const regexp::FormalRegExp& second)
{
	return regexp::FormalRegExp(regexp::FormalRegExpAlternation(first.getRegExp(), second.getRegExp()));
}

regexp::UnboundedRegExp RegExpAlternate::alternate(const regexp::UnboundedRegExp& first, const regexp::UnboundedRegExp& second)
{
	regexp::UnboundedRegExpAlternation con;
	con.appendElement(first.getRegExp());
	con.appendElement(second.getRegExp());
	return regexp::UnboundedRegExp(con);
}

void RegExpAlternate::Visit(void* data, const regexp::FormalRegExp& first, const regexp::FormalRegExp& second) const
{
	RegExpBase* &ret = *(RegExpBase**) data;
	ret = std::move(RegExpAlternate::alternate(first, second)).plunder();
}

void RegExpAlternate::Visit(void* data, const regexp::UnboundedRegExp& first, const regexp::UnboundedRegExp& second) const
{
	RegExpBase* &ret = *(RegExpBase**) data;
	ret = std::move(RegExpAlternate::alternate(first, second)).plunder();
}

const RegExpAlternate RegExpAlternate::REG_EXP_ALTERNATE;

} /* namespace regexp */
