/*
 * RegExpConcatenate.cpp
 *
 *  Created on: 17. 10. 2014
 *	  Author: Jan Travnicek
 */

#include "RegExpConcatenate.h"

namespace regexp
{

regexp::RegExp RegExpConcatenate::concatenate(const regexp::RegExp& first, const regexp::RegExp& second)
{
	RegExpBase* out;
	Accept((void*) &out, first.getData(), second.getData(), RegExpConcatenate::REG_EXP_CONCATENATE);
	regexp::RegExp res(*out);
	delete out;
	return res;
}

regexp::FormalRegExp RegExpConcatenate::concatenate(const regexp::FormalRegExp& first, const regexp::FormalRegExp& second)
{
	return regexp::FormalRegExp(regexp::FormalRegExpConcatenation(first.getRegExp(), second.getRegExp()));
}

regexp::UnboundedRegExp RegExpConcatenate::concatenate(const regexp::UnboundedRegExp& first, const regexp::UnboundedRegExp& second)
{
	regexp::UnboundedRegExpConcatenation con;
	con.appendElement(first.getRegExp());
	con.appendElement(second.getRegExp());
	return regexp::UnboundedRegExp(con);
}

void RegExpConcatenate::Visit(void* data, const regexp::FormalRegExp& first, const regexp::FormalRegExp& second) const
{
	RegExpBase* &ret = *(RegExpBase**) data;
	ret = std::move(RegExpConcatenate::concatenate(first, second)).plunder();
}

void RegExpConcatenate::Visit(void* data, const regexp::UnboundedRegExp& first, const regexp::UnboundedRegExp& second) const
{
	RegExpBase* &ret = *(RegExpBase**) data;
	ret = std::move(RegExpConcatenate::concatenate(first, second)).plunder();
}

const RegExpConcatenate RegExpConcatenate::REG_EXP_CONCATENATE;

} /* namespace regexp */
