/*
 * RegExpDerivation.h
 *
 *  Created on: 19. 1. 2014
 *      Author: Tomas Pecka
 */

#ifndef REGEXPDERIVATION_H_
#define REGEXPDERIVATION_H_

#include <regexp/RegExp.h>
#include <regexp/formal/FormalRegExpElements.h>
#include <regexp/unbounded/UnboundedRegExpElements.h>

#include <string/LinearString.h>

namespace regexp
{

/**
 * Calculates derivation of regular expression.
 *
 * Sources:
 *  - Melichar, definition 2.91 in chapter 2.4.3
 *  - Brzozowski, J. A. - Derivatives of regular expressions (1964)
 */
class RegExpDerivation : public regexp::VisitableRegExpBase::const_visitor_type, regexp::FormalRegExpElement::const_visitor_type, regexp::UnboundedRegExpElement::const_visitor_type
{
public:
    /**
     * @param regexp RegExp to derivate
     * @param string String to derivate given RegExp over
     * @return derivative of regexp
     */
    static regexp::RegExp derivation(const regexp::RegExp& regexp, const string::LinearString& string);

    static regexp::FormalRegExp derivation(const regexp::FormalRegExp& regexp, const string::LinearString& string);
    static regexp::UnboundedRegExp derivation(const regexp::UnboundedRegExp& regexp, const string::LinearString& string);

private:
    void Visit(void*, const regexp::FormalRegExp& empty) const;
    void Visit(void*, const regexp::UnboundedRegExp& empty) const;

    void Visit(void*, const regexp::FormalRegExpAlternation& alternation) const;
    void Visit(void*, const regexp::FormalRegExpConcatenation& concatenation) const;
    void Visit(void*, const regexp::FormalRegExpIteration& iteration) const;
    void Visit(void*, const regexp::FormalRegExpSymbol& symbol) const;
    void Visit(void*, const regexp::FormalRegExpEpsilon& epsilon) const;
    void Visit(void*, const regexp::FormalRegExpEmpty& empty) const;

    void Visit(void*, const regexp::UnboundedRegExpAlternation& alternation) const;
    void Visit(void*, const regexp::UnboundedRegExpConcatenation& concatenation) const;
    void Visit(void*, const regexp::UnboundedRegExpIteration& iteration) const;
    void Visit(void*, const regexp::UnboundedRegExpSymbol& symbol) const;
    void Visit(void*, const regexp::UnboundedRegExpEpsilon& epsilon) const;
    void Visit(void*, const regexp::UnboundedRegExpEmpty& empty) const;

    static const RegExpDerivation REGEXP_DERIVATION;
};

} /* namespace regexp */

#endif /* REGEXPDERIVATION_H_ */
