/*
 * RegExpEpsilon.cpp
 *
 *  Created on: 19. 1. 2014
 *      Author: Tomas Pecka
 */

#include "RegExpEpsilon.h"

namespace regexp
{

bool RegExpEpsilon::languageContainsEpsilon(const regexp::RegExp& regexp)
{
    bool out;
    regexp.getData().Accept((void*) &out, RegExpEpsilon::REG_EXP_EPSILON);
    return out;
}

bool RegExpEpsilon::languageContainsEpsilon(const regexp::FormalRegExp& regexp)
{
	bool out;
	regexp.getRegExp().Accept((void*) &out, RegExpEpsilon::REG_EXP_EPSILON);
	return out;
}

bool RegExpEpsilon::languageContainsEpsilon(const regexp::UnboundedRegExp& regexp)
{
	bool out;
	regexp.getRegExp().Accept((void*) &out, RegExpEpsilon::REG_EXP_EPSILON);
	return out;
}


bool RegExpEpsilon::languageContainsEpsilon(const regexp::FormalRegExpElement& element)
{
	bool out;
	element.Accept((void*) &out, RegExpEpsilon::REG_EXP_EPSILON);
	return out;
}

bool RegExpEpsilon::languageContainsEpsilon(const regexp::UnboundedRegExpElement& element)
{
	bool out;
	element.Accept((void*) &out, RegExpEpsilon::REG_EXP_EPSILON);
	return out;
}

// ---------------------------------------------------------------------------

void RegExpEpsilon::Visit(void* data, const regexp::UnboundedRegExpAlternation& alternation) const
{
	bool &ret = *(bool*) data;

	for(const auto& element : alternation.getElements())
	{
		static_cast<const regexp::UnboundedRegExpElement&>(*element).Accept(data, RegExpEpsilon::REG_EXP_EPSILON);
		if(ret)
		{
			ret = true;
			return;
		}
	}
	
	ret = false;
}

void RegExpEpsilon::Visit(void* data, const regexp::UnboundedRegExpConcatenation& concatenation) const
{
	bool &ret = *(bool*) data;

	for(const auto& element : concatenation.getElements())
	{
		static_cast<const regexp::UnboundedRegExpElement&>(*element).Accept(data, RegExpEpsilon::REG_EXP_EPSILON);
		if(!ret)
		{
			ret = false;
			return;
		}
	}
	
	ret = true;
}

void RegExpEpsilon::Visit(void* data, const regexp::UnboundedRegExpIteration&) const
{
	bool &ret = *(bool*) data;
	ret = true;
}

void RegExpEpsilon::Visit(void* data, const regexp::UnboundedRegExpSymbol&) const
{
	bool &ret = *(bool*) data;
	ret = false;
}

void RegExpEpsilon::Visit(void* data, const regexp::UnboundedRegExpEpsilon&) const
{
	bool &ret = *(bool*) data;
	ret = true;
}

void RegExpEpsilon::Visit(void* data, const regexp::UnboundedRegExpEmpty&) const
{
	bool &ret = *(bool*) data;
	ret = false;
}

// ----------------------------------------------------------------------------

void RegExpEpsilon::Visit(void* data, const regexp::FormalRegExpAlternation& alternation) const
{
	bool &ret = *(bool*) data;

	static_cast<const regexp::FormalRegExpElement&>(alternation.getLeftElement()).Accept(data, RegExpEpsilon::REG_EXP_EPSILON);
	bool retLeft = ret;

	static_cast<const regexp::FormalRegExpElement&>(alternation.getRightElement()).Accept(data, RegExpEpsilon::REG_EXP_EPSILON);
	bool retRight = ret;

	ret = retLeft || retRight;
}

void RegExpEpsilon::Visit(void* data, const regexp::FormalRegExpConcatenation& concatenation) const
{
	bool &ret = *(bool*) data;

	static_cast<const regexp::FormalRegExpElement&>(concatenation.getLeftElement()).Accept(data, RegExpEpsilon::REG_EXP_EPSILON);
	bool retLeft = ret;

	static_cast<const regexp::FormalRegExpElement&>(concatenation.getRightElement()).Accept(data, RegExpEpsilon::REG_EXP_EPSILON);
	bool retRight = ret;

	ret = retLeft && retRight;
}

void RegExpEpsilon::Visit(void* data, const regexp::FormalRegExpIteration&) const
{
	bool &ret = *(bool*) data;
	ret = true;
}

void RegExpEpsilon::Visit(void* data, const regexp::FormalRegExpSymbol&) const
{
	bool &ret = *(bool*) data;
	ret = false;
}

void RegExpEpsilon::Visit(void* data, const regexp::FormalRegExpEmpty&) const
{
	bool &ret = *(bool*) data;
	ret = false;
}

void RegExpEpsilon::Visit(void* data, const regexp::FormalRegExpEpsilon&) const
{
	bool &ret = *(bool*) data;
	ret = true;
}

// ---------------------------------------------------------------------------

void RegExpEpsilon::Visit(void* data, const regexp::FormalRegExp& regexp) const
{
    bool &ret = *(bool*) data;
    ret = RegExpEpsilon::REG_EXP_EPSILON.languageContainsEpsilon(regexp);
}

void RegExpEpsilon::Visit(void* data, const regexp::UnboundedRegExp& regexp) const
{
    bool &ret = *(bool*) data;
    ret = RegExpEpsilon::REG_EXP_EPSILON.languageContainsEpsilon(regexp);
}

const RegExpEpsilon RegExpEpsilon::REG_EXP_EPSILON;

} /* namespace regexp */
