/*
 * RegExpEpsilon.h
 *
 *  Created on: 19. 1. 2014
 *      Author: Jan Travnicek
 */

#ifndef REG_EXP_EPSILON_H_
#define REG_EXP_EPSILON_H_

#include <regexp/RegExp.h>
#include <regexp/formal/FormalRegExpElements.h>
#include <regexp/unbounded/UnboundedRegExpElements.h>

namespace regexp
{

/**
 * Checks, whether regexp (or its subtree) describes epsilon (empty string).
 * TODO: Better API?
 *
 */
class RegExpEpsilon : public regexp::VisitableRegExpBase::const_visitor_type, regexp::FormalRegExpElement::const_visitor_type, regexp::UnboundedRegExpElement::const_visitor_type
{
public:
    static bool languageContainsEpsilon(const regexp::RegExp& regexp);

    static bool languageContainsEpsilon(const regexp::FormalRegExp& regexp);
    static bool languageContainsEpsilon(const regexp::UnboundedRegExp& regexp);

	static bool languageContainsEpsilon(const regexp::UnboundedRegExpElement& element);
	static bool languageContainsEpsilon(const regexp::FormalRegExpElement& element);

private:
	void Visit(void* data, const regexp::UnboundedRegExp& regexp) const;
	void Visit(void* data, const regexp::FormalRegExp& regexp) const;

	void Visit(void* data, const regexp::UnboundedRegExpAlternation& alternation) const;
	void Visit(void* data, const regexp::UnboundedRegExpConcatenation& concatenation) const;
	void Visit(void* data, const regexp::UnboundedRegExpIteration& iteration) const;
	void Visit(void* data, const regexp::UnboundedRegExpSymbol& symbol) const;
	void Visit(void* data, const regexp::UnboundedRegExpEmpty& empty) const;
	void Visit(void* data, const regexp::UnboundedRegExpEpsilon& epsilon) const;

	void Visit(void* data, const regexp::FormalRegExpAlternation& alternation) const;
	void Visit(void* data, const regexp::FormalRegExpConcatenation& concatenation) const;
	void Visit(void* data, const regexp::FormalRegExpIteration& iteration) const;
	void Visit(void* data, const regexp::FormalRegExpSymbol& symbol) const;
	void Visit(void* data, const regexp::FormalRegExpEmpty& empty) const;
	void Visit(void* data, const regexp::FormalRegExpEpsilon& epsilon) const;


    static const RegExpEpsilon REG_EXP_EPSILON;
};

} /* namespace regexp */

#endif /* REG_EXP_EPSILON_H_ */
