/*
 * RegExpOptimize.cpp
 *
 *  Created on: 20. 1. 2014
 *	  Author: Tomas Pecka
 */

#include "RegExpOptimize.h"

#include "RegExpEpsilon.h"

#include <cassert>
#include <iostream>
#include <iostream>

namespace regexp {

regexp::RegExp RegExpOptimize::optimize(const regexp::RegExp& regexp)
{
    regexp::RegExp * out = NULL;
    regexp.getData().Accept((void*) &out, RegExpOptimize::REG_EXP_OPTIMIZE);
    regexp::RegExp res( std::move( * out ) );
    delete out;
    return res;
}

void RegExpOptimize::Visit(void* userData, const regexp::FormalRegExp& regexp) const
{
    regexp::RegExp * &ret = *(regexp::RegExp **) userData;

    ret = new regexp::RegExp( optimize( regexp ) );
}

void RegExpOptimize::Visit(void* userData, const regexp::UnboundedRegExp& regexp) const
{
    regexp::RegExp * &ret = *(regexp::RegExp **) userData;

    ret = new regexp::RegExp( optimize( regexp ) );
}

FormalRegExp RegExpOptimize::optimize( FormalRegExp const & regexp )
{
	FormalRegExpElement* optimized = RegExpOptimize::REG_EXP_OPTIMIZE.optimize( & regexp.getRegExp( ) );

	FormalRegExp ret( std::move( * optimized ) );

	delete optimized;

	return ret;
}

void RegExpOptimize::optimize( FormalRegExpElement & element )
{
	FormalRegExpElement* optimized = RegExpOptimize::REG_EXP_OPTIMIZE.optimize( & element );

	FormalRegExpAlternation * alternation = dynamic_cast<FormalRegExpAlternation *>( & element );
	if( alternation ) {
		FormalRegExpAlternation * alternationOptimized = dynamic_cast<FormalRegExpAlternation *>( optimized );
		if( alternationOptimized ) {
			* alternation = std::move( * alternationOptimized );
			delete alternationOptimized;
		} else {
			* alternation = FormalRegExpAlternation { std::move( * optimized ), FormalRegExpEmpty { } };
			delete optimized;
		}
		return;
	}

	FormalRegExpConcatenation * concatenation = dynamic_cast<FormalRegExpConcatenation *>( & element );
	if( concatenation ) {
		FormalRegExpConcatenation * concatenationOptimized = dynamic_cast<FormalRegExpConcatenation *>( optimized );
		if( concatenationOptimized ) {
			* concatenation = std::move( * concatenationOptimized );
			delete concatenationOptimized;
		} else {
			* concatenation = FormalRegExpConcatenation { std::move( * optimized ), FormalRegExpEpsilon { } };
			delete optimized;
		}
		return;
	}

	FormalRegExpIteration * iteration = dynamic_cast<FormalRegExpIteration *>( & element );
	if( iteration ) {
		FormalRegExpIteration * iterationOptimized = dynamic_cast<FormalRegExpIteration *>( optimized );
		if( iterationOptimized ) {
			* iteration = std::move( * iterationOptimized );
			delete iterationOptimized;
		} else {
			* iteration = FormalRegExpIteration { std::move( * optimized ) };
			delete optimized;
		}
		return;
	}

	// Nothing to optimize original element was FormalRegExpSymbol, FormalRegExpEpsilon, or FormalRegExpEmpty
	return;
}

UnboundedRegExp RegExpOptimize::optimize( UnboundedRegExp const & regexp )
{
	UnboundedRegExpElement* optimized = RegExpOptimize::REG_EXP_OPTIMIZE.optimize( & regexp.getRegExp( ) );

	UnboundedRegExp ret( std::move( * optimized ) );

	delete optimized;

	return ret;
}

void RegExpOptimize::optimize( UnboundedRegExpElement & element ) {
	UnboundedRegExpElement* optimized = RegExpOptimize::REG_EXP_OPTIMIZE.optimize( & element );

	UnboundedRegExpAlternation * alternation = dynamic_cast<UnboundedRegExpAlternation *>( & element );
	if( alternation ) {
		UnboundedRegExpAlternation * alternationOptimized = dynamic_cast<UnboundedRegExpAlternation *>( optimized );
		if( alternationOptimized ) {
			* alternation = std::move( * alternationOptimized );
			delete alternationOptimized;
		} else {
			* alternation = UnboundedRegExpAlternation { };
			alternation->appendElement( std::move( * optimized ) );
			delete optimized;
		}
		return;
	}

	UnboundedRegExpConcatenation * concatenation = dynamic_cast<UnboundedRegExpConcatenation *>( & element );
	if( concatenation ) {
		UnboundedRegExpConcatenation * concatenationOptimized = dynamic_cast<UnboundedRegExpConcatenation *>( optimized );
		if( concatenationOptimized ) {
			* concatenation = std::move( * concatenationOptimized );
			delete concatenationOptimized;
		} else {
			* concatenation = UnboundedRegExpConcatenation { };
			concatenation->appendElement( std::move( * optimized ) );
			delete optimized;
		}
		return;
	}

	UnboundedRegExpIteration * iteration = dynamic_cast<UnboundedRegExpIteration *>( & element );
	if( iteration ) {
		UnboundedRegExpIteration * iterationOptimized = dynamic_cast<UnboundedRegExpIteration *>( optimized );
		if( iterationOptimized ) {
			* iteration = std::move( * iterationOptimized );
			delete iterationOptimized;
		} else {
			* iteration = UnboundedRegExpIteration { std::move( * optimized ) };
			delete optimized;
		}
		return;
	}

	// Nothing to optimize original element was UnboundedRegExpSymbol, UnboundedRegExpEpsilon, or UnboundedRegExpEmpty
	return;
}

const RegExpOptimize RegExpOptimize::REG_EXP_OPTIMIZE;

#include "RegExpOptimizeUnboundedPart.cxx"
#include "RegExpOptimizeFormalPart.cxx"

}
