#include <list>
#include "determinizeTest.h"

#include "determinize/nfa/NFADeterminizer.h"
#include "determinize/idpda/IDPDADeterminizer.h"

#include "factory/DataFactory.hpp"

#define CPPUNIT_IMPLY(x, y) CPPUNIT_ASSERT(!(x) || (y))

CPPUNIT_TEST_SUITE_REGISTRATION( determinizeTest );

void determinizeTest::setUp() {
}

void determinizeTest::tearDown() {
}

void determinizeTest::testDeterminizeNFA() {
  automaton::NFA automaton(automaton::State(1));

  automaton.addState(automaton::State(1));
  automaton.addState(automaton::State(2));
  automaton.addState(automaton::State(3));
  automaton.addInputSymbol(alphabet::symbolFrom("a"));
  automaton.addInputSymbol(alphabet::symbolFrom("b"));

  automaton.addTransition(automaton::State(1), alphabet::symbolFrom("a"), automaton::State(2));
  automaton.addTransition(automaton::State(2), alphabet::symbolFrom("b"), automaton::State(1));

  automaton.addFinalState(automaton::State(3));

  automaton::DFA determinized = determinize::NFADeterminizer::determinize(automaton);

  CPPUNIT_ASSERT(determinized.getStates().size() == 3);

}

void determinizeTest::testDeterminizeIDPDA() {
  automaton::InputDrivenNPDA automaton(alphabet::symbolFrom('S'));

  automaton.addInputSymbol(alphabet::symbolFrom("a"));
  automaton.addInputSymbol(alphabet::symbolFrom("b"));

  automaton.setPushdownStoreOperation(alphabet::symbolFrom("a"), std::vector<alphabet::Symbol>{}, std::vector<alphabet::Symbol> {});
  automaton.setPushdownStoreOperation(alphabet::symbolFrom("b"), std::vector<alphabet::Symbol>{}, std::vector<alphabet::Symbol> {});

  automaton.addState(automaton::State(1));
  automaton.addState(automaton::State(2));
  automaton.addState(automaton::State(3));
  automaton.addTransition(automaton::State(1), alphabet::symbolFrom("a"), automaton::State(2));
  automaton.addTransition(automaton::State(2), alphabet::symbolFrom("b"), automaton::State(1));

  automaton.addInitialState(automaton::State(1));
  automaton.addFinalState(automaton::State(3));

  automaton::DPDA determinized = determinize::IDPDADeterminizer::determinize(automaton);

  CPPUNIT_ASSERT(determinized.getStates().size() == 3);
}

void determinizeTest::testDeterminizeVPA() {
  automaton::VisiblyPushdownNPDA automaton(alphabet::Symbol(alphabet::BottomOfTheStackSymbol {}));

  automaton.addCallInputSymbol(alphabet::symbolFrom('a'));
  automaton.addReturnInputSymbol(alphabet::symbolFrom('^'));

  automaton.addStackSymbol(alphabet::symbolFrom('A'));
  automaton.addStackSymbol(alphabet::symbolFrom('B'));
  automaton.addStackSymbol(alphabet::symbolFrom('C'));
  automaton.addStackSymbol(alphabet::symbolFrom('D'));
  automaton.addStackSymbol(alphabet::symbolFrom('E'));
  automaton.addStackSymbol(alphabet::symbolFrom('F'));
  automaton.addStackSymbol(alphabet::symbolFrom('T'));

  automaton.addState(automaton::State(0));
  automaton.addState(automaton::State(1));
  automaton.addState(automaton::State(2));
  automaton.addState(automaton::State(3));
  automaton.addState(automaton::State(4));
  automaton.addState(automaton::State(5));
  automaton.addState(automaton::State(6));


  automaton.addCallTransition(automaton::State(0), alphabet::symbolFrom('a'), automaton::State(0), alphabet::symbolFrom('A'));
  automaton.addReturnTransition(automaton::State(0), alphabet::symbolFrom('^'), alphabet::symbolFrom('A'), automaton::State(0));

  automaton.addCallTransition(automaton::State(0), alphabet::symbolFrom('a'), automaton::State(1), alphabet::symbolFrom('B'));

  automaton.addCallTransition(automaton::State(1), alphabet::symbolFrom('a'), automaton::State(2), alphabet::symbolFrom('C'));
  automaton.addCallTransition(automaton::State(2), alphabet::symbolFrom('a'), automaton::State(2), alphabet::symbolFrom('D'));
  automaton.addReturnTransition(automaton::State(2), alphabet::symbolFrom('^'), alphabet::symbolFrom('D'), automaton::State(2));
  automaton.addReturnTransition(automaton::State(2), alphabet::symbolFrom('^'), alphabet::symbolFrom('C'), automaton::State(3));

  automaton.addCallTransition(automaton::State(3), alphabet::symbolFrom('a'), automaton::State(4), alphabet::symbolFrom('E'));
  automaton.addCallTransition(automaton::State(4), alphabet::symbolFrom('a'), automaton::State(4), alphabet::symbolFrom('F'));
  automaton.addReturnTransition(automaton::State(4), alphabet::symbolFrom('^'), alphabet::symbolFrom('F'), automaton::State(4));
  automaton.addReturnTransition(automaton::State(4), alphabet::symbolFrom('^'), alphabet::symbolFrom('E'), automaton::State(5));

  automaton.addReturnTransition(automaton::State(5), alphabet::symbolFrom('^'), alphabet::symbolFrom('B'), automaton::State(6));

  automaton.addInitialState(automaton::State(0));
  automaton.addFinalState(automaton::State(4));
}
