#include "GrammarPropertiesTest.h"
#include <factory/DataFactory.hpp>

#include <label/PrimitiveLabel.h>
#include <alphabet/LabeledSymbol.h>
#include <alphabet/Symbol.h>

#include "grammar/GrammarPropertiesCFG.h"

#define CPPUNIT_IMPLY(x, y) CPPUNIT_ASSERT(!(x) || (y))

CPPUNIT_TEST_SUITE_REGISTRATION( GrammarPropertiesTest );

void GrammarPropertiesTest::setUp() {
}

void GrammarPropertiesTest::tearDown() {
}

void GrammarPropertiesTest::testNullable() {
	alphabet::Symbol X = alphabet::symbolFrom('X');
	alphabet::Symbol Y = alphabet::symbolFrom('Y');
	alphabet::Symbol Z = alphabet::symbolFrom('Z');
	alphabet::Symbol d = alphabet::symbolFrom('d');

	grammar::CFG grammar(X);
	grammar.setTerminalAlphabet({d});
	grammar.setNonterminalAlphabet({{X, Y, Z}});
	grammar.setInitialSymbol(X);

	grammar.addRule(X, std::vector<alphabet::Symbol>{ d });
	grammar.addRule(X, std::vector<alphabet::Symbol>{ Y });
	grammar.addRule(Y, std::vector<alphabet::Symbol>{ d });
	grammar.addRule(Y, std::vector<alphabet::Symbol>{ });
	grammar.addRule(Z, std::vector<alphabet::Symbol>{ d });
	grammar.addRule(Z, std::vector<alphabet::Symbol>{ X, Y, Z });

	std::set<alphabet::Symbol> res = {X, Y};
	CPPUNIT_ASSERT(res == grammar::GrammarPropertiesCFG::getNullableNonterminals(grammar));
}

void GrammarPropertiesTest::testUnitRules() {
	alphabet::Symbol S = alphabet::symbolFrom('S');
	alphabet::Symbol A = alphabet::symbolFrom('A');
	alphabet::Symbol B = alphabet::symbolFrom('B');
	alphabet::Symbol C = alphabet::symbolFrom('C');
	alphabet::Symbol a = alphabet::symbolFrom('a');
	alphabet::Symbol b = alphabet::symbolFrom('b');

	grammar::LeftLG llg(S);
	llg.setTerminalAlphabet({a, b});
	llg.setNonterminalAlphabet({S, A, B, C});
	llg.setInitialSymbol(S);

	llg.addRule(S, std::make_pair(A, std::vector<alphabet::Symbol>{}));
	llg.addRule(S, std::make_pair(B, std::vector<alphabet::Symbol>{}));
	llg.addRule(A, std::make_pair(A, std::vector<alphabet::Symbol>{a}));
	llg.addRule(A, std::make_pair(B, std::vector<alphabet::Symbol>{}));
	llg.addRule(A, std::make_pair(C, std::vector<alphabet::Symbol>{}));
	llg.addRule(B, std::make_pair(B, std::vector<alphabet::Symbol>{b, a}));
	llg.addRule(B, std::make_pair(C, std::vector<alphabet::Symbol>{b}));
	llg.addRule(C, {b});
	llg.addRule(C, std::make_pair(C, std::vector<alphabet::Symbol>{a}));

	std::set<alphabet::Symbol> N_S = {S, A, B, C};
	std::set<alphabet::Symbol> N_A = {A, B, C};
	std::set<alphabet::Symbol> N_B = {B};
	std::set<alphabet::Symbol> N_C = {C};

	CPPUNIT_ASSERT(N_S == grammar::GrammarPropertiesCFG::getNonterminalUnitRuleCycle(llg, S));
	CPPUNIT_ASSERT(N_A == grammar::GrammarPropertiesCFG::getNonterminalUnitRuleCycle(llg, A));
	CPPUNIT_ASSERT(N_B == grammar::GrammarPropertiesCFG::getNonterminalUnitRuleCycle(llg, B));
	CPPUNIT_ASSERT(N_C == grammar::GrammarPropertiesCFG::getNonterminalUnitRuleCycle(llg, C));
}
