#include "RegExpConcatenateTest.h"

#include "regexp/RegExpConcatenate.h"

#include <sstream>
#include <regexp/RegExp.h>
#include <regexp/RegExpFromStringParser.h>
#include "std/set.hpp"

CPPUNIT_TEST_SUITE_REGISTRATION( RegExpConcatenateTest );

void RegExpConcatenateTest::setUp() {
}

void RegExpConcatenateTest::tearDown() {
}

void RegExpConcatenateTest::testRegExpConcatenate() {
	{
		std::string input1 = "(#E a b)";
		std::stringstream inputs1(input1);
		regexp::RegExpFromStringParser parser1(inputs1);
		regexp::RegExp re1 = parser1.parseValue();

		std::string input2 = "(#E a c)";
		std::stringstream inputs2(input2);
		regexp::RegExpFromStringParser parser2(inputs2);
		regexp::RegExp re2 = parser2.parseValue();

		regexp::RegExp re = regexp::RegExpConcatenate::concatenate(re1, re2);

		std::string inputr = "(#E a b)(#E a c)";
		std::stringstream inputsr(inputr);
		regexp::RegExpFromStringParser parserr(inputsr);
		regexp::RegExp rer = parserr.parseValue();

		CPPUNIT_ASSERT(re == rer);
	}
	{
		std::string input1 = "(#E a b)";
		std::stringstream inputs1(input1);
		regexp::RegExpFromStringParser parser1(inputs1);
		regexp::RegExp re1 = parser1.parseValue();

		regexp::RegExp re2(regexp::FormalRegExp(regexp::FormalRegExpConcatenation(regexp::FormalRegExpEmpty {}, regexp::FormalRegExpEpsilon {})));

		regexp::RegExp re = regexp::RegExpConcatenate::concatenate(re1, re2);

		std::string inputr = "(#E a b)(#0 #E )";
		std::stringstream inputsr(inputr);
		regexp::RegExpFromStringParser parserr(inputsr);
		regexp::RegExp rer = parserr.parseValue();

		CPPUNIT_ASSERT(re == rer);
	}
	{
		regexp::RegExp re1(regexp::FormalRegExp(regexp::FormalRegExpConcatenation(regexp::FormalRegExpEmpty {}, regexp::FormalRegExpEpsilon {})));

		std::string input1 = "(#E a b)";
		std::stringstream inputs1(input1);
		regexp::RegExpFromStringParser parser1(inputs1);
		regexp::RegExp re2 = parser1.parseValue();

		regexp::RegExp re = regexp::RegExpConcatenate::concatenate(re1, re2);

		std::string inputr = "(#0 #E )(#E a b)";
		std::stringstream inputsr(inputr);
		regexp::RegExpFromStringParser parserr(inputsr);
		regexp::RegExp tmp = parserr.parseValue();
		regexp::RegExp rer(regexp::FormalRegExp(static_cast<const regexp::UnboundedRegExp&>(tmp.getData())));

		std::cout << (static_cast<const regexp::UnboundedRegExp&>(tmp.getData())).getAlphabet() << std::endl;
		std::cout << (static_cast<const regexp::FormalRegExp&>(tmp.getData())).getAlphabet() << std::endl;

		std::cout << re << std::endl;
		std::cout << rer << std::endl;
		CPPUNIT_ASSERT(re == rer);
	}
}
