/*
 * FromXMLParsers.cpp
 *
 * Created on: Apr 1, 2013
 * Author: Jan Travnicek
 */

#include "Api.hpp"

namespace alib {

const label::LabelFromXMLParser FromXMLParsers::labelParser;
const alphabet::SymbolFromXMLParser FromXMLParsers::symbolParser;
const regexp::RegExpFromXMLParser FromXMLParsers::regexpParser;
const string::StringFromXMLParser FromXMLParsers::stringParser;
const automaton::AutomatonFromXMLParser FromXMLParsers::automatonParser;
const grammar::GrammarFromXMLParser FromXMLParsers::grammarParser;
const exception::ExceptionFromXMLParser FromXMLParsers::exceptionParser;
const alib::ObjectFromXMLParser FromXMLParsers::objectParser;
const container::ContainerFromXMLParser FromXMLParsers::containerParser;
const primitive::PrimitiveFromXMLParser FromXMLParsers::primitiveParser;

const label::LabelToXMLComposer ToXMLComposers::labelComposer;
const alphabet::SymbolToXMLComposer ToXMLComposers::symbolComposer;
const regexp::RegExpToXMLComposer ToXMLComposers::regexpComposer;
const string::StringToXMLComposer ToXMLComposers::stringComposer;
const automaton::AutomatonToXMLComposer ToXMLComposers::automatonComposer;
const grammar::GrammarToXMLComposer ToXMLComposers::grammarComposer;
const alib::ObjectToXMLComposer ToXMLComposers::objectComposer;
const exception::ExceptionToXMLComposer ToXMLComposers::exceptionComposer;
const container::ContainerToXMLComposer ToXMLComposers::containerComposer;
const primitive::PrimitiveToXMLComposer ToXMLComposers::primitiveComposer;

const ToXMLComposers ToXMLComposers::toXMLComposers;

Void api<Void>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::objectParser.parseVoid(input);
}

std::list<sax::Token> api<Void>::compose(const Void& data) {
	return ToXMLComposers::objectComposer.compose(data);
}

container::Container api<container::Container>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::containerParser.parseContainer(input);
}

std::list<sax::Token> api<container::Container>::compose(const container::Container& data) {
	return ToXMLComposers::containerComposer.compose(data);
}

container::ObjectsSet api<container::ObjectsSet>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::containerParser.parseObjectsSet(input);
}

std::list<sax::Token> api<container::ObjectsSet>::compose(const container::ObjectsSet& data) {
	return ToXMLComposers::containerComposer.compose(data);
}

container::ObjectsVector api<container::ObjectsVector>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::containerParser.parseObjectsVector(input);
}

std::list<sax::Token> api<container::ObjectsVector>::compose(const container::ObjectsVector& data) {
	return ToXMLComposers::containerComposer.compose(data);
}

container::ObjectsPair api<container::ObjectsPair>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::containerParser.parseObjectsPair(input);
}

std::list<sax::Token> api<container::ObjectsPair>::compose(const container::ObjectsPair& data) {
	return ToXMLComposers::containerComposer.compose(data);
}

container::ObjectsMap api<container::ObjectsMap>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::containerParser.parseObjectsMap(input);
}

std::list<sax::Token> api<container::ObjectsMap>::compose(const container::ObjectsMap& data) {
	return ToXMLComposers::containerComposer.compose(data);
}

alib::Object api<alib::Object>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::objectParser.parseObject(input);
}

std::list<sax::Token> api<alib::Object>::compose(const alib::Object& data) {
	return ToXMLComposers::objectComposer.compose(data);
}

exception::AlibException api<exception::AlibException>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::exceptionParser.parseException(input);
}

std::list<sax::Token> api<exception::AlibException>::compose(const exception::AlibException& data) {
	return ToXMLComposers::exceptionComposer.compose(data);
}

alphabet::Symbol api<alphabet::Symbol>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::symbolParser.parseSymbol(input);
}

std::list<sax::Token> api<alphabet::Symbol>::compose(const alphabet::Symbol& data) {
	return ToXMLComposers::symbolComposer.compose(data);
}

alphabet::LabeledSymbol api<alphabet::LabeledSymbol>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::symbolParser.parseLabeledSymbol(input);
}

std::list<sax::Token> api<alphabet::LabeledSymbol>::compose(const alphabet::LabeledSymbol& data) {
	return ToXMLComposers::symbolComposer.compose(data);
}

alphabet::BlankSymbol api<alphabet::BlankSymbol>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::symbolParser.parseBlankSymbol(input);
}

std::list<sax::Token> api<alphabet::BlankSymbol>::compose(const alphabet::BlankSymbol& data) {
	return ToXMLComposers::symbolComposer.compose(data);
}

alphabet::BottomOfTheStackSymbol api<alphabet::BottomOfTheStackSymbol>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::symbolParser.parseBottomOfTheStackSymbol(input);
}

std::list<sax::Token> api<alphabet::BottomOfTheStackSymbol>::compose(const alphabet::BottomOfTheStackSymbol& data) {
	return ToXMLComposers::symbolComposer.compose(data);
}

alphabet::EndSymbol api<alphabet::EndSymbol>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::symbolParser.parseEndSymbol(input);
}

std::list<sax::Token> api<alphabet::EndSymbol>::compose(const alphabet::EndSymbol& data) {
	return ToXMLComposers::symbolComposer.compose(data);
}


automaton::Automaton api<automaton::Automaton>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::automatonParser.parseAutomaton(input);
}

std::list<sax::Token> api<automaton::Automaton>::compose(const automaton::Automaton& data) {
	return ToXMLComposers::automatonComposer.compose(data);
}

automaton::EpsilonNFA api<automaton::EpsilonNFA>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::automatonParser.parseEpsilonNFA(input);
}

std::list<sax::Token> api<automaton::EpsilonNFA>::compose(const automaton::EpsilonNFA& data) {
	return ToXMLComposers::automatonComposer.compose(data);
}

automaton::MultiInitialStateNFA api<automaton::MultiInitialStateNFA>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::automatonParser.parseMultiInitialStateNFA(input);
}

std::list<sax::Token> api<automaton::MultiInitialStateNFA>::compose(const automaton::MultiInitialStateNFA& data) {
	return ToXMLComposers::automatonComposer.compose(data);
}

automaton::NFA api<automaton::NFA>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::automatonParser.parseNFA(input);
}

std::list<sax::Token> api<automaton::NFA>::compose(const automaton::NFA& data) {
	return ToXMLComposers::automatonComposer.compose(data);
}

automaton::DFA api<automaton::DFA>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::automatonParser.parseDFA(input);
}

std::list<sax::Token> api<automaton::DFA>::compose(const automaton::DFA& data) {
	return ToXMLComposers::automatonComposer.compose(data);
}

automaton::ExtendedNFA api<automaton::ExtendedNFA>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::automatonParser.parseExtendedNFA(input);
}

std::list<sax::Token> api<automaton::ExtendedNFA>::compose(const automaton::ExtendedNFA& data) {
	return ToXMLComposers::automatonComposer.compose(data);
}

automaton::CompactNFA api<automaton::CompactNFA>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::automatonParser.parseCompactNFA(input);
}

std::list<sax::Token> api<automaton::CompactNFA>::compose(const automaton::CompactNFA& data) {
	return ToXMLComposers::automatonComposer.compose(data);
}

automaton::DPDA api<automaton::DPDA>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::automatonParser.parseDPDA(input);
}

std::list<sax::Token> api<automaton::DPDA>::compose(const automaton::DPDA& data) {
	return ToXMLComposers::automatonComposer.compose(data);
}

automaton::SinglePopDPDA api<automaton::SinglePopDPDA>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::automatonParser.parseSinglePopDPDA(input);
}

std::list<sax::Token> api<automaton::SinglePopDPDA>::compose(const automaton::SinglePopDPDA& data) {
	return ToXMLComposers::automatonComposer.compose(data);
}

automaton::InputDrivenNPDA api<automaton::InputDrivenNPDA>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::automatonParser.parseInputDrivenNPDA(input);
}

std::list<sax::Token> api<automaton::InputDrivenNPDA>::compose(const automaton::InputDrivenNPDA& data) {
	return ToXMLComposers::automatonComposer.compose(data);
}

automaton::VisiblyPushdownDPDA api<automaton::VisiblyPushdownDPDA>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::automatonParser.parseVisiblyPushdownDPDA(input);
}

std::list<sax::Token> api<automaton::VisiblyPushdownDPDA>::compose(const automaton::VisiblyPushdownDPDA& data) {
	return ToXMLComposers::automatonComposer.compose(data);
}

automaton::VisiblyPushdownNPDA api<automaton::VisiblyPushdownNPDA>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::automatonParser.parseVisiblyPushdownNPDA(input);
}

std::list<sax::Token> api<automaton::VisiblyPushdownNPDA>::compose(const automaton::VisiblyPushdownNPDA& data) {
	return ToXMLComposers::automatonComposer.compose(data);
}

automaton::RealTimeHeightDeterministicDPDA api<automaton::RealTimeHeightDeterministicDPDA>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::automatonParser.parseRealTimeHeightDeterministicDPDA(input);
}

std::list<sax::Token> api<automaton::RealTimeHeightDeterministicDPDA>::compose(const automaton::RealTimeHeightDeterministicDPDA& data) {
	return ToXMLComposers::automatonComposer.compose(data);
}

automaton::RealTimeHeightDeterministicNPDA api<automaton::RealTimeHeightDeterministicNPDA>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::automatonParser.parseRealTimeHeightDeterministicNPDA(input);
}

std::list<sax::Token> api<automaton::RealTimeHeightDeterministicNPDA>::compose(const automaton::RealTimeHeightDeterministicNPDA& data) {
	return ToXMLComposers::automatonComposer.compose(data);
}

automaton::NPDA api<automaton::NPDA>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::automatonParser.parseNPDA(input);
}

std::list<sax::Token> api<automaton::NPDA>::compose(const automaton::NPDA& data) {
	return ToXMLComposers::automatonComposer.compose(data);
}

automaton::SinglePopNPDA api<automaton::SinglePopNPDA>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::automatonParser.parseSinglePopNPDA(input);
}

std::list<sax::Token> api<automaton::SinglePopNPDA>::compose(const automaton::SinglePopNPDA& data) {
	return ToXMLComposers::automatonComposer.compose(data);
}

automaton::OneTapeDTM api<automaton::OneTapeDTM>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::automatonParser.parseOneTapeDTM(input);
}

std::list<sax::Token> api<automaton::OneTapeDTM>::compose(const automaton::OneTapeDTM& data) {
	return ToXMLComposers::automatonComposer.compose(data);
}

automaton::State api<automaton::State>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::automatonParser.parseState(input);
}

std::list<sax::Token> api<automaton::State>::compose(const automaton::State& data) {
	return ToXMLComposers::automatonComposer.compose(data);
}


grammar::Grammar api<grammar::Grammar>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::grammarParser.parseGrammar(input);
}

std::list<sax::Token> api<grammar::Grammar>::compose(const grammar::Grammar& data) {
	return ToXMLComposers::grammarComposer.compose(data);
}

grammar::LeftLG api<grammar::LeftLG>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::grammarParser.parseLeftLG(input);
}

std::list<sax::Token> api<grammar::LeftLG>::compose(const grammar::LeftLG& data) {
	return ToXMLComposers::grammarComposer.compose(data);
}

grammar::LeftRG api<grammar::LeftRG>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::grammarParser.parseLeftRG(input);
}

std::list<sax::Token> api<grammar::LeftRG>::compose(const grammar::LeftRG& data) {
	return ToXMLComposers::grammarComposer.compose(data);
}

grammar::RightLG api<grammar::RightLG>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::grammarParser.parseRightLG(input);
}

std::list<sax::Token> api<grammar::RightLG>::compose(const grammar::RightLG& data) {
	return ToXMLComposers::grammarComposer.compose(data);
}

grammar::RightRG api<grammar::RightRG>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::grammarParser.parseRightRG(input);
}

std::list<sax::Token> api<grammar::RightRG>::compose(const grammar::RightRG& data) {
	return ToXMLComposers::grammarComposer.compose(data);
}

grammar::LG api<grammar::LG>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::grammarParser.parseLG(input);
}

std::list<sax::Token> api<grammar::LG>::compose(const grammar::LG& data) {
	return ToXMLComposers::grammarComposer.compose(data);
}

grammar::CFG api<grammar::CFG>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::grammarParser.parseCFG(input);
}

std::list<sax::Token> api<grammar::CFG>::compose(const grammar::CFG& data) {
	return ToXMLComposers::grammarComposer.compose(data);
}

grammar::EpsilonFreeCFG api<grammar::EpsilonFreeCFG>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::grammarParser.parseEpsilonFreeCFG(input);
}

std::list<sax::Token> api<grammar::EpsilonFreeCFG>::compose(const grammar::EpsilonFreeCFG& data) {
	return ToXMLComposers::grammarComposer.compose(data);
}

grammar::CNF api<grammar::CNF>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::grammarParser.parseCNF(input);
}

std::list<sax::Token> api<grammar::CNF>::compose(const grammar::CNF& data) {
	return ToXMLComposers::grammarComposer.compose(data);
}

grammar::GNF api<grammar::GNF>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::grammarParser.parseGNF(input);
}

std::list<sax::Token> api<grammar::GNF>::compose(const grammar::GNF& data) {
	return ToXMLComposers::grammarComposer.compose(data);
}

grammar::CSG api<grammar::CSG>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::grammarParser.parseCSG(input);
}

std::list<sax::Token> api<grammar::CSG>::compose(const grammar::CSG& data) {
	return ToXMLComposers::grammarComposer.compose(data);
}

grammar::NonContractingGrammar api<grammar::NonContractingGrammar>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::grammarParser.parseNonContractingGrammar(input);
}

std::list<sax::Token> api<grammar::NonContractingGrammar>::compose(const grammar::NonContractingGrammar& data) {
	return ToXMLComposers::grammarComposer.compose(data);
}

grammar::ContextPreservingUnrestrictedGrammar api<grammar::ContextPreservingUnrestrictedGrammar>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::grammarParser.parseContextPreservingUnrestrictedGrammar(input);
}

std::list<sax::Token> api<grammar::ContextPreservingUnrestrictedGrammar>::compose(const grammar::ContextPreservingUnrestrictedGrammar& data) {
	return ToXMLComposers::grammarComposer.compose(data);
}

grammar::UnrestrictedGrammar api<grammar::UnrestrictedGrammar>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::grammarParser.parseUnrestrictedGrammar(input);
}

std::list<sax::Token> api<grammar::UnrestrictedGrammar>::compose(const grammar::UnrestrictedGrammar& data) {
	return ToXMLComposers::grammarComposer.compose(data);
}


label::Label api<label::Label>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::labelParser.parseLabel(input);
}

std::list<sax::Token> api<label::Label>::compose(const label::Label& data) {
	return ToXMLComposers::labelComposer.compose(data);
}

label::PrimitiveLabel api<label::PrimitiveLabel>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::labelParser.parsePrimitiveLabel(input);
}

std::list<sax::Token> api<label::PrimitiveLabel>::compose(const label::PrimitiveLabel& data) {
	return ToXMLComposers::labelComposer.compose(data);
}

label::HexavigesimalLabel api<label::HexavigesimalLabel>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::labelParser.parseHexavigesimalLabel(input);
}

std::list<sax::Token> api<label::HexavigesimalLabel>::compose(const label::HexavigesimalLabel& data) {
	return ToXMLComposers::labelComposer.compose(data);
}

label::ObjectLabel api<label::ObjectLabel>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::labelParser.parseObjectLabel(input);
}

std::list<sax::Token> api<label::ObjectLabel>::compose(const label::ObjectLabel& data) {
	return ToXMLComposers::labelComposer.compose(data);
}

label::LabelSetLabel api<label::LabelSetLabel>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::labelParser.parseLabelSetLabel(input);
}

std::list<sax::Token> api<label::LabelSetLabel>::compose(const label::LabelSetLabel& data) {
	return ToXMLComposers::labelComposer.compose(data);
}

label::LabelPairLabel api<label::LabelPairLabel>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::labelParser.parseLabelPairLabel(input);
}

std::list<sax::Token> api<label::LabelPairLabel>::compose(const label::LabelPairLabel& data) {
	return ToXMLComposers::labelComposer.compose(data);
}


regexp::RegExp api<regexp::RegExp>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::regexpParser.parseRegExp(input);
}

std::list<sax::Token> api<regexp::RegExp>::compose(const regexp::RegExp& data) {
	return ToXMLComposers::regexpComposer.compose(data);
}

regexp::UnboundedRegExp api<regexp::UnboundedRegExp>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::regexpParser.parseUnboundedRegExp(input);
}

std::list<sax::Token> api<regexp::UnboundedRegExp>::compose(const regexp::UnboundedRegExp& data) {
	return ToXMLComposers::regexpComposer.compose(data);
}

regexp::FormalRegExp api<regexp::FormalRegExp>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::regexpParser.parseFormalRegExp(input);
}

std::list<sax::Token> api<regexp::FormalRegExp>::compose(const regexp::FormalRegExp& data) {
	return ToXMLComposers::regexpComposer.compose(data);
}


string::String api<string::String>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::stringParser.parseString(input);
}

std::list<sax::Token> api<string::String>::compose(const string::String& data) {
	return ToXMLComposers::stringComposer.compose(data);
}

string::LinearString api<string::LinearString>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::stringParser.parseLinearString(input);
}

std::list<sax::Token> api<string::LinearString>::compose(const string::LinearString& data) {
	return ToXMLComposers::stringComposer.compose(data);
}

string::CyclicString api<string::CyclicString>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::stringParser.parseCyclicString(input);
}

std::list<sax::Token> api<string::CyclicString>::compose(const string::CyclicString& data) {
	return ToXMLComposers::stringComposer.compose(data);
}

string::Epsilon api<string::Epsilon>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::stringParser.parseEpsilon(input);
}

std::list<sax::Token> api<string::Epsilon>::compose(const string::Epsilon& data) {
	return ToXMLComposers::stringComposer.compose(data);
}


primitive::Primitive api<primitive::Primitive>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::primitiveParser.parsePrimitive(input);
}

std::list<sax::Token> api<primitive::Primitive>::compose(const primitive::Primitive& data) {
	return ToXMLComposers::primitiveComposer.compose(data);
}

primitive::String api<primitive::String>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::primitiveParser.parseString(input);
}

std::list<sax::Token> api<primitive::String>::compose(const primitive::String& data) {
	return ToXMLComposers::primitiveComposer.compose(data);
}

primitive::Integer api<primitive::Integer>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::primitiveParser.parseInteger(input);
}

std::list<sax::Token> api<primitive::Integer>::compose(const primitive::Integer& data) {
	return ToXMLComposers::primitiveComposer.compose(data);
}

primitive::Character api<primitive::Character>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::primitiveParser.parseCharacter(input);
}

std::list<sax::Token> api<primitive::Character>::compose(const primitive::Character& data) {
	return ToXMLComposers::primitiveComposer.compose(data);
}


void ToXMLComposers::Visit(void* data, const Void& voidObject) const {
	*((std::list<sax::Token>*) data) = std::move(api<Void>::compose(voidObject));
}

void ToXMLComposers::Visit(void* data, const container::ObjectsSet& container) const {
	*((std::list<sax::Token>*) data) = std::move(api<container::ObjectsSet>::compose(container));
}

void ToXMLComposers::Visit(void* data, const container::ObjectsVector& container) const {
	*((std::list<sax::Token>*) data) = std::move(api<container::ObjectsVector>::compose(container));
}

void ToXMLComposers::Visit(void* data, const container::ObjectsPair& container) const {
	*((std::list<sax::Token>*) data) = std::move(api<container::ObjectsPair>::compose(container));
}

void ToXMLComposers::Visit(void* data, const container::ObjectsMap& container) const {
	*((std::list<sax::Token>*) data) = std::move(api<container::ObjectsMap>::compose(container));
}

void ToXMLComposers::Visit(void* data, const exception::AlibException& symbol) const {
	*((std::list<sax::Token>*) data) = std::move(api<exception::AlibException>::compose(symbol));
}

void ToXMLComposers::Visit(void* data, const alphabet::BlankSymbol& symbol) const {
	*((std::list<sax::Token>*) data) = std::move(api<alphabet::BlankSymbol>::compose(symbol));
}

void ToXMLComposers::Visit(void* data, const alphabet::BottomOfTheStackSymbol& symbol) const {
	*((std::list<sax::Token>*) data) = std::move(api<alphabet::BottomOfTheStackSymbol>::compose(symbol));
}

void ToXMLComposers::Visit(void* data, const alphabet::EndSymbol& symbol) const {
	*((std::list<sax::Token>*) data) = std::move(api<alphabet::EndSymbol>::compose(symbol));
}

void ToXMLComposers::Visit(void* data, const alphabet::LabeledSymbol& symbol) const {
	*((std::list<sax::Token>*) data) = std::move(api<alphabet::LabeledSymbol>::compose(symbol));
}

void ToXMLComposers::Visit(void* data, const automaton::EpsilonNFA& automaton) const {
	*((std::list<sax::Token>*) data) = std::move(api<automaton::EpsilonNFA>::compose(automaton));
}

void ToXMLComposers::Visit(void* data, const automaton::MultiInitialStateNFA& automaton) const {
	*((std::list<sax::Token>*) data) = std::move(api<automaton::MultiInitialStateNFA>::compose(automaton));
}

void ToXMLComposers::Visit(void* data, const automaton::NFA& automaton) const {
	*((std::list<sax::Token>*) data) = std::move(api<automaton::NFA>::compose(automaton));
}

void ToXMLComposers::Visit(void* data, const automaton::DFA& automaton) const {
	*((std::list<sax::Token>*) data) = std::move(api<automaton::DFA>::compose(automaton));
}

void ToXMLComposers::Visit(void* data, const automaton::ExtendedNFA& automaton) const {
	*((std::list<sax::Token>*) data) = std::move(api<automaton::ExtendedNFA>::compose(automaton));
}

void ToXMLComposers::Visit(void* data, const automaton::CompactNFA& automaton) const {
	*((std::list<sax::Token>*) data) = std::move(api<automaton::CompactNFA>::compose(automaton));
}

void ToXMLComposers::Visit(void* data, const automaton::DPDA& automaton) const {
	*((std::list<sax::Token>*) data) = std::move(api<automaton::DPDA>::compose(automaton));
}

void ToXMLComposers::Visit(void* data, const automaton::SinglePopDPDA& automaton) const {
	*((std::list<sax::Token>*) data) = std::move(api<automaton::SinglePopDPDA>::compose(automaton));
}

void ToXMLComposers::Visit(void* data, const automaton::InputDrivenNPDA& automaton) const {
	*((std::list<sax::Token>*) data) = std::move(api<automaton::InputDrivenNPDA>::compose(automaton));
}

void ToXMLComposers::Visit(void* data, const automaton::VisiblyPushdownDPDA& automaton) const {
	*((std::list<sax::Token>*) data) = std::move(api<automaton::VisiblyPushdownDPDA>::compose(automaton));
}

void ToXMLComposers::Visit(void* data, const automaton::VisiblyPushdownNPDA& automaton) const {
	*((std::list<sax::Token>*) data) = std::move(api<automaton::VisiblyPushdownNPDA>::compose(automaton));
}

void ToXMLComposers::Visit(void* data, const automaton::RealTimeHeightDeterministicDPDA& automaton) const {
	*((std::list<sax::Token>*) data) = std::move(api<automaton::RealTimeHeightDeterministicDPDA>::compose(automaton));
}

void ToXMLComposers::Visit(void* data, const automaton::RealTimeHeightDeterministicNPDA& automaton) const {
	*((std::list<sax::Token>*) data) = std::move(api<automaton::RealTimeHeightDeterministicNPDA>::compose(automaton));
}

void ToXMLComposers::Visit(void* data, const automaton::NPDA& automaton) const {
	*((std::list<sax::Token>*) data) = std::move(api<automaton::NPDA>::compose(automaton));
}

void ToXMLComposers::Visit(void* data, const automaton::SinglePopNPDA& automaton) const {
	*((std::list<sax::Token>*) data) = std::move(api<automaton::SinglePopNPDA>::compose(automaton));
}

void ToXMLComposers::Visit(void* data, const automaton::OneTapeDTM& automaton) const {
	*((std::list<sax::Token>*) data) = std::move(api<automaton::OneTapeDTM>::compose(automaton));
}

void ToXMLComposers::Visit(void* data, const grammar::LeftLG& grammar) const {
	*((std::list<sax::Token>*) data) = std::move(api<grammar::LeftLG>::compose(grammar));
}

void ToXMLComposers::Visit(void* data, const grammar::LeftRG& grammar) const {
	*((std::list<sax::Token>*) data) = std::move(api<grammar::LeftRG>::compose(grammar));
}

void ToXMLComposers::Visit(void* data, const grammar::RightLG& grammar) const {
	*((std::list<sax::Token>*) data) = std::move(api<grammar::RightLG>::compose(grammar));
}

void ToXMLComposers::Visit(void* data, const grammar::RightRG& grammar) const {
	*((std::list<sax::Token>*) data) = std::move(api<grammar::RightRG>::compose(grammar));
}

void ToXMLComposers::Visit(void* data, const grammar::LG& grammar) const {
	*((std::list<sax::Token>*) data) = std::move(api<grammar::LG>::compose(grammar));
}

void ToXMLComposers::Visit(void* data, const grammar::CFG& grammar) const {
	*((std::list<sax::Token>*) data) = std::move(api<grammar::CFG>::compose(grammar));
}

void ToXMLComposers::Visit(void* data, const grammar::EpsilonFreeCFG& grammar) const {
	*((std::list<sax::Token>*) data) = std::move(api<grammar::EpsilonFreeCFG>::compose(grammar));
}

void ToXMLComposers::Visit(void* data, const grammar::CNF& grammar) const {
	*((std::list<sax::Token>*) data) = std::move(api<grammar::CNF>::compose(grammar));
}

void ToXMLComposers::Visit(void* data, const grammar::GNF& grammar) const {
	*((std::list<sax::Token>*) data) = std::move(api<grammar::GNF>::compose(grammar));
}

void ToXMLComposers::Visit(void* data, const grammar::CSG& grammar) const {
	*((std::list<sax::Token>*) data) = std::move(api<grammar::CSG>::compose(grammar));
}

void ToXMLComposers::Visit(void* data, const grammar::NonContractingGrammar& grammar) const {
	*((std::list<sax::Token>*) data) = std::move(api<grammar::NonContractingGrammar>::compose(grammar));
}

void ToXMLComposers::Visit(void* data, const grammar::ContextPreservingUnrestrictedGrammar& grammar) const {
	*((std::list<sax::Token>*) data) = std::move(api<grammar::ContextPreservingUnrestrictedGrammar>::compose(grammar));
}

void ToXMLComposers::Visit(void* data, const grammar::UnrestrictedGrammar& grammar) const {
	*((std::list<sax::Token>*) data) = std::move(api<grammar::UnrestrictedGrammar>::compose(grammar));
}

void ToXMLComposers::Visit(void* data, const label::PrimitiveLabel& label) const {
	*((std::list<sax::Token>*) data) = std::move(api<label::PrimitiveLabel>::compose(label));
}

void ToXMLComposers::Visit(void* data, const label::HexavigesimalLabel& label) const {
	*((std::list<sax::Token>*) data) = std::move(api<label::HexavigesimalLabel>::compose(label));
}

void ToXMLComposers::Visit(void* data, const label::ObjectLabel& label) const {
	*((std::list<sax::Token>*) data) = std::move(api<label::ObjectLabel>::compose(label));
}

void ToXMLComposers::Visit(void* data, const label::LabelSetLabel& label) const {
	*((std::list<sax::Token>*) data) = std::move(api<label::LabelSetLabel>::compose(label));
}

void ToXMLComposers::Visit(void* data, const label::LabelPairLabel& label) const {
	*((std::list<sax::Token>*) data) = std::move(api<label::LabelPairLabel>::compose(label));
}

void ToXMLComposers::Visit(void* data, const regexp::UnboundedRegExp& regexp) const {
	*((std::list<sax::Token>*) data) = std::move(api<regexp::UnboundedRegExp>::compose(regexp));
}

void ToXMLComposers::Visit(void* data, const regexp::FormalRegExp& regexp) const {
	*((std::list<sax::Token>*) data) = std::move(api<regexp::FormalRegExp>::compose(regexp));
}

void ToXMLComposers::Visit(void* data, const string::Epsilon& string) const {
	*((std::list<sax::Token>*) data) = std::move(api<string::Epsilon>::compose(string));
}

void ToXMLComposers::Visit(void* data, const string::CyclicString& string) const {
	*((std::list<sax::Token>*) data) = std::move(api<string::CyclicString>::compose(string));
}

void ToXMLComposers::Visit(void* data, const string::LinearString& string) const {
	*((std::list<sax::Token>*) data) = std::move(api<string::LinearString>::compose(string));
}

void ToXMLComposers::Visit(void* data, const primitive::String& primitive) const {
	*((std::list<sax::Token>*) data) = std::move(api<primitive::String>::compose(primitive));
}

void ToXMLComposers::Visit(void* data, const primitive::Integer& primitive) const {
	*((std::list<sax::Token>*) data) = std::move(api<primitive::Integer>::compose(primitive));
}

void ToXMLComposers::Visit(void* data, const primitive::Character& primitive) const {
	*((std::list<sax::Token>*) data) = std::move(api<primitive::Character>::compose(primitive));
}

} /* namespace alib */
