/*
 * FromXMLParsers.hpp
 *
 * Created on: Apr 1, 2013
 * Author: Jan Travnicek
 */

#ifndef FROM_XML_PARSERS_HPP_
#define FROM_XML_PARSERS_HPP_

#include "label/LabelFromXMLParser.h"
#include "alphabet/SymbolFromXMLParser.h"
#include "regexp/RegExpFromXMLParser.h"
#include "string/StringFromXMLParser.h"
#include "automaton/AutomatonFromXMLParser.h"
#include "grammar/GrammarFromXMLParser.h"
#include "object/ObjectFromXMLParser.h"
#include "exception/ExceptionFromXMLParser.h"
#include "primitive/PrimitiveFromXMLParser.h"

#include "label/LabelToXMLComposer.h"
#include "alphabet/SymbolToXMLComposer.h"
#include "regexp/RegExpToXMLComposer.h"
#include "string/StringToXMLComposer.h"
#include "automaton/AutomatonToXMLComposer.h"
#include "grammar/GrammarToXMLComposer.h"
#include "object/ObjectToXMLComposer.h"
#include "exception/ExceptionToXMLComposer.h"
#include "primitive/PrimitiveToXMLComposer.h"

#include "object/ObjectBase.h"

namespace alib {

template<typename T>
struct api {
	static T parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const T& data);
};

template<>
struct api<Void> {
	static Void parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const Void& data);
};

template<typename T>
struct api<std::set<T>> {
	static std::set<T> parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const std::set<T>& data);
};

template<>
struct api<container::ObjectsSet> {
	static container::ObjectsSet parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const container::ObjectsSet& data);
};

template<typename T>
struct api<std::vector<T>> {
	static std::vector<T> parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const std::vector<T>& data);
};

template<>
struct api<container::ObjectsVector> {
	static container::ObjectsVector parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const container::ObjectsVector& data);
};

template<typename T, typename R>
struct api<std::pair<T, R>> {
	static std::pair<T, R> parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const std::pair<T, R>& data);
};

template<>
struct api<container::ObjectsPair> {
	static container::ObjectsPair parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const container::ObjectsPair& data);
};

template<typename T, typename R>
struct api<std::map<T, R>> {
	static std::map<T, R> parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const std::map<T, R>& data);
};

template<>
struct api<container::ObjectsMap> {
	static container::ObjectsMap parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const container::ObjectsMap& data);
};


template<>
struct api<Object> {
	static Object parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const Object& data);
};


template<>
struct api<exception::AlibException> {
	static exception::AlibException parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const exception::AlibException& data);
};


template<>
struct api<alphabet::Symbol> {
	static alphabet::Symbol parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const alphabet::Symbol& data);
};

template<>
struct api<alphabet::LabeledSymbol> {
	static alphabet::LabeledSymbol parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const alphabet::LabeledSymbol& data);
};

template<>
struct api<alphabet::BlankSymbol> {
	static alphabet::BlankSymbol parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const alphabet::BlankSymbol& data);
};

template<>
struct api<alphabet::BottomOfTheStackSymbol> {
	static alphabet::BottomOfTheStackSymbol parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const alphabet::BottomOfTheStackSymbol& data);
};

template<>
struct api<alphabet::EndSymbol> {
	static alphabet::EndSymbol parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const alphabet::EndSymbol& data);
};


template<>
struct api<automaton::Automaton> {
	static automaton::Automaton parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const automaton::Automaton& data);
};

template<>
struct api<automaton::EpsilonNFA> {
	static automaton::EpsilonNFA parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const automaton::EpsilonNFA& data);
};

template<>
struct api<automaton::MultiInitialStateNFA> {
	static automaton::MultiInitialStateNFA parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const automaton::MultiInitialStateNFA& data);
};

template<>
struct api<automaton::NFA> {
	static automaton::NFA parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const automaton::NFA& data);
};

template<>
struct api<automaton::DFA> {
	static automaton::DFA parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const automaton::DFA& data);
};

template<>
struct api<automaton::ExtendedNFA> {
	static automaton::ExtendedNFA parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const automaton::ExtendedNFA& data);
};

template<>
struct api<automaton::CompactNFA> {
	static automaton::CompactNFA parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const automaton::CompactNFA& data);
};

template<>
struct api<automaton::DPDA> {
	static automaton::DPDA parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const automaton::DPDA& data);
};

template<>
struct api<automaton::SinglePopDPDA> {
	static automaton::SinglePopDPDA parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const automaton::SinglePopDPDA& data);
};

template<>
struct api<automaton::InputDrivenNPDA> {
	static automaton::InputDrivenNPDA parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const automaton::InputDrivenNPDA& data);
};

template<>
struct api<automaton::VisiblyPushdownDPDA> {
	static automaton::VisiblyPushdownDPDA parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const automaton::VisiblyPushdownDPDA& data);
};

template<>
struct api<automaton::VisiblyPushdownNPDA> {
	static automaton::VisiblyPushdownNPDA parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const automaton::VisiblyPushdownNPDA& data);
};

template<>
struct api<automaton::RealTimeHeightDeterministicDPDA> {
	static automaton::RealTimeHeightDeterministicDPDA parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const automaton::RealTimeHeightDeterministicDPDA& data);
};

template<>
struct api<automaton::RealTimeHeightDeterministicNPDA> {
	static automaton::RealTimeHeightDeterministicNPDA parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const automaton::RealTimeHeightDeterministicNPDA& data);
};

template<>
struct api<automaton::NPDA> {
	static automaton::NPDA parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const automaton::NPDA& data);
};

template<>
struct api<automaton::SinglePopNPDA> {
	static automaton::SinglePopNPDA parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const automaton::SinglePopNPDA& data);
};

template<>
struct api<automaton::OneTapeDTM> {
	static automaton::OneTapeDTM parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const automaton::OneTapeDTM& data);
};

template<>
struct api<automaton::State> {
	static automaton::State parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const automaton::State& data);
};

template<>
struct api<grammar::Grammar> {
	static grammar::Grammar parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const grammar::Grammar& data);
};

template<>
struct api<grammar::LeftLG> {
	static grammar::LeftLG parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const grammar::LeftLG& data);
};

template<>
struct api<grammar::LeftRG> {
	static grammar::LeftRG parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const grammar::LeftRG& data);
};

template<>
struct api<grammar::RightLG> {
	static grammar::RightLG parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const grammar::RightLG& data);
};

template<>
struct api<grammar::RightRG> {
	static grammar::RightRG parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const grammar::RightRG& data);
};

template<>
struct api<grammar::LG> {
	static grammar::LG parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const grammar::LG& data);
};

template<>
struct api<grammar::CFG> {
	static grammar::CFG parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const grammar::CFG& data);
};

template<>
struct api<grammar::EpsilonFreeCFG> {
	static grammar::EpsilonFreeCFG parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const grammar::EpsilonFreeCFG& data);
};

template<>
struct api<grammar::CNF> {
	static grammar::CNF parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const grammar::CNF& data);
};

template<>
struct api<grammar::GNF> {
	static grammar::GNF parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const grammar::GNF& data);
};

template<>
struct api<grammar::CSG> {
	static grammar::CSG parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const grammar::CSG& data);
};

template<>
struct api<grammar::NonContractingGrammar> {
	static grammar::NonContractingGrammar parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const grammar::NonContractingGrammar& data);
};

template<>
struct api<grammar::ContextPreservingUnrestrictedGrammar> {
	static grammar::ContextPreservingUnrestrictedGrammar parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const grammar::ContextPreservingUnrestrictedGrammar& data);
};

template<>
struct api<grammar::UnrestrictedGrammar> {
	static grammar::UnrestrictedGrammar parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const grammar::UnrestrictedGrammar& data);
};


template<>
struct api<label::Label> {
	static label::Label parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const label::Label& data);
};

template<>
struct api<label::PrimitiveLabel> {
	static label::PrimitiveLabel parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const label::PrimitiveLabel& data);
};

template<>
struct api<label::HexavigesimalLabel> {
	static label::HexavigesimalLabel parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const label::HexavigesimalLabel& data);
};

template<>
struct api<label::ObjectLabel> {
	static label::ObjectLabel parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const label::ObjectLabel& data);
};

template<>
struct api<label::LabelSetLabel> {
	static label::LabelSetLabel parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const label::LabelSetLabel& data);
};

template<>
struct api<label::LabelPairLabel> {
	static label::LabelPairLabel parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const label::LabelPairLabel& data);
};


template<>
struct api<regexp::RegExp> {
	static regexp::RegExp parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const regexp::RegExp& data);
};

template<>
struct api<regexp::UnboundedRegExp> {
	static regexp::UnboundedRegExp parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const regexp::UnboundedRegExp& data);
};

template<>
struct api<regexp::FormalRegExp> {
	static regexp::FormalRegExp parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const regexp::FormalRegExp& data);
};


template<>
struct api<string::String> {
	static string::String parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const string::String& data);
};

template<>
struct api<string::LinearString> {
	static string::LinearString parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const string::LinearString& data);
};

template<>
struct api<string::CyclicString> {
	static string::CyclicString parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const string::CyclicString& data);
};

template<>
struct api<string::Epsilon> {
	static string::Epsilon parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const string::Epsilon& data);
};


template<>
struct api<primitive::Primitive> {
	static primitive::Primitive parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const primitive::Primitive& data);
};

template<>
struct api<primitive::String> {
	static primitive::String parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const primitive::String& data);
};

template<>
struct api<primitive::Integer> {
	static primitive::Integer parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const primitive::Integer& data);
};

template<>
struct api<primitive::Character> {
	static primitive::Character parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const primitive::Character& data);
};

} /* namespace alib */

#include "container/ContainerFromXMLParser.hpp"

#include "container/ContainerToXMLComposer.hpp"

namespace alib {

template<>
struct api<container::Container> {
	static container::Container parse(std::list<sax::Token>& input);
	static std::list<sax::Token> compose(const container::Container& data);
};

/**
 * Basic exception from which are derived all other exceptions.
 * Contains reason why the exception occured.
 */
class FromXMLParsers {
public:
	static const label::LabelFromXMLParser labelParser;
	static const alphabet::SymbolFromXMLParser symbolParser;
	static const regexp::RegExpFromXMLParser regexpParser;
	static const string::StringFromXMLParser stringParser;
	static const automaton::AutomatonFromXMLParser automatonParser;
	static const grammar::GrammarFromXMLParser grammarParser;
	static const exception::ExceptionFromXMLParser exceptionParser;
	static const ObjectFromXMLParser objectParser;
	static const container::ContainerFromXMLParser containerParser;
	static const primitive::PrimitiveFromXMLParser primitiveParser;

};

class ToXMLComposers : public VisitableObjectBase::const_visitor_type {
	void Visit(void*, const Void& voidObject) const;

	void Visit(void*, const exception::AlibException& symbol) const;

	void Visit(void*, const alphabet::LabeledSymbol& symbol) const;
	void Visit(void*, const alphabet::BlankSymbol& symbol) const;
	void Visit(void*, const alphabet::BottomOfTheStackSymbol& symbol) const;
	void Visit(void*, const alphabet::EndSymbol& symbol) const;

	void Visit(void*, const automaton::EpsilonNFA& automaton) const;
	void Visit(void*, const automaton::MultiInitialStateNFA& automaton) const;
	void Visit(void*, const automaton::NFA& automaton) const;
	void Visit(void*, const automaton::DFA& automaton) const;
	void Visit(void*, const automaton::ExtendedNFA& automaton) const;
	void Visit(void*, const automaton::CompactNFA& automaton) const;
	void Visit(void*, const automaton::DPDA& automaton) const;
	void Visit(void*, const automaton::SinglePopDPDA& automaton) const;
	void Visit(void*, const automaton::InputDrivenNPDA& automaton) const;
	void Visit(void*, const automaton::VisiblyPushdownDPDA& automaton) const;
	void Visit(void*, const automaton::VisiblyPushdownNPDA& automaton) const;
	void Visit(void*, const automaton::RealTimeHeightDeterministicDPDA& automaton) const;
	void Visit(void*, const automaton::RealTimeHeightDeterministicNPDA& automaton) const;
	void Visit(void*, const automaton::NPDA& automaton) const;
	void Visit(void*, const automaton::SinglePopNPDA& automaton) const;
	void Visit(void*, const automaton::OneTapeDTM& automaton) const;

	void Visit(void*, const grammar::LeftLG& grammar) const;
	void Visit(void*, const grammar::LeftRG& grammar) const;
	void Visit(void*, const grammar::RightLG& grammar) const;
	void Visit(void*, const grammar::RightRG& grammar) const;
	void Visit(void*, const grammar::LG& grammar) const;
	void Visit(void*, const grammar::CFG& grammar) const;
	void Visit(void*, const grammar::EpsilonFreeCFG& grammar) const;
	void Visit(void*, const grammar::CNF& grammar) const;
	void Visit(void*, const grammar::GNF& grammar) const;
	void Visit(void*, const grammar::CSG& grammar) const;
	void Visit(void*, const grammar::NonContractingGrammar& grammar) const;
	void Visit(void*, const grammar::ContextPreservingUnrestrictedGrammar& grammar) const;
	void Visit(void*, const grammar::UnrestrictedGrammar& grammar) const;

	void Visit(void*, const label::PrimitiveLabel& label) const;
	void Visit(void*, const label::HexavigesimalLabel& label) const;
	void Visit(void*, const label::ObjectLabel& label) const;
	void Visit(void*, const label::LabelSetLabel& label) const;
	void Visit(void*, const label::LabelPairLabel& label) const;

	void Visit(void*, const regexp::UnboundedRegExp& regexp) const;
	void Visit(void*, const regexp::FormalRegExp& regexp) const;

	void Visit(void*, const string::LinearString& string) const;
	void Visit(void*, const string::CyclicString& string) const;
	void Visit(void*, const string::Epsilon& string) const;

	void Visit(void*, const container::ObjectsSet& set) const;
	void Visit(void*, const container::ObjectsVector& set) const;
	void Visit(void*, const container::ObjectsPair& set) const;
	void Visit(void*, const container::ObjectsMap& set) const;

	void Visit(void*, const primitive::String& primitive) const;
	void Visit(void*, const primitive::Integer& primitive) const;
	void Visit(void*, const primitive::Character& primitive) const;

public:
	static const label::LabelToXMLComposer labelComposer;
	static const alphabet::SymbolToXMLComposer symbolComposer;
	static const regexp::RegExpToXMLComposer regexpComposer;
	static const string::StringToXMLComposer stringComposer;
	static const automaton::AutomatonToXMLComposer automatonComposer;
	static const grammar::GrammarToXMLComposer grammarComposer;
	static const exception::ExceptionToXMLComposer exceptionComposer;
	static const ObjectToXMLComposer objectComposer;
	static const container::ContainerToXMLComposer containerComposer;
	static const primitive::PrimitiveToXMLComposer primitiveComposer;

	static const ToXMLComposers toXMLComposers;
};

template<typename T>
std::set<T> api<std::set<T>>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::containerParser.parseSet<T>(input);
}

template<typename T>
std::list<sax::Token> api<std::set<T>>::compose(const std::set<T>& input) {
	return ToXMLComposers::containerComposer.compose<T>(input);
}

template<typename T>
std::vector<T> api<std::vector<T>>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::containerParser.parseVector<T>(input);
}

template<typename T>
std::list<sax::Token> api<std::vector<T>>::compose(const std::vector<T>& input) {
	return ToXMLComposers::containerComposer.compose<T>(input);
}

template<typename T, typename R>
std::pair<T, R> api<std::pair<T, R>>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::containerParser.parsePair<T, R>(input);
}

template<typename T, typename R>
std::list<sax::Token> api<std::pair<T, R>>::compose(const std::pair<T, R>& input) {
	return ToXMLComposers::containerComposer.compose<T, R>(input);
}

template<typename T, typename R>
std::map<T, R> api<std::map<T, R>>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::containerParser.parseMap<T, R>(input);
}

template<typename T, typename R>
std::list<sax::Token> api<std::map<T, R>>::compose(const std::map<T, R>& input) {
	return ToXMLComposers::containerComposer.compose<T, R>(input);
}

} /* namespace alib */

#endif /* FROM_XML_PARSERS_HPP_ */
