/*
 * Symbol.cpp
 *
 *  Created on: Apr 16, 2013
 *      Author: Jan Travnicek
 */

#include "Symbol.h"
#include "../label/NextLabel.h"
#include "LabeledSymbol.h"
#include <climits>
#include "../exception/AlibException.h"

namespace alphabet {

alphabet::Symbol createUniqueSymbol(const alphabet::Symbol& base, const std::set<alphabet::Symbol>& terminalAlphabet, const std::set<alphabet::Symbol>& nonterminalAlphabet) {
	label::NextLabel nextLabelCreator;

	if(alib::ObjectBase::typeId<LabeledSymbol>() != base.getData().selfTypeId())
		throw exception::AlibException("Could not create unique symbol with nonlabeled base symbol " + (std::string) base + "." );

	label::Label nextLabel = static_cast<const alphabet::LabeledSymbol&>(base.getData()).getLabel();

	int i = 0;
	do {
		nextLabel = nextLabelCreator.nextLabel(nextLabel);
		alphabet::Symbol attempt = alphabet::Symbol(alphabet::LabeledSymbol(nextLabel));
		if(terminalAlphabet.count(attempt) == 0 && nonterminalAlphabet.count(attempt) == 0)
			return attempt;
	} while(++i < INT_MAX);

	throw exception::AlibException("Could not create unique symbol with base symbol " + (std::string) base + "." );
}

alphabet::Symbol symbolFrom( int number ) {
	return alphabet::Symbol { alphabet::LabeledSymbol { label::labelFrom ( number ) } };
}

alphabet::Symbol symbolFrom( char character ) {
	return alphabet::Symbol { alphabet::LabeledSymbol { label::labelFrom ( character ) } };
}

alphabet::Symbol symbolFrom( std::string string ) {
	return alphabet::Symbol { alphabet::LabeledSymbol { label::labelFrom ( string ) } };
}

} /* namespace alphabet */

