/*
 * SymbolFromStringLexer.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#include "SymbolFromStringLexer.h"
#include "../label/LabelFromStringLexer.h"

namespace alphabet {

SymbolFromStringLexer::SymbolFromStringLexer(std::istream& in) : m_In(in) {
	m_Current.type = TokenType::ERROR;
	m_Current.value = "";
}

SymbolFromStringLexer& SymbolFromStringLexer::next() {
	if(m_Current.type == TokenType::TEOF) return *this;
	char character;
	m_Current.value = "";

L0:
	character = m_In.get();
	if(m_In.eof()) {
		m_Current.type = TokenType::TEOF;
		return *this;
	} else if(character == ' ' || character == '\n' || character == '\t') {
		goto L0;
	} else if(character == '#') {
		goto L1;
	} else {
		m_In.putback(character);
		while(!m_Current.value.empty()) {
			m_In.putback(m_Current.value.back());
			m_Current.value.pop_back();
		}
		m_In.clear();
		m_Current.type = TokenType::ERROR;
		return *this;
	}
L1:
	character = m_In.get();
	if(m_In.eof()) {
		m_Current.type = TokenType::TEOF;
		return *this;
	} else if(character == 'B') {
		m_Current.type = TokenType::BLANK;
		return *this;
	} else if(character == 'Z') {
		m_Current.type = TokenType::BOTTOM;
		return *this;
	} else if(character == 'E') {
		m_Current.type = TokenType::END;
		return *this;
	} else {
		m_In.putback(character);
		while(!m_Current.value.empty()) {
			m_In.putback(m_Current.value.back());
			m_Current.value.pop_back();
		}
		m_In.clear();
		m_Current.type = TokenType::ERROR;
		return *this;
	}
}

SymbolFromStringLexer::Token SymbolFromStringLexer::token() {
	return m_Current;
}

} /* namespace alphabet */
