/*
 * SymbolFromXMLParser.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#include "SymbolFromXMLParser.h"

#include "../sax/ParserException.h"
#include "../label/Label.h"

#include "../Api.hpp"

namespace alphabet {

Symbol SymbolFromXMLParser::parseSymbol(std::list<sax::Token>& input) const {
	return parseSymbol(input, std::set<FEATURES>({FEATURES::LABELED, FEATURES::BLANK, FEATURES::BOTTOM, FEATURES::END}));
}

Symbol SymbolFromXMLParser::parseSymbol(std::list<sax::Token>& input, const std::set<FEATURES>& features) const {
	if(isToken(input, sax::Token::TokenType::START_ELEMENT, "LabeledSymbol")) {
		if(!features.count(FEATURES::LABELED)) throw exception::AlibException();
		return Symbol(parseLabeledSymbol(input));
	} else if(isToken(input, sax::Token::TokenType::START_ELEMENT, "BlankSymbol")) {
		if(!features.count(FEATURES::BLANK)) throw exception::AlibException();
		return Symbol(parseBlankSymbol(input));
	} else if(isToken(input, sax::Token::TokenType::START_ELEMENT, "BottomOfTheStackSymbol")) {
		if(!features.count(FEATURES::BOTTOM)) throw exception::AlibException();
		return Symbol(parseBottomOfTheStackSymbol(input));
	} else if(isToken(input, sax::Token::TokenType::START_ELEMENT, "EndSymbol")) {
		if(!features.count(FEATURES::END)) throw exception::AlibException();
		return Symbol(parseEndSymbol(input));
	} else {
		throw sax::ParserException(sax::Token("LabeledSymbol, BlankSymbol, BottomOfTheStackSymbol, EndSymbol", sax::Token::TokenType::START_ELEMENT), input.front());
	}
}

bool SymbolFromXMLParser::first(std::list<sax::Token>& input) const {
	if(isToken(input, sax::Token::TokenType::START_ELEMENT, "LabeledSymbol") || isToken(input, sax::Token::TokenType::START_ELEMENT, "BlankSymbol") || isToken(input, sax::Token::TokenType::START_ELEMENT, "BottomOfTheStackSymbol") || isToken(input, sax::Token::TokenType::START_ELEMENT, "EndSymbol")) {
		return true;
	} else {
		return false;
	}
}

LabeledSymbol SymbolFromXMLParser::parseLabeledSymbol(std::list<sax::Token>& input) const {
	popToken(input, sax::Token::TokenType::START_ELEMENT, "LabeledSymbol");
	label::Label data = alib::api<label::Label>::parse(input);
	popToken(input, sax::Token::TokenType::END_ELEMENT, "LabeledSymbol");
	return LabeledSymbol(data);
}

BlankSymbol SymbolFromXMLParser::parseBlankSymbol(std::list<sax::Token>& input) const {
	popToken(input, sax::Token::TokenType::START_ELEMENT, "BlankSymbol");
	popToken(input, sax::Token::TokenType::END_ELEMENT, "BlankSymbol");
	return BlankSymbol();
}

BottomOfTheStackSymbol SymbolFromXMLParser::parseBottomOfTheStackSymbol(std::list<sax::Token>& input) const {
	popToken(input, sax::Token::TokenType::START_ELEMENT, "BottomOfTheStackSymbol");
	popToken(input, sax::Token::TokenType::END_ELEMENT, "BottomOfTheStackSymbol");
	return BottomOfTheStackSymbol();
}

EndSymbol SymbolFromXMLParser::parseEndSymbol(std::list<sax::Token>& input) const {
	popToken(input, sax::Token::TokenType::START_ELEMENT, "EndSymbol");
	popToken(input, sax::Token::TokenType::END_ELEMENT, "EndSymbol");
	return EndSymbol();
}

} /* namespace alphabet */
