/*
 * AutomatonBase.h
 *
 *  Created on: Apr 10, 2013
 *      Author: Jan Travnicek
 */

#ifndef AUTOMATON_BASE_H_
#define AUTOMATON_BASE_H_

#include "../common/base.hpp"
#include "../object/ObjectBase.h"
#include "../exception/AlibException.h"

namespace automaton {

class AutomatonBase;

typedef std::acceptor_base<AutomatonBase,
			automaton::DFA, automaton::NFA, automaton::MultiInitialStateNFA, automaton::EpsilonNFA, automaton::CompactNFA, automaton::ExtendedNFA, automaton::DPDA, automaton::SinglePopDPDA, automaton::InputDrivenNPDA, automaton::VisiblyPushdownDPDA, automaton::VisiblyPushdownNPDA, automaton::RealTimeHeightDeterministicDPDA, automaton::RealTimeHeightDeterministicNPDA, automaton::NPDA, automaton::SinglePopNPDA, automaton::OneTapeDTM
	> VisitableAutomatonBase;

/**
 * Abstract base class for all automata.
 */
class AutomatonBase : public alib::ObjectBase, public VisitableAutomatonBase {
public:
	using VisitableAutomatonBase::Accept;
	using alib::VisitableObjectBase::Accept;

	virtual AutomatonBase* clone() const = 0;
	virtual AutomatonBase* plunder() && = 0;

};

class VisitableConstFSMBase : public VisitableAutomatonBase::const_visitor_type {

	void Visit(void*, const DPDA&) const {
		throw exception::AlibException("Unsupported automaton type DPDA");
	}

	void Visit(void*, const SinglePopDPDA&) const {
		throw exception::AlibException("Unsupported automaton type SinglePopDPDA");
	}

	void Visit(void*, const InputDrivenNPDA&) const {
		throw exception::AlibException("Unsupported automaton type InputDrivenNPDA");
	}

	void Visit(void*, const VisiblyPushdownDPDA&) const {
		throw exception::AlibException("Unsupported automaton type VisiblyPushdownDPDA");
	}

	void Visit(void*, const VisiblyPushdownNPDA&) const {
		throw exception::AlibException("Unsupported automaton type VisiblyPushdownNPDA");
	}

	void Visit(void*, const RealTimeHeightDeterministicDPDA&) const {
		throw exception::AlibException("Unsupported automaton type RealTimeHeightDeterministicDPDA");
	}

	void Visit(void*, const RealTimeHeightDeterministicNPDA&) const {
		throw exception::AlibException("Unsupported automaton type RealTimeHeightDeterministicNPDA");
	}

	void Visit(void*, const NPDA&) const {
		throw exception::AlibException("Unsupported automaton type NPDA");
	}

	void Visit(void*, const SinglePopNPDA&) const {
		throw exception::AlibException("Unsupported automaton type SinglePopNPDA");
	}

	void Visit(void*, const OneTapeDTM&) const {
		throw exception::AlibException("Unsupported automaton type OneTapeDTM");
	}
};

class VisitableConstPDABase : public VisitableAutomatonBase::const_visitor_type {

	void Visit(void*, const EpsilonNFA&) const {
		throw exception::AlibException("Unsupported automaton type EpsilonNFA");
	}

	void Visit(void*, const MultiInitialStateNFA&) const {
		throw exception::AlibException("Unsupported automaton type MultiInitialStateNFA");
	}

	void Visit(void*, const NFA&) const {
		throw exception::AlibException("Unsupported automaton type NFA");
	}

	void Visit(void*, const DFA&) const {
		throw exception::AlibException("Unsupported automaton type DFA");
	}

	void Visit(void*, const ExtendedNFA&) const {
		throw exception::AlibException("Unsupported automaton type ExtendedDFA");
	}

	void Visit(void*, const CompactNFA&) const {
		throw exception::AlibException("Unsupported automaton type CompactNFA");
	}

	void Visit(void*, const OneTapeDTM&) const {
		throw exception::AlibException("Unsupported automaton type OneTapeDTM");
	}

};

class VisitableConstNondeterministicAutomatonBase : public VisitableAutomatonBase::const_visitor_type {

	void Visit(void*, const DFA&) const {
		throw exception::AlibException("Unsupported automaton type DFA");
	}

	void Visit(void*, const DPDA&) const {
		throw exception::AlibException("Unsupported automaton type DPDA");
	}

	void Visit(void*, const SinglePopDPDA&) const {
		throw exception::AlibException("Unsupported automaton type SinglePopDPDA");
	}

	void Visit(void*, const VisiblyPushdownDPDA&) const {
		throw exception::AlibException("Unsupported automaton type VisiblyPushdownDPDA");
	}

	void Visit(void*, const RealTimeHeightDeterministicDPDA&) const {
		throw exception::AlibException("Unsupported automaton type RealTimeHeightDeterministicDPDA");
	}

	void Visit(void*, const OneTapeDTM&) const {
		throw exception::AlibException("Unsupported automaton type OneTapeDTM");
	}
};

class VisitableConstDeterministicAutomatonBase : public VisitableAutomatonBase::const_visitor_type {

	void Visit(void*, const EpsilonNFA&) const {
		throw exception::AlibException("Unsupported automaton type EpsilonNFA");
	}

	void Visit(void*, const MultiInitialStateNFA&) const {
		throw exception::AlibException("Unsupported automaton type MultiInitialStateNFA");
	}

	void Visit(void*, const NFA&) const {
		throw exception::AlibException("Unsupported automaton type NFA");
	}

	void Visit(void*, const ExtendedNFA&) const {
		throw exception::AlibException("Unsupported automaton type ExtendedDFA");
	}

	void Visit(void*, const CompactNFA&) const {
		throw exception::AlibException("Unsupported automaton type CompactNFA");
	}

	void Visit(void*, const InputDrivenNPDA&) const {
		throw exception::AlibException("Unsupported automaton type InputDrivenNPDA");
	}

	void Visit(void*, const VisiblyPushdownNPDA&) const {
		throw exception::AlibException("Unsupported automaton type VisiblyPushdownNPDA");
	}

	void Visit(void*, const RealTimeHeightDeterministicNPDA&) const {
		throw exception::AlibException("Unsupported automaton type RealTimeHeightDeterministicNPDA");
	}

	void Visit(void*, const NPDA&) const {
		throw exception::AlibException("Unsupported automaton type NPDA");
	}

	void Visit(void*, const SinglePopNPDA&) const {
		throw exception::AlibException("Unsupported automaton type SinglePopNPDA");
	}
};

} /* namespace automaton */

#endif /* AUTOMATON_BASE_H_ */

