/*
 * AutomatonToXMLComposer.cpp
 *
 *  Created on: Nov 11, 2013
 *      Author: Jan Travnicek
 */

#include "AutomatonToXMLComposer.h"
#include "../alphabet/BlankSymbol.h"

#include "../Api.hpp"

namespace automaton {

void AutomatonToXMLComposer::composeStates(std::list<sax::Token>& out, const std::set<State>& states) const {
	out.push_back(sax::Token("states", sax::Token::TokenType::START_ELEMENT));
	for (const auto& state : states) {
		out.splice(out.end(), alib::api<label::Label>::compose(state.getName()));
	}
	out.push_back(sax::Token("states", sax::Token::TokenType::END_ELEMENT));
}

void AutomatonToXMLComposer::composeInputAlphabet(std::list<sax::Token>& out, const std::set<alphabet::Symbol>& symbols) const {
	out.push_back(sax::Token("inputAlphabet", sax::Token::TokenType::START_ELEMENT));
	for (const auto& symbol : symbols) {
		out.splice(out.end(), alib::api<alphabet::Symbol>::compose(symbol));
	}
	out.push_back(sax::Token("inputAlphabet", sax::Token::TokenType::END_ELEMENT));
}

void AutomatonToXMLComposer::composeCallInputAlphabet(std::list<sax::Token>& out, const std::set<alphabet::Symbol>& symbols) const {
	out.push_back(sax::Token("callInputAlphabet", sax::Token::TokenType::START_ELEMENT));
	for (const auto& symbol : symbols) {
		out.splice(out.end(), alib::api<alphabet::Symbol>::compose(symbol));
	}
	out.push_back(sax::Token("callInputAlphabet", sax::Token::TokenType::END_ELEMENT));
}

void AutomatonToXMLComposer::composeReturnInputAlphabet(std::list<sax::Token>& out, const std::set<alphabet::Symbol>& symbols) const {
	out.push_back(sax::Token("returnInputAlphabet", sax::Token::TokenType::START_ELEMENT));
	for (const auto& symbol : symbols) {
		out.splice(out.end(), alib::api<alphabet::Symbol>::compose(symbol));
	}
	out.push_back(sax::Token("returnInputAlphabet", sax::Token::TokenType::END_ELEMENT));
}

void AutomatonToXMLComposer::composeLocalInputAlphabet(std::list<sax::Token>& out, const std::set<alphabet::Symbol>& symbols) const {
	out.push_back(sax::Token("localInputAlphabet", sax::Token::TokenType::START_ELEMENT));
	for (const auto& symbol : symbols) {
		out.splice(out.end(), alib::api<alphabet::Symbol>::compose(symbol));
	}
	out.push_back(sax::Token("localInputAlphabet", sax::Token::TokenType::END_ELEMENT));
}

void AutomatonToXMLComposer::composeInitialStates(std::list<sax::Token>& out, const std::set<State>& states) const {
	out.push_back(sax::Token("initialStates", sax::Token::TokenType::START_ELEMENT));
	for (const auto& state : states) {
		out.splice(out.end(), alib::api<label::Label>::compose(state.getName()));
	}
	out.push_back(sax::Token("initialStates", sax::Token::TokenType::END_ELEMENT));
}

void AutomatonToXMLComposer::composeInitialState(std::list<sax::Token>& out, const State& state) const {
	out.push_back(sax::Token("initialState", sax::Token::TokenType::START_ELEMENT));
	out.splice(out.end(), alib::api<label::Label>::compose(state.getName()));
	out.push_back(sax::Token("initialState", sax::Token::TokenType::END_ELEMENT));
}

void AutomatonToXMLComposer::composeFinalStates(std::list<sax::Token>& out, const std::set<State>& states) const {
	out.push_back(sax::Token("finalStates", sax::Token::TokenType::START_ELEMENT));
	for (const auto& state : states) {
		out.splice(out.end(), alib::api<label::Label>::compose(state.getName()));
	}
	out.push_back(sax::Token("finalStates", sax::Token::TokenType::END_ELEMENT));
}

void AutomatonToXMLComposer::composeStackAlphabet(std::list<sax::Token>& out, const std::set<alphabet::Symbol>& symbols) const {
	out.push_back(sax::Token("stackAlphabet", sax::Token::TokenType::START_ELEMENT));
	for (const auto& symbol : symbols) {
		out.splice(out.end(), alib::api<alphabet::Symbol>::compose(symbol));
	}
	out.push_back(sax::Token("stackAlphabet", sax::Token::TokenType::END_ELEMENT));
}

void AutomatonToXMLComposer::composeInitialStackSymbols(std::list<sax::Token>& out, const std::set<alphabet::Symbol>& symbols) const {
	out.push_back(sax::Token("initialStackSymbols", sax::Token::TokenType::START_ELEMENT));
	for (const auto& symbol : symbols) {
		out.splice(out.end(), alib::api<alphabet::Symbol>::compose(symbol));
	}
	out.push_back(sax::Token("initialStackSymbols", sax::Token::TokenType::END_ELEMENT));
}

void AutomatonToXMLComposer::composeInitialStackSymbol(std::list<sax::Token>& out, const alphabet::Symbol& symbol) const {
	out.push_back(sax::Token("initialStackSymbol", sax::Token::TokenType::START_ELEMENT));
	out.splice(out.end(), alib::api<alphabet::Symbol>::compose(symbol));
	out.push_back(sax::Token("initialStackSymbol", sax::Token::TokenType::END_ELEMENT));
}

void AutomatonToXMLComposer::composeTapeAlphabet(std::list<sax::Token>& out, const std::set<alphabet::Symbol>& symbols) const {
	out.push_back(sax::Token("tapeAlphabet", sax::Token::TokenType::START_ELEMENT));
	for (const auto& symbol : symbols) {
		out.splice(out.end(), alib::api<alphabet::Symbol>::compose(symbol));
	}
	out.push_back(sax::Token("tapeAlphabet", sax::Token::TokenType::END_ELEMENT));
}

void AutomatonToXMLComposer::composeBlankSymbol(std::list<sax::Token>& out, const alphabet::Symbol& symbol) const {
	out.push_back(sax::Token("blankSymbol", sax::Token::TokenType::START_ELEMENT));
	out.splice(out.end(), alib::api<alphabet::Symbol>::compose(symbol));
	out.push_back(sax::Token("blankSymbol", sax::Token::TokenType::END_ELEMENT));
}

void AutomatonToXMLComposer::composeBottomOfTheStackSymbol(std::list<sax::Token>& out, const alphabet::Symbol& symbol) const {
	out.push_back(sax::Token("bottomOfTheStackSymbol", sax::Token::TokenType::START_ELEMENT));
	out.splice(out.end(), alib::api<alphabet::Symbol>::compose(symbol));
	out.push_back(sax::Token("bottomOfTheStackSymbol", sax::Token::TokenType::END_ELEMENT));
}

void AutomatonToXMLComposer::composeInputToPushdownStoreOperation(std::list<sax::Token>& out, const automaton::InputDrivenNPDA& automaton) const {
	
	out.push_back(sax::Token("inputToPushdownStoreOperations", sax::Token::TokenType::START_ELEMENT));
	for(const auto& pushdownStoreOperation : automaton.getPushdownStoreOperations()) {
		out.push_back(sax::Token("operation", sax::Token::TokenType::START_ELEMENT));
	
		out.splice(out.end(), alib::api<alphabet::Symbol>::compose(pushdownStoreOperation.first));
		
		composeTransitionPop(out, pushdownStoreOperation.second.first);
		composeTransitionPush(out, pushdownStoreOperation.second.second);

		out.push_back(sax::Token("operation", sax::Token::TokenType::END_ELEMENT));
	}
	out.push_back(sax::Token("inputToPushdownStoreOperations", sax::Token::TokenType::END_ELEMENT));
}

void AutomatonToXMLComposer::composeTransitions(std::list<sax::Token>& out, const DFA& automaton) const {
	out.push_back(sax::Token("transitions", sax::Token::TokenType::START_ELEMENT));
	for(const auto& transition : automaton.getTransitions()) {
		out.push_back(sax::Token("transition", sax::Token::TokenType::START_ELEMENT));

		composeTransitionFrom(out, transition.first.first);
		composeTransitionInputSymbol(out, transition.first.second);
		composeTransitionTo(out, transition.second);

		out.push_back(sax::Token("transition", sax::Token::TokenType::END_ELEMENT));
	}

	out.push_back(sax::Token("transitions", sax::Token::TokenType::END_ELEMENT));
}

void AutomatonToXMLComposer::composeTransitions(std::list<sax::Token>& out, const NFA& automaton) const {
	out.push_back(sax::Token("transitions", sax::Token::TokenType::START_ELEMENT));
	for(const auto& transition : automaton.getTransitions()) {
		for(const auto& targetState: transition.second) {
			out.push_back(sax::Token("transition", sax::Token::TokenType::START_ELEMENT));

			composeTransitionFrom(out, transition.first.first);
			composeTransitionInputSymbol(out, transition.first.second);
			composeTransitionTo(out, targetState);

			out.push_back(sax::Token("transition", sax::Token::TokenType::END_ELEMENT));
		}
	}

	out.push_back(sax::Token("transitions", sax::Token::TokenType::END_ELEMENT));
}

void AutomatonToXMLComposer::composeTransitions(std::list<sax::Token>& out, const MultiInitialStateNFA& automaton) const {
	out.push_back(sax::Token("transitions", sax::Token::TokenType::START_ELEMENT));
	for(const auto& transition : automaton.getTransitions()) {
		for(const auto& targetState: transition.second) {
			out.push_back(sax::Token("transition", sax::Token::TokenType::START_ELEMENT));

			composeTransitionFrom(out, transition.first.first);
			composeTransitionInputSymbol(out, transition.first.second);
			composeTransitionTo(out, targetState);

			out.push_back(sax::Token("transition", sax::Token::TokenType::END_ELEMENT));
		}
	}

	out.push_back(sax::Token("transitions", sax::Token::TokenType::END_ELEMENT));
}

void AutomatonToXMLComposer::composeTransitions(std::list<sax::Token>& out, const EpsilonNFA& automaton) const {
	out.push_back(sax::Token("transitions", sax::Token::TokenType::START_ELEMENT));
	for(const auto& transition : automaton.getTransitions()) {
		for(const auto& targetState: transition.second) {
			out.push_back(sax::Token("transition", sax::Token::TokenType::START_ELEMENT));

			composeTransitionFrom(out, transition.first.first);
			composeTransitionInputEpsilonSymbol(out, transition.first.second);
			composeTransitionTo(out, targetState);

			out.push_back(sax::Token("transition", sax::Token::TokenType::END_ELEMENT));
		}
	}

	out.push_back(sax::Token("transitions", sax::Token::TokenType::END_ELEMENT));
}

void AutomatonToXMLComposer::composeTransitions(std::list<sax::Token>& out, const CompactNFA& automaton) const {
	out.push_back(sax::Token("transitions", sax::Token::TokenType::START_ELEMENT));
	for(const auto& transition : automaton.getTransitions()) {
		for(const auto& targetState: transition.second) {
			out.push_back(sax::Token("transition", sax::Token::TokenType::START_ELEMENT));

			composeTransitionFrom(out, transition.first.first);
			composeTransitionInputString(out, transition.first.second);
			composeTransitionTo(out, targetState);

			out.push_back(sax::Token("transition", sax::Token::TokenType::END_ELEMENT));
		}
	}

	out.push_back(sax::Token("transitions", sax::Token::TokenType::END_ELEMENT));
}

void AutomatonToXMLComposer::composeTransitions(std::list<sax::Token>& out, const ExtendedNFA& automaton) const {
	out.push_back(sax::Token("transitions", sax::Token::TokenType::START_ELEMENT));
	for(const auto& transition : automaton.getTransitions()) {
		for(const auto& targetState: transition.second) {
			out.push_back(sax::Token("transition", sax::Token::TokenType::START_ELEMENT));

			composeTransitionFrom(out, transition.first.first);
			composeTransitionInputRegexp(out, transition.first.second);
			composeTransitionTo(out, targetState);

			out.push_back(sax::Token("transition", sax::Token::TokenType::END_ELEMENT));
		}
	}

	out.push_back(sax::Token("transitions", sax::Token::TokenType::END_ELEMENT));
}

void AutomatonToXMLComposer::composeTransitions(std::list<sax::Token>& out, const DPDA& automaton) const {
	out.push_back(sax::Token("transitions", sax::Token::TokenType::START_ELEMENT));
	for(const auto& transition : automaton.getTransitions()) {
		out.push_back(sax::Token("transition", sax::Token::TokenType::START_ELEMENT));

		composeTransitionFrom(out, std::get<0>(transition.first));
		composeTransitionInputEpsilonSymbol(out, std::get<1>(transition.first));
		composeTransitionPop(out, std::get<2>(transition.first));
		composeTransitionTo(out, transition.second.first);
		composeTransitionPush(out, transition.second.second);

		out.push_back(sax::Token("transition", sax::Token::TokenType::END_ELEMENT));
	}

	out.push_back(sax::Token("transitions", sax::Token::TokenType::END_ELEMENT));
}

void AutomatonToXMLComposer::composeTransitions(std::list<sax::Token>& out, const SinglePopDPDA& automaton) const {
	out.push_back(sax::Token("transitions", sax::Token::TokenType::START_ELEMENT));
	for(const auto& transition : automaton.getTransitions()) {
		out.push_back(sax::Token("transition", sax::Token::TokenType::START_ELEMENT));

		composeTransitionFrom(out, std::get<0>(transition.first));
		composeTransitionInputEpsilonSymbol(out, std::get<1>(transition.first));
		composeTransitionSinglePop(out, std::get<2>(transition.first));
		composeTransitionTo(out, transition.second.first);
		composeTransitionPush(out, transition.second.second);

		out.push_back(sax::Token("transition", sax::Token::TokenType::END_ELEMENT));
	}

	out.push_back(sax::Token("transitions", sax::Token::TokenType::END_ELEMENT));
}

void AutomatonToXMLComposer::composeTransitions(std::list<sax::Token>& out, const InputDrivenNPDA& automaton) const {
	out.push_back(sax::Token("transitions", sax::Token::TokenType::START_ELEMENT));
	for(const auto& transition : automaton.getTransitions()) {
		for(const auto& target: transition.second) {
			out.push_back(sax::Token("transition", sax::Token::TokenType::START_ELEMENT));

			composeTransitionFrom(out, transition.first.first);
			composeTransitionInputSymbol(out, transition.first.second);
			composeTransitionTo(out, target);

			out.push_back(sax::Token("transition", sax::Token::TokenType::END_ELEMENT));
		}
	}

	out.push_back(sax::Token("transitions", sax::Token::TokenType::END_ELEMENT));
}

void AutomatonToXMLComposer::composeTransitions(std::list<sax::Token>& out, const VisiblyPushdownDPDA& automaton) const {
	out.push_back(sax::Token("transitions", sax::Token::TokenType::START_ELEMENT));
	for(const auto& transition : automaton.getCallTransitions()) {
		out.push_back(sax::Token("callTransition", sax::Token::TokenType::START_ELEMENT));

		composeTransitionFrom(out, transition.first.first);
		composeTransitionInputSymbol(out, transition.first.second);
		composeTransitionTo(out, transition.second.first);
		composeTransitionSinglePush(out, transition.second.second);

		out.push_back(sax::Token("callTransition", sax::Token::TokenType::END_ELEMENT));
	}
	for(const auto& transition : automaton.getReturnTransitions()) {
		out.push_back(sax::Token("returnTransition", sax::Token::TokenType::START_ELEMENT));

		composeTransitionFrom(out, std::get<0>(transition.first));
		composeTransitionInputSymbol(out, std::get<1>(transition.first));
		composeTransitionSinglePop(out, std::get<2>(transition.first));
		composeTransitionTo(out, transition.second);

		out.push_back(sax::Token("returnTransition", sax::Token::TokenType::END_ELEMENT));
	}
	for(const auto& transition : automaton.getLocalTransitions()) {
		out.push_back(sax::Token("localTransition", sax::Token::TokenType::START_ELEMENT));

		composeTransitionFrom(out, transition.first.first);
		composeTransitionInputSymbol(out, transition.first.second);
		composeTransitionTo(out, transition.second);

		out.push_back(sax::Token("localTransition", sax::Token::TokenType::END_ELEMENT));
	}
	out.push_back(sax::Token("transitions", sax::Token::TokenType::END_ELEMENT));
}

void AutomatonToXMLComposer::composeTransitions(std::list<sax::Token>& out, const VisiblyPushdownNPDA& automaton) const {
	out.push_back(sax::Token("transitions", sax::Token::TokenType::START_ELEMENT));
	for(const auto& transition : automaton.getCallTransitions()) {
		for(const auto& target: transition.second) {
			out.push_back(sax::Token("callTransition", sax::Token::TokenType::START_ELEMENT));

			composeTransitionFrom(out, transition.first.first);
			composeTransitionInputSymbol(out, transition.first.second);
			composeTransitionTo(out, target.first);
			composeTransitionSinglePush(out, target.second);

			out.push_back(sax::Token("callTransition", sax::Token::TokenType::END_ELEMENT));
		}
	}
	for(const auto& transition : automaton.getReturnTransitions()) {
		for(const auto& target: transition.second) {
			out.push_back(sax::Token("returnTransition", sax::Token::TokenType::START_ELEMENT));

			composeTransitionFrom(out, std::get<0>(transition.first));
			composeTransitionInputSymbol(out, std::get<1>(transition.first));
			composeTransitionSinglePop(out, std::get<2>(transition.first));
			composeTransitionTo(out, target);

			out.push_back(sax::Token("returnTransition", sax::Token::TokenType::END_ELEMENT));
		}
	}
	for(const auto& transition : automaton.getLocalTransitions()) {
		for(const auto& target: transition.second) {
			out.push_back(sax::Token("localTransition", sax::Token::TokenType::START_ELEMENT));

			composeTransitionFrom(out, transition.first.first);
			composeTransitionInputSymbol(out, transition.first.second);
			composeTransitionTo(out, target);

			out.push_back(sax::Token("localTransition", sax::Token::TokenType::END_ELEMENT));
		}
	}
	out.push_back(sax::Token("transitions", sax::Token::TokenType::END_ELEMENT));
}

void AutomatonToXMLComposer::composeTransitions(std::list<sax::Token>& out, const RealTimeHeightDeterministicDPDA& automaton) const {
	out.push_back(sax::Token("transitions", sax::Token::TokenType::START_ELEMENT));
	for(const auto& transition : automaton.getCallTransitions()) {
		out.push_back(sax::Token("callTransition", sax::Token::TokenType::START_ELEMENT));

		composeTransitionFrom(out, transition.first.first);
		composeTransitionInputEpsilonSymbol(out, transition.first.second);
		composeTransitionTo(out, transition.second.first);
		composeTransitionSinglePush(out, transition.second.second);

		out.push_back(sax::Token("callTransition", sax::Token::TokenType::END_ELEMENT));
	}
	for(const auto& transition : automaton.getReturnTransitions()) {
		out.push_back(sax::Token("returnTransition", sax::Token::TokenType::START_ELEMENT));

		composeTransitionFrom(out, std::get<0>(transition.first));
		composeTransitionInputEpsilonSymbol(out, std::get<1>(transition.first));
		composeTransitionSinglePop(out, std::get<2>(transition.first));
		composeTransitionTo(out, transition.second);

		out.push_back(sax::Token("returnTransition", sax::Token::TokenType::END_ELEMENT));
	}
	for(const auto& transition : automaton.getLocalTransitions()) {
		out.push_back(sax::Token("localTransition", sax::Token::TokenType::START_ELEMENT));

		composeTransitionFrom(out, transition.first.first);
		composeTransitionInputEpsilonSymbol(out, transition.first.second);
		composeTransitionTo(out, transition.second);

		out.push_back(sax::Token("localTransition", sax::Token::TokenType::END_ELEMENT));
	}
	out.push_back(sax::Token("transitions", sax::Token::TokenType::END_ELEMENT));
}

void AutomatonToXMLComposer::composeTransitions(std::list<sax::Token>& out, const RealTimeHeightDeterministicNPDA& automaton) const {
	out.push_back(sax::Token("transitions", sax::Token::TokenType::START_ELEMENT));
	for(const auto& transition : automaton.getCallTransitions()) {
		for(const auto& target: transition.second) {
			out.push_back(sax::Token("callTransition", sax::Token::TokenType::START_ELEMENT));

			composeTransitionFrom(out, transition.first.first);
			composeTransitionInputEpsilonSymbol(out, transition.first.second);
			composeTransitionTo(out, target.first);
			composeTransitionSinglePush(out, target.second);

			out.push_back(sax::Token("callTransition", sax::Token::TokenType::END_ELEMENT));
		}
	}
	for(const auto& transition : automaton.getReturnTransitions()) {
		for(const auto& target: transition.second) {
			out.push_back(sax::Token("returnTransition", sax::Token::TokenType::START_ELEMENT));

			composeTransitionFrom(out, std::get<0>(transition.first));
			composeTransitionInputEpsilonSymbol(out, std::get<1>(transition.first));
			composeTransitionSinglePop(out, std::get<2>(transition.first));
			composeTransitionTo(out, target);

			out.push_back(sax::Token("returnTransition", sax::Token::TokenType::END_ELEMENT));
		}
	}
	for(const auto& transition : automaton.getLocalTransitions()) {
		for(const auto& target: transition.second) {
			out.push_back(sax::Token("localTransition", sax::Token::TokenType::START_ELEMENT));

			composeTransitionFrom(out, transition.first.first);
			composeTransitionInputEpsilonSymbol(out, transition.first.second);
			composeTransitionTo(out, target);

			out.push_back(sax::Token("localTransition", sax::Token::TokenType::END_ELEMENT));
		}
	}
	out.push_back(sax::Token("transitions", sax::Token::TokenType::END_ELEMENT));
}

void AutomatonToXMLComposer::composeTransitions(std::list<sax::Token>& out, const NPDA& automaton) const {
	out.push_back(sax::Token("transitions", sax::Token::TokenType::START_ELEMENT));
	for(const auto& transition : automaton.getTransitions()) {
		for(const auto& target: transition.second) {
			out.push_back(sax::Token("transition", sax::Token::TokenType::START_ELEMENT));

			composeTransitionFrom(out, std::get<0>(transition.first));
			composeTransitionInputEpsilonSymbol(out, std::get<1>(transition.first));
			composeTransitionPop(out, std::get<2>(transition.first));
			composeTransitionTo(out, target.first);
			composeTransitionPush(out, target.second);

			out.push_back(sax::Token("transition", sax::Token::TokenType::END_ELEMENT));
		}
	}

	out.push_back(sax::Token("transitions", sax::Token::TokenType::END_ELEMENT));
}

void AutomatonToXMLComposer::composeTransitions(std::list<sax::Token>& out, const SinglePopNPDA& automaton) const {
	out.push_back(sax::Token("transitions", sax::Token::TokenType::START_ELEMENT));
	for(const auto& transition : automaton.getTransitions()) {
		for(const auto& target: transition.second) {
			out.push_back(sax::Token("transition", sax::Token::TokenType::START_ELEMENT));

			composeTransitionFrom(out, std::get<0>(transition.first));
			composeTransitionInputEpsilonSymbol(out, std::get<1>(transition.first));
			composeTransitionSinglePop(out, std::get<2>(transition.first));
			composeTransitionTo(out, target.first);
			composeTransitionPush(out, target.second);

			out.push_back(sax::Token("transition", sax::Token::TokenType::END_ELEMENT));
		}
	}

	out.push_back(sax::Token("transitions", sax::Token::TokenType::END_ELEMENT));
}

void AutomatonToXMLComposer::composeTransitions(std::list<sax::Token>& out, const OneTapeDTM& automaton) const {
	out.push_back(sax::Token("transitions", sax::Token::TokenType::START_ELEMENT));
	for(const auto& transition : automaton.getTransitions()) {
		out.push_back(sax::Token("transition", sax::Token::TokenType::START_ELEMENT));

		composeTransitionFrom(out, transition.first.first);
		composeTransitionInputSymbol(out, transition.first.second);
		composeTransitionTo(out, std::get<0>(transition.second));
		composeTransitionOutputSymbol(out, std::get<1>(transition.second));
		composeTransitionShift(out, std::get<2>(transition.second));

		out.push_back(sax::Token("transition", sax::Token::TokenType::END_ELEMENT));
	}

	out.push_back(sax::Token("transitions", sax::Token::TokenType::END_ELEMENT));
}

void AutomatonToXMLComposer::composeTransitionTo(std::list<sax::Token>& out, const State& state) const {
	out.push_back(sax::Token("to", sax::Token::TokenType::START_ELEMENT));
	out.splice(out.end(), alib::api<label::Label>::compose(state.getName()));
	out.push_back(sax::Token("to", sax::Token::TokenType::END_ELEMENT));
}

void AutomatonToXMLComposer::composeTransitionFrom(std::list<sax::Token>& out, const State& state) const {
	out.push_back(sax::Token("from", sax::Token::TokenType::START_ELEMENT));
	out.splice(out.end(), alib::api<label::Label>::compose(state.getName()));
	out.push_back(sax::Token("from", sax::Token::TokenType::END_ELEMENT));
}

void AutomatonToXMLComposer::composeTransitionShift(std::list<sax::Token>& out, const Shift shift) const {
	out.push_back(sax::Token("shift", sax::Token::TokenType::START_ELEMENT));
	out.push_back(sax::Token(SHIFT_NAMES [shift], sax::Token::TokenType::CHARACTER));
	out.push_back(sax::Token("shift", sax::Token::TokenType::END_ELEMENT));
}

void AutomatonToXMLComposer::composeTransitionPop(std::list<sax::Token>& out, const std::vector<alphabet::Symbol>& symbols) const {
	out.push_back(sax::Token("pop", sax::Token::TokenType::START_ELEMENT));
	for (const auto& symbol : symbols) {
		out.splice(out.end(), alib::api<alphabet::Symbol>::compose(symbol));
	}
	out.push_back(sax::Token("pop", sax::Token::TokenType::END_ELEMENT));
}

void AutomatonToXMLComposer::composeTransitionSinglePop(std::list<sax::Token>& out, const alphabet::Symbol& symbol) const {
	out.push_back(sax::Token("pop", sax::Token::TokenType::START_ELEMENT));
	out.splice(out.end(), alib::api<alphabet::Symbol>::compose(symbol));
	out.push_back(sax::Token("pop", sax::Token::TokenType::END_ELEMENT));
}

void AutomatonToXMLComposer::composeTransitionPush(std::list<sax::Token>& out, const std::vector<alphabet::Symbol>& symbols) const {
	out.push_back(sax::Token("push", sax::Token::TokenType::START_ELEMENT));
	for (const auto& symbol : symbols) {
		out.splice(out.end(), alib::api<alphabet::Symbol>::compose(symbol));
	}
	out.push_back(sax::Token("push", sax::Token::TokenType::END_ELEMENT));
}

void AutomatonToXMLComposer::composeTransitionSinglePush(std::list<sax::Token>& out, const alphabet::Symbol& symbol) const {
	out.push_back(sax::Token("push", sax::Token::TokenType::START_ELEMENT));
	out.splice(out.end(), alib::api<alphabet::Symbol>::compose(symbol));
	out.push_back(sax::Token("push", sax::Token::TokenType::END_ELEMENT));
}

void AutomatonToXMLComposer::composeTransitionInputSymbol(std::list<sax::Token>& out, const alphabet::Symbol& symbol) const {
	out.push_back(sax::Token("input", sax::Token::TokenType::START_ELEMENT));
	out.splice(out.end(), alib::api<alphabet::Symbol>::compose(symbol));
	out.push_back(sax::Token("input", sax::Token::TokenType::END_ELEMENT));
}

void AutomatonToXMLComposer::composeTransitionOutputSymbol(std::list<sax::Token>& out, const alphabet::Symbol& symbol) const {
	out.push_back(sax::Token("output", sax::Token::TokenType::START_ELEMENT));
	out.splice(out.end(), alib::api<alphabet::Symbol>::compose(symbol));
	out.push_back(sax::Token("output", sax::Token::TokenType::END_ELEMENT));
}

void AutomatonToXMLComposer::composeTransitionInputEpsilonSymbol(std::list<sax::Token>& out, const std::variant<string::Epsilon, alphabet::Symbol>& symbol) const {
	out.push_back(sax::Token("input", sax::Token::TokenType::START_ELEMENT));
	if(symbol.is<string::Epsilon>()) {
		out.push_back(sax::Token("epsilon", sax::Token::TokenType::START_ELEMENT));
		out.push_back(sax::Token("epsilon", sax::Token::TokenType::END_ELEMENT));
	} else {
		out.splice(out.end(), alib::api<alphabet::Symbol>::compose(symbol.get<alphabet::Symbol>()));
	}
	out.push_back(sax::Token("input", sax::Token::TokenType::END_ELEMENT));
}

void AutomatonToXMLComposer::composeTransitionOutputEpsilonSymbol(std::list<sax::Token>& out, const std::variant<string::Epsilon, alphabet::Symbol>& symbol) const {
	out.push_back(sax::Token("output", sax::Token::TokenType::START_ELEMENT));
	if(symbol.is<string::Epsilon>()) {
		out.push_back(sax::Token("epsilon", sax::Token::TokenType::START_ELEMENT));
		out.push_back(sax::Token("epsilon", sax::Token::TokenType::END_ELEMENT));
	} else {
		out.splice(out.end(), alib::api<alphabet::Symbol>::compose(symbol.get<alphabet::Symbol>()));
	}
	out.push_back(sax::Token("output", sax::Token::TokenType::END_ELEMENT));
}

void AutomatonToXMLComposer::composeTransitionInputString(std::list<sax::Token>& out, const string::LinearString& string) const {
	out.push_back(sax::Token("input", sax::Token::TokenType::START_ELEMENT));
	out.splice(out.end(), alib::api<string::LinearString>::compose(string));
	out.push_back(sax::Token("input", sax::Token::TokenType::END_ELEMENT));
}

void AutomatonToXMLComposer::composeTransitionInputRegexp(std::list<sax::Token>& out, const regexp::RegExp& regexp) const {
	out.push_back(sax::Token("input", sax::Token::TokenType::START_ELEMENT));
	out.splice(out.end(), alib::api<regexp::RegExp>::compose(regexp));
	out.push_back(sax::Token("input", sax::Token::TokenType::END_ELEMENT));
}

std::list<sax::Token> AutomatonToXMLComposer::compose(const Automaton& automaton) const {
	std::list<sax::Token> out;
	automaton.getData().Accept((void*) &out, alib::ToXMLComposers::toXMLComposers);
	return out;
}

std::list<sax::Token> AutomatonToXMLComposer::compose(const AutomatonBase& automaton) const {
	std::list<sax::Token> out;
	automaton.Accept((void*) &out, alib::ToXMLComposers::toXMLComposers);
	return out;
}

std::list<sax::Token> AutomatonToXMLComposer::compose(const DFA& automaton) const {
	std::list<sax::Token> out;
	out.push_back(sax::Token("DFA", sax::Token::TokenType::START_ELEMENT));

	composeStates(out, automaton.getStates());
	composeInputAlphabet(out, automaton.getInputAlphabet());
	composeInitialState(out, automaton.getInitialState());
	composeFinalStates(out, automaton.getFinalStates());
	composeTransitions(out, automaton);

	out.push_back(sax::Token("DFA", sax::Token::TokenType::END_ELEMENT));
	return out;
}

std::list<sax::Token> AutomatonToXMLComposer::compose(const NFA& automaton) const {
	std::list<sax::Token> out;
	out.push_back(sax::Token("NFA", sax::Token::TokenType::START_ELEMENT));

	composeStates(out, automaton.getStates());
	composeInputAlphabet(out, automaton.getInputAlphabet());
	composeInitialState(out, automaton.getInitialState());
	composeFinalStates(out, automaton.getFinalStates());
	composeTransitions(out, automaton);

	out.push_back(sax::Token("NFA", sax::Token::TokenType::END_ELEMENT));
	return out;
}

std::list<sax::Token> AutomatonToXMLComposer::compose(const MultiInitialStateNFA& automaton) const {
	std::list<sax::Token> out;
	out.push_back(sax::Token("MultiInitialStateNFA", sax::Token::TokenType::START_ELEMENT));

	composeStates(out, automaton.getStates());
	composeInputAlphabet(out, automaton.getInputAlphabet());
	composeInitialStates(out, automaton.getInitialStates());
	composeFinalStates(out, automaton.getFinalStates());
	composeTransitions(out, automaton);

	out.push_back(sax::Token("MultiInitialStateNFA", sax::Token::TokenType::END_ELEMENT));
	return out;
}

std::list<sax::Token> AutomatonToXMLComposer::compose(const EpsilonNFA& automaton) const {
	std::list<sax::Token> out;
	out.push_back(sax::Token("EpsilonNFA", sax::Token::TokenType::START_ELEMENT));

	composeStates(out, automaton.getStates());
	composeInputAlphabet(out, automaton.getInputAlphabet());
	composeInitialState(out, automaton.getInitialState());
	composeFinalStates(out, automaton.getFinalStates());
	composeTransitions(out, automaton);

	out.push_back(sax::Token("EpsilonNFA", sax::Token::TokenType::END_ELEMENT));
	return out;
}

std::list<sax::Token> AutomatonToXMLComposer::compose(const ExtendedNFA& automaton) const {
	std::list<sax::Token> out;
	out.push_back(sax::Token("ExtendedNFA", sax::Token::TokenType::START_ELEMENT));

	composeStates(out, automaton.getStates());
	composeInputAlphabet(out, automaton.getInputAlphabet());
	composeInitialState(out, automaton.getInitialState());
	composeFinalStates(out, automaton.getFinalStates());
	composeTransitions(out, automaton);

	out.push_back(sax::Token("ExtendedNFA", sax::Token::TokenType::END_ELEMENT));
	return out;
}

std::list<sax::Token> AutomatonToXMLComposer::compose(const CompactNFA& automaton) const {
	std::list<sax::Token> out;
	out.push_back(sax::Token("CompactNFA", sax::Token::TokenType::START_ELEMENT));

	composeStates(out, automaton.getStates());
	composeInputAlphabet(out, automaton.getInputAlphabet());
	composeInitialState(out, automaton.getInitialState());
	composeFinalStates(out, automaton.getFinalStates());
	composeTransitions(out, automaton);

	out.push_back(sax::Token("CompactNFA", sax::Token::TokenType::END_ELEMENT));
	return out;
}

std::list<sax::Token> AutomatonToXMLComposer::compose(const DPDA& automaton) const {
	std::list<sax::Token> out;
	out.push_back(sax::Token("DPDA", sax::Token::TokenType::START_ELEMENT));

	composeStates(out, automaton.getStates());
	composeInputAlphabet(out, automaton.getInputAlphabet());
	composeStackAlphabet(out, automaton.getStackAlphabet());
	composeInitialState(out, automaton.getInitialState());
	composeInitialStackSymbol(out, automaton.getInitialSymbol());
	composeFinalStates(out, automaton.getFinalStates());
	composeTransitions(out, automaton);

	out.push_back(sax::Token("DPDA", sax::Token::TokenType::END_ELEMENT));
	return out;
}

std::list<sax::Token> AutomatonToXMLComposer::compose(const SinglePopDPDA& automaton) const {
	std::list<sax::Token> out;
	out.push_back(sax::Token("SinglePopDPDA", sax::Token::TokenType::START_ELEMENT));

	composeStates(out, automaton.getStates());
	composeInputAlphabet(out, automaton.getInputAlphabet());
	composeStackAlphabet(out, automaton.getStackAlphabet());
	composeInitialState(out, automaton.getInitialState());
	composeInitialStackSymbol(out, automaton.getInitialSymbol());
	composeFinalStates(out, automaton.getFinalStates());
	composeTransitions(out, automaton);

	out.push_back(sax::Token("SinglePopDPDA", sax::Token::TokenType::END_ELEMENT));
	return out;
}

std::list<sax::Token> AutomatonToXMLComposer::compose(const InputDrivenNPDA& automaton) const {
	std::list<sax::Token> out;
	out.push_back(sax::Token("InputDrivenNPDA", sax::Token::TokenType::START_ELEMENT));

	composeStates(out, automaton.getStates());
	composeInputAlphabet(out, automaton.getInputAlphabet());
	composeStackAlphabet(out, automaton.getStackAlphabet());
	composeInitialStates(out, automaton.getInitialStates());
	composeInitialStackSymbol(out, automaton.getInitialSymbol());
	composeFinalStates(out, automaton.getFinalStates());
	composeInputToPushdownStoreOperation(out, automaton);
	composeTransitions(out, automaton);

	out.push_back(sax::Token("InputDrivenNPDA", sax::Token::TokenType::END_ELEMENT));
	return out;
}

std::list<sax::Token> AutomatonToXMLComposer::compose(const VisiblyPushdownDPDA& automaton) const {
	std::list<sax::Token> out;
	out.push_back(sax::Token("VisiblyPushdownDPDA", sax::Token::TokenType::START_ELEMENT));

	composeStates(out, automaton.getStates());
	composeCallInputAlphabet(out, automaton.getCallInputAlphabet());
	composeReturnInputAlphabet(out, automaton.getReturnInputAlphabet());
	composeLocalInputAlphabet(out, automaton.getLocalInputAlphabet());
	composeStackAlphabet(out, automaton.getStackAlphabet());
	composeInitialState(out, automaton.getInitialState());
	composeBottomOfTheStackSymbol(out, automaton.getBottomOfTheStackSymbol());
	composeFinalStates(out, automaton.getFinalStates());
	composeTransitions(out, automaton);

	out.push_back(sax::Token("VisiblyPushdownDPDA", sax::Token::TokenType::END_ELEMENT));
	return out;
}

std::list<sax::Token> AutomatonToXMLComposer::compose(const VisiblyPushdownNPDA& automaton) const {
	std::list<sax::Token> out;
	out.push_back(sax::Token("VisiblyPushdownNPDA", sax::Token::TokenType::START_ELEMENT));

	composeStates(out, automaton.getStates());
	composeCallInputAlphabet(out, automaton.getCallInputAlphabet());
	composeReturnInputAlphabet(out, automaton.getReturnInputAlphabet());
	composeLocalInputAlphabet(out, automaton.getLocalInputAlphabet());
	composeStackAlphabet(out, automaton.getStackAlphabet());
	composeInitialStates(out, automaton.getInitialStates());
	composeBottomOfTheStackSymbol(out, automaton.getBottomOfTheStackSymbol());
	composeFinalStates(out, automaton.getFinalStates());
	composeTransitions(out, automaton);

	out.push_back(sax::Token("VisiblyPushdownNPDA", sax::Token::TokenType::END_ELEMENT));
	return out;
}

std::list<sax::Token> AutomatonToXMLComposer::compose(const RealTimeHeightDeterministicDPDA& automaton) const {
	std::list<sax::Token> out;
	out.push_back(sax::Token("RealTimeHeightDeterministicDPDA", sax::Token::TokenType::START_ELEMENT));

	composeStates(out, automaton.getStates());
	composeInputAlphabet(out, automaton.getInputAlphabet());
	composeStackAlphabet(out, automaton.getStackAlphabet());
	composeInitialState(out, automaton.getInitialState());
	composeBottomOfTheStackSymbol(out, automaton.getBottomOfTheStackSymbol());
	composeFinalStates(out, automaton.getFinalStates());
	composeTransitions(out, automaton);

	out.push_back(sax::Token("RealTimeHeightDeterministicDPDA", sax::Token::TokenType::END_ELEMENT));
	return out;
}

std::list<sax::Token> AutomatonToXMLComposer::compose(const RealTimeHeightDeterministicNPDA& automaton) const {
	std::list<sax::Token> out;
	out.push_back(sax::Token("RealTimeHeightDeterministicNPDA", sax::Token::TokenType::START_ELEMENT));

	composeStates(out, automaton.getStates());
	composeInputAlphabet(out, automaton.getInputAlphabet());
	composeStackAlphabet(out, automaton.getStackAlphabet());
	composeInitialStates(out, automaton.getInitialStates());
	composeBottomOfTheStackSymbol(out, automaton.getBottomOfTheStackSymbol());
	composeFinalStates(out, automaton.getFinalStates());
	composeTransitions(out, automaton);

	out.push_back(sax::Token("RealTimeHeightDeterministicNPDA", sax::Token::TokenType::END_ELEMENT));
	return out;
}

std::list<sax::Token> AutomatonToXMLComposer::compose(const NPDA& automaton) const {
	std::list<sax::Token> out;
	out.push_back(sax::Token("NPDA", sax::Token::TokenType::START_ELEMENT));

	composeStates(out, automaton.getStates());
	composeInputAlphabet(out, automaton.getInputAlphabet());
	composeStackAlphabet(out, automaton.getStackAlphabet());
	composeInitialStates(out, automaton.getInitialStates());
	composeInitialStackSymbols(out, automaton.getInitialSymbols());
	composeFinalStates(out, automaton.getFinalStates());
	composeTransitions(out, automaton);

	out.push_back(sax::Token("NPDA", sax::Token::TokenType::END_ELEMENT));
	return out;
}

std::list<sax::Token> AutomatonToXMLComposer::compose(const SinglePopNPDA& automaton) const {
	std::list<sax::Token> out;
	out.push_back(sax::Token("SinglePopNPDA", sax::Token::TokenType::START_ELEMENT));

	composeStates(out, automaton.getStates());
	composeInputAlphabet(out, automaton.getInputAlphabet());
	composeStackAlphabet(out, automaton.getStackAlphabet());
	composeInitialStates(out, automaton.getInitialStates());
	composeInitialStackSymbols(out, automaton.getInitialSymbols());
	composeFinalStates(out, automaton.getFinalStates());
	composeTransitions(out, automaton);

	out.push_back(sax::Token("SinglePopNPDA", sax::Token::TokenType::END_ELEMENT));
	return out;
}

std::list<sax::Token> AutomatonToXMLComposer::compose(const OneTapeDTM& automaton) const {
	std::list<sax::Token> out;
	out.push_back(sax::Token("OneTapeDTM", sax::Token::TokenType::START_ELEMENT));

	composeStates(out, automaton.getStates());
	composeInputAlphabet(out, automaton.getInputAlphabet());
	composeTapeAlphabet(out, automaton.getTapeAlphabet());
	composeBlankSymbol(out, automaton.getBlankSymbol());
	composeInitialState(out, automaton.getInitialState());
	composeFinalStates(out, automaton.getFinalStates());
	composeTransitions(out, automaton);

	out.push_back(sax::Token("OneTapeDTM", sax::Token::TokenType::END_ELEMENT));
	return out;
}

std::list<sax::Token> AutomatonToXMLComposer::compose(const State& state) const {
	std::list<sax::Token> out;
	out.splice(out.end(), alib::api<label::Label>::compose(state.getName()));
	return out;
}

} /* namespace automaton */

