/*
 * FiniteAutomatonFromStringLexer.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#include "FiniteAutomatonFromStringLexer.h"

namespace automaton {

FiniteAutomatonFromStringLexer::FiniteAutomatonFromStringLexer(std::istream& in) : m_In(in) {
	m_Current.type = TokenType::ERROR;
	m_Current.value = "";
}

FiniteAutomatonFromStringLexer& FiniteAutomatonFromStringLexer::next() {
	if(m_Current.type == TokenType::TEOF) return *this;
	char character;
	m_Current.value = "";

L0:
	character = m_In.get();
	if(m_In.eof()) {
		m_Current.type = TokenType::TEOF;
		return *this;
	} else if(character == ' ' || character == '\t') {
		goto L0;
	} else if(character == '\n') {
		m_Current.type = TokenType::NEW_LINE;
		m_Current.value += character;
		return *this;
	} else if(character == '<') {
		m_Current.type = TokenType::OUT;
		m_Current.value += character;
		return *this;
	} else if(character == '>') {
		m_Current.type = TokenType::IN;
		m_Current.value += character;
		return *this;
	} else if(character == '|') {
		m_Current.type = TokenType::SEPARATOR;
		m_Current.value += character;
		return *this;
	} else if(character == '-') {
		m_Current.type = TokenType::NONE;
		m_Current.value += character;
		return *this;
	} else if(character == '#') {
		m_Current.value += character;
		goto L1;
	} else if(m_In.unget(), m_In >> "MISNFA") {
		m_Current.type = TokenType::MULTI_INITIAL_STATE_NFA;
		m_Current.value = "MISNFA";
		return *this;
	} else if(m_In.clear(), m_In >> "ENFA") {
		m_Current.type = TokenType::EPSILON_NFA;
		m_Current.value = "ENFA";
		return *this;
	} else if(m_In.clear(), m_In >> "NFA") {
		m_Current.type = TokenType::NFA;
		m_Current.value = "NFA";
		return *this;
	} else if(m_In.clear(), m_In >> "DFA") {
		m_Current.type = TokenType::DFA;
		m_Current.value = "DFA";
		return *this;
	} else {
		m_In.putback(character);
		while(!m_Current.value.empty()) {
			m_In.putback(m_Current.value.back());
			m_In.clear();
			m_Current.value.pop_back();
		}
		m_In.clear();
		m_Current.type = TokenType::ERROR;
		return *this;
	}
L1:
	character = m_In.get();
	if(m_In.eof()) {
		m_Current.type = TokenType::TEOF;
		return *this;
	} else if(character == 'E') {
		m_Current.value += character;
		m_Current.type = TokenType::EPSILON;
		return *this;
	} else {
		m_In.putback(character);
		while(!m_Current.value.empty()) {
			m_In.putback(m_Current.value.back());
			m_Current.value.pop_back();
		}
		m_In.clear();
		m_Current.type = TokenType::ERROR;
		return *this;
	}
}

FiniteAutomatonFromStringLexer::Token FiniteAutomatonFromStringLexer::token() {
	return m_Current;
}

} /* namespace automaton */
