/*
 * PushdownStoreAlphabet.cpp
 *
 *  Created on: Apr 10, 2013
 *      Author: Jan Travnicek
 */

#include "PushdownStoreAlphabet.h"
#include "../AutomatonException.h"
#include <algorithm>

namespace automaton {

bool PushdownStoreAlphabet::addStackSymbol(const alphabet::Symbol& symbol) {
	return stackAlphabet.insert(symbol).second;
}

void PushdownStoreAlphabet::setStackSymbols(const std::set<alphabet::Symbol>& newSymbols) {
	std::set<alphabet::Symbol> removed;
	std::set_difference(stackAlphabet.begin(), stackAlphabet.end(), newSymbols.begin(), newSymbols.end(), std::inserter(removed, removed.end()));

	std::set<alphabet::Symbol> added;
	std::set_difference(newSymbols.begin(), newSymbols.end(), stackAlphabet.begin(), stackAlphabet.end(), std::inserter(added, added.end()));
	
	for(const alphabet::Symbol& removedSymbol : removed) {
		removeStackSymbol(removedSymbol);
	}

	for(const alphabet::Symbol& addedSymbol : added) {
		addStackSymbol(addedSymbol);
	}
}

const std::set<alphabet::Symbol>& PushdownStoreAlphabet::getStackAlphabet() const {
	return stackAlphabet;
}

} /* namespace automaton */
