/*
 * PushdownStoreAlphabet.h
 *
 *  Created on: Apr 10, 2013
 *      Author: Jan Travnicek
 */

#ifndef PUSHDOWN_STORE_ALPHABET_H_
#define PUSHDOWN_STORE_ALPHABET_H_

#include <set>
#include "../../alphabet/Symbol.h"

namespace automaton {

/**
 * Push Down Automaton
 */
class PushdownStoreAlphabet {
protected:
	std::set<alphabet::Symbol> stackAlphabet;
public:
	/**
	 * Adds symbol to the stack alphabet.
	 * @param symbol Symbol to add
	 * @throws AutomatonException when symbol is already present in stack alphabet
	 */
	bool addStackSymbol(const alphabet::Symbol& symbol);

	/**
	 * Sets stack symbols of the automaton.
	 * @param symbols Symbols to set
	 * @throws AutomatonException when symbol is already present in stack alphabet
	 */
	void setStackSymbols(const std::set<alphabet::Symbol>& symbols);

	/**
	 * Removes symbol from the stack alphabet.
	 * @param symbol Symbol to remove
	 * @throws AutomatonException when symbol is used in transition
	 * (popped from or pushed to the stack), when it is a start symbol
	 * or when it is not present in the stack alphabet
	 */
	virtual bool removeStackSymbol(const alphabet::Symbol& symbol) = 0;

	/**
	 * @return the stack alphabet
	 */
	const std::set<alphabet::Symbol>& getStackAlphabet() const;

};

} /* namespace automaton */

#endif /* PUSHDOWN_STORE_ALPHABET_H_ */
