/*
 * wrapper.h
 *
 *  Created on: Apr 10, 2013
 *      Author: Jan Travnicek
 */

#ifndef WRAPPER_H_
#define WRAPPER_H_

#include <ostream>

namespace alib {

template<typename T>
class wrapper {
protected:
	T* data;

public:
	explicit wrapper(const T& data) : data(data.clone()) {

	}

	explicit wrapper(T&& data) : data(std::move(data).plunder()) {

	}

	wrapper(const wrapper& other) : data(other.data->clone()) {

	}

	wrapper(wrapper&& other) noexcept : data(other.data) {
		other.data = NULL;
	}

	wrapper& operator=(const wrapper& other) {
		if(this == &other) return *this;

		delete this->data;
		this->data = other.data->clone();

		return *this;
	}

	wrapper& operator=(wrapper&& other) noexcept {
		std::swap(this->data, other.data);
		return *this;
	}

	virtual ~wrapper() noexcept {
		delete this->data;
	}

	const T& getData() const {
		return *data;
	}

	T& getData() {
		return *data;
	}

	void setData(const T& data) {
		delete this->data;
		this->data = data.clone();
	}

	void setData(T&& data) {
		delete this->data;
		this->data = std::move(data).plunder();
	}

	bool operator>=(const wrapper& other) const {
		return *(this->data) >= *(other.data);
	}

	bool operator<=(const wrapper& other) const {
		return *(this->data) <= *(other.data);
	}

	bool operator>(const wrapper& other) const {
		return *(this->data) > *(other.data);
	}

	bool operator<(const wrapper& other) const {
		return *(this->data) < *(other.data);
	}

	bool operator!=(const wrapper& other) const {
		return *(this->data) != *(other.data);
	}

	bool operator==(const wrapper& other) const {
		return *(this->data) == *(other.data);
	}

	friend std::ostream& operator<<(std::ostream& os, const wrapper& instance) {
		os << *(instance.data);
		return os;
	}

	operator std::string () const {
		return (std::string) *data;
	}
};

} /* namespace alib */

#endif /* WRAPPER_H_ */

