/*
 * ContainerToXMLComposer.cpp
 *
 *  Created on: Oct 12, 2013
 *      Author: Jan Travnicek
 */

#include "ContainerToXMLComposer.hpp"

namespace container {

std::list<sax::Token> ContainerToXMLComposer::compose(const ContainerBase& container) const {
	std::list<sax::Token> out;
	container.Accept((void*) &out, alib::ToXMLComposers::toXMLComposers);
	return out;
}

std::list<sax::Token> ContainerToXMLComposer::compose(const Container& container) const {
	std::list<sax::Token> out;
	container.getData().Accept((void*) &out, alib::ToXMLComposers::toXMLComposers);
	return out;
}

std::list<sax::Token> ContainerToXMLComposer::compose(const ObjectsSet& container) const {
	std::list<sax::Token> out;
	out.push_back(sax::Token("set", sax::Token::TokenType::START_ELEMENT));

	for(const alib::Object& item : container) {
		out.splice(out.end(), alib::api<alib::Object>::compose(item));
	}

	out.push_back(sax::Token("set", sax::Token::TokenType::END_ELEMENT));
	return out;
}

std::list<sax::Token> ContainerToXMLComposer::compose(const ObjectsVector& container) const {
	std::list<sax::Token> out;
	out.push_back(sax::Token("vector", sax::Token::TokenType::START_ELEMENT));

	for(const alib::Object& item : container) {
		out.splice(out.end(), alib::api<alib::Object>::compose(item));
	}

	out.push_back(sax::Token("vector", sax::Token::TokenType::END_ELEMENT));
	return out;
}

std::list<sax::Token> ContainerToXMLComposer::compose(const ObjectsPair& container) const {
	std::list<sax::Token> out;
	out.push_back(sax::Token("pair", sax::Token::TokenType::START_ELEMENT));

	out.splice(out.end(), alib::api<alib::Object>::compose(container.first));
	out.splice(out.end(), alib::api<alib::Object>::compose(container.second));

	out.push_back(sax::Token("pair", sax::Token::TokenType::END_ELEMENT));
	return out;
}

std::list<sax::Token> ContainerToXMLComposer::compose(const ObjectsMap& container) const {
	std::list<sax::Token> out;
	out.push_back(sax::Token("map", sax::Token::TokenType::START_ELEMENT));

	for(const std::pair<alib::Object, alib::Object>& item : container) {
		out.splice(out.end(), alib::api<std::pair<alib::Object, alib::Object>>::compose(item));
	}

	out.push_back(sax::Token("map", sax::Token::TokenType::END_ELEMENT));
	return out;
}

} /* namespace container */

