/*
 * EpsilonFreeCFG.h
 *
 *  Created on: Nov 17, 2013
 *      Author: Jan Travnicek
 */

#ifndef EPSILON_FREE_CFG_H_
#define EPSILON_FREE_CFG_H_

#include "../GrammarBase.h"
#include <map>
#include <vector>
#include <sstream>
#include "../common/TerminalNonterminalAlphabetInitialSymbol.h"

namespace grammar {

/**
 * Epsilon free context free grammar. Type 2 in Chomsky hierarchy. Produces context free languages.
 */
class EpsilonFreeCFG : public std::acceptor<EpsilonFreeCFG, VisitableGrammarBase, std::acceptor<EpsilonFreeCFG, alib::VisitableObjectBase, GrammarBase> >, public TerminalNonterminalAlphabetInitialSymbol {
	std::map<alphabet::Symbol, std::set<std::vector<alphabet::Symbol>>> rules;
	bool generatesEpsilon;

public:
	explicit EpsilonFreeCFG(const alphabet::Symbol& initialSymbol);

	explicit EpsilonFreeCFG(const std::set<alphabet::Symbol>& nonTerminalSymbols, const std::set<alphabet::Symbol>& terminalSymbols, const alphabet::Symbol& initialSymbol);

	virtual GrammarBase* clone() const;

	virtual GrammarBase* plunder() &&;

	bool addRule(const alphabet::Symbol& leftHandSide, const std::vector<alphabet::Symbol>& rightHandSide);

	const std::map<alphabet::Symbol, std::set<std::vector<alphabet::Symbol>>> & getRules() const;

	bool removeRule(const alphabet::Symbol& leftHandSide, const std::vector<alphabet::Symbol>& rightHandSide);

	bool addRawRule(const alphabet::Symbol& leftHandSide, const std::vector<alphabet::Symbol>& rightHandSide);

	std::map<alphabet::Symbol, std::set<std::vector<alphabet::Symbol>>> getRawRules() const;

	bool removeRawRule(const alphabet::Symbol& leftHandSide, const std::vector<alphabet::Symbol>& rightHandSide);

	bool removeTerminalSymbol(const alphabet::Symbol& symbol);

	bool removeNonterminalSymbol(const alphabet::Symbol& symbol);

	void setGeneratesEpsilon(bool genEps);
	bool getGeneratesEpsilon() const;

	virtual bool operator <(const alib::ObjectBase& other) const;
	virtual bool operator ==(const alib::ObjectBase& other) const;
	virtual bool operator >(const alib::ObjectBase& other) const;

	virtual bool operator==(const EpsilonFreeCFG& other) const;

	virtual bool operator<(const EpsilonFreeCFG& other) const;

	virtual void operator>>(std::ostream& os) const;

	virtual operator std::string () const;

	virtual int selfTypeId() const {
		return typeId<EpsilonFreeCFG>();
	}
};

} /* namespace grammar */

#endif /* EPSILON_FREE_CFG_H_ */
