/*
 * GrammarFromStringParser.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#include "GrammarFromStringParser.h"

#include "../exception/AlibException.h"


namespace grammar {

GrammarFromStringParser::GrammarFromStringParser(std::istream& input) : m_GrammarLexer(input), m_SymbolParser(input) {

}

Grammar GrammarFromStringParser::parse() {
	return parse(std::set<FEATURES>({}));
}

Grammar GrammarFromStringParser::parse(const std::set<FEATURES>& features) {
	throw exception::AlibException();
}

bool GrammarFromStringParser::first() {
	GrammarFromStringLexer::Token token = m_GrammarLexer.next().token();
	if(token.type == GrammarFromStringLexer::TokenType::TUPLE_BEGIN) {
		return true;
	} else {
		return false;
	}
}

bool GrammarFromStringParser::next() {
	GrammarFromStringLexer::Token token = m_GrammarLexer.next().token();
	if(token.type == GrammarFromStringLexer::TokenType::COMMA || token.type == GrammarFromStringLexer::TokenType::SEPARATOR || token.type == GrammarFromStringLexer::TokenType::SET_BEGIN || token.type == GrammarFromStringLexer::TokenType::SET_END || token.type == GrammarFromStringLexer::TokenType::TUPLE_BEGIN || token.type == GrammarFromStringLexer::TokenType::TUPLE_END || token.type == GrammarFromStringLexer::TokenType::EPSILON || token.type == GrammarFromStringLexer::TokenType::MAPS_TO) {
		return true;
	} else {
		return false;
	}
}

bool GrammarFromStringParser::last() {
	GrammarFromStringLexer::Token token = m_GrammarLexer.next().token();
	if(token.type == GrammarFromStringLexer::TokenType::TEOF) {
		return true;
	} else {
		return false;
	}
}

std::set<alphabet::Symbol> GrammarFromStringParser::parseSet() {
	std::set<alphabet::Symbol> res;

	GrammarFromStringLexer::Token token = m_GrammarLexer.token();
	if(token.type != GrammarFromStringLexer::TokenType::SET_BEGIN) {
		throw exception::AlibException();
	}
	next() || m_SymbolParser.first() || m_SymbolParser.m_LabelParser.first();

	token = m_GrammarLexer.token();
	if(token.type != GrammarFromStringLexer::TokenType::SET_END) while(true) {
		alphabet::Symbol symbol = m_SymbolParser.parse();
		res.insert(symbol);

		next();
		token = m_GrammarLexer.token();
		if(token.type != GrammarFromStringLexer::TokenType::COMMA) {
			break;
		}
		next() || m_SymbolParser.first() || m_SymbolParser.m_LabelParser.first();
	}
	
	if(token.type != GrammarFromStringLexer::TokenType::SET_END) {
		throw exception::AlibException();
	}
	next();

	return res;
}

} /* namespace grammar */
