/*
 * LabelSetLabel.h
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travnicek
 */

#ifndef LABEL_SET_LABEL_H_
#define LABEL_SET_LABEL_H_

#include <string>
#include <ostream>

#include "Label.h"
#include "LabelBase.h"
#include <set>

namespace label {

/**
 * Represents symbol in an alphabet.
 */
class LabelSetLabel : public std::acceptor<LabelSetLabel, VisitableLabelBase, std::acceptor<LabelSetLabel, alib::VisitableObjectBase, LabelBase> > {
protected:
	std::set<Label> label;

public:
	/**
	 * Creates new symbol with given name.
	 * @param symbol name of the symbol
	 */
	explicit LabelSetLabel(const std::set<Label>& label);

	explicit LabelSetLabel(std::set<Label>&& label);

	virtual LabelBase* clone() const;

	virtual LabelBase* plunder() &&;

	virtual bool operator<(const alib::ObjectBase& other) const;

	virtual bool operator>(const alib::ObjectBase& other) const;

	virtual bool operator==(const alib::ObjectBase& other) const;

	/**
	 * @return name of the symbol
	 */
	const std::set<Label>& getData() const;

	virtual bool operator <(const LabelSetLabel& other) const;
	virtual bool operator ==(const LabelSetLabel& other) const;

	virtual void operator>>(std::ostream&) const;

	virtual operator std::string () const;

	virtual int selfTypeId() const {
		return typeId<LabelSetLabel>();
	}
};

} /* namespace label */

#endif /* LABEL_SET_LABEL_H_ */

