/*
 * PrimitiveFromStringLexer.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#include "PrimitiveFromStringLexer.h"

namespace primitive {

PrimitiveFromStringLexer::PrimitiveFromStringLexer(std::istream& in) : m_In(in) {
	m_Current.type = TokenType::ERROR;
	m_Current.value = "";
}

PrimitiveFromStringLexer& PrimitiveFromStringLexer::next() {
	if(m_Current.type == TokenType::TEOF) return *this;
	char character;
	m_Current.value = "";

L0:
	character = m_In.get();
	if(m_In.eof()) {
		m_Current.type = TokenType::TEOF;
		return *this;
	} else if(character == ' ' || character == '\n' || character == '\t') {
		goto L0;
	} else if((character >= 'a' && character <= 'z') || (character >= 'A' && character <= 'Z')) {
		m_Current.type = TokenType::CHAR;
		m_Current.value += character;
		return *this;
	} else if(character >= '0' && character <= '9') {
		m_Current.type = TokenType::INTEGER;
		m_Current.value += character;
		goto L1;
	} else if(character == '\'') {
		m_Current.type = TokenType::STRING;
		goto L3;
	} else {
		m_In.putback(character);
		while(!m_Current.value.empty()) {
			m_In.putback(m_Current.value.back());
			m_Current.value.pop_back();
		}
		m_In.clear();
		m_Current.type = TokenType::ERROR;
		return *this;
	}
L1:
	character = m_In.get();
	if(m_In.eof()) {
		return *this;
	} else if(character >= '0' && character <= '9') {
		m_Current.value += character;
		goto L1;
	} else {
		m_In.unget();
		return *this;
	}
L3:
	character = m_In.get();
	if(m_In.eof()) {
		m_Current.type = TokenType::TEOF;
		return *this;
	} else if(character == '\'') {
		return *this;
	} else if(character == '\\') {
		goto L4;
	} else {
		m_Current.value += character;
		goto L3;
	}
L4:
	character = m_In.get();
	if(m_In.eof()) {
		m_Current.type = TokenType::TEOF;
		return *this;
	} else if(character == '\'' || character == '\\') {
		m_Current.value += character;
		goto L3;
	} else {
		m_In.putback(character);
		while(!m_Current.value.empty()) {
			m_In.putback(m_Current.value.back());
			m_Current.value.pop_back();
		}
		m_In.clear();
		m_Current.type = TokenType::ERROR;
		return *this;
	}
}

PrimitiveFromStringLexer::Token PrimitiveFromStringLexer::token() {
	return m_Current;
}

} /* namespace primitive */
