/*
 * FormalRegExpEmpty.cpp
 *
 *  Created on: Jan 30, 2014
 *      Author: Jan Travnicek
 */

#include "FormalRegExpEmpty.h"
#include "../unbounded/UnboundedRegExpEmpty.h"

namespace regexp {

FormalRegExpEmpty::FormalRegExpEmpty() {
  // so that default constructor is available
}

FormalRegExpEmpty::FormalRegExpEmpty(const FormalRegExpEmpty&) {
  // so that copy constructor is available
}

FormalRegExpEmpty::FormalRegExpEmpty(FormalRegExpEmpty&&) noexcept {
  this->attachRegExp(NULL);
}

FormalRegExpEmpty& FormalRegExpEmpty::operator =(const FormalRegExpEmpty&) {
  //this is actually different than default implementation
  return *this;
}

FormalRegExpEmpty& FormalRegExpEmpty::operator =(FormalRegExpEmpty&&) noexcept {
  //this is actually different than default implementation
  return *this;
}

FormalRegExpElement* FormalRegExpEmpty::clone() const {
	return new FormalRegExpEmpty(*this);
}

FormalRegExpElement* FormalRegExpEmpty::plunder() && {
	return new FormalRegExpEmpty(std::move(*this));
}

UnboundedRegExpElement* FormalRegExpEmpty::cloneAsUnbounded() const {
	return new UnboundedRegExpEmpty();
}

bool FormalRegExpEmpty::operator<(const FormalRegExpElement& other) const {
	return other > *this;
}

bool FormalRegExpEmpty::operator==(const FormalRegExpElement& other) const {
	return other == *this;
}

bool FormalRegExpEmpty::operator>(const FormalRegExpElement& other) const {
	return other < *this;
}


bool FormalRegExpEmpty::operator<(const FormalRegExpEmpty&) const {
	return false;
}

bool FormalRegExpEmpty::operator==(const FormalRegExpEmpty&) const {
	return true;
}

void FormalRegExpEmpty::operator>>(std::ostream& out) const {
	out << "(FormalRegExpEmpty)";
}

bool FormalRegExpEmpty::testSymbol( const alphabet::Symbol & ) const {
	return false;
}

bool FormalRegExpEmpty::attachRegExp(const FormalRegExp * regexp ) {
	if(this->parentRegExp == regexp) return true;
	this->parentRegExp = regexp;
	return true;
}

void FormalRegExpEmpty::computeMinimalAlphabet( std::set<alphabet::Symbol>&) const {

}

FormalRegExpEmpty::operator std::string() const {
	std::stringstream ss;
	ss << *this;
	return ss.str();
}

} /* namespace regexp */
