/*
 * UnboundedRegExpIteration.h
 *
 *  Created on: Nov 23, 2013
 *      Author: Martin Zak
 */

#ifndef UNBOUNDED_REG_EXP_ITERATION_H_
#define UNBOUNDED_REG_EXP_ITERATION_H_

#include "UnboundedRegExpElement.h"
#include "UnboundedRegExpEmpty.h"

namespace regexp {

class RegExpOptimize;

/**
 * Represents iteration operator in the regular expression. Contains one UnboundedRegExpElement
 * as operand.
 */
class UnboundedRegExpIteration: public std::acceptor<UnboundedRegExpIteration, UnboundedRegExpElement, UnboundedRegExpElement> {
protected:
	UnboundedRegExpElement* element;

	/**
	 * @copydoc UnboundedRegExpElement::cloneAsFormal() const
	 */
	virtual FormalRegExpElement* cloneAsFormal() const;

	/**
	 * @copydoc UnboundedRegExpElement::testSymbol() const
	 */
	virtual bool testSymbol( const alphabet::Symbol & symbol ) const;

	/**
	 * @copydoc UnboundedRegExpElement::attachRegExp()
	 */
	virtual bool attachRegExp ( const UnboundedRegExp * regexp );
	
	/**
	 * @copydoc UnboundedRegExpElement::computeMinimalAlphabet()
	 */
	virtual void computeMinimalAlphabet( std::set<alphabet::Symbol>& alphabet ) const;
	
public:
	explicit UnboundedRegExpIteration(UnboundedRegExpElement&&);
	explicit UnboundedRegExpIteration(const UnboundedRegExpElement&);
	
	UnboundedRegExpIteration(const UnboundedRegExpIteration& other);
	UnboundedRegExpIteration(UnboundedRegExpIteration&& other) noexcept;
	UnboundedRegExpIteration& operator =(const UnboundedRegExpIteration& other);
	UnboundedRegExpIteration& operator =(UnboundedRegExpIteration&& other);
	virtual ~UnboundedRegExpIteration() noexcept;

	/**
	 * @copydoc UnboundedRegExpElement::clone() const
	 */
	virtual UnboundedRegExpElement* clone() const;
	
	/**
	 * @copydoc UnboundedRegExpElement::plunder() const
	 */
	virtual UnboundedRegExpElement* plunder() &&;

	/**
	 * @return element
	 */
	const UnboundedRegExpElement & getElement() const;
	
	/**
	 * @return element
	 */
	UnboundedRegExpElement & getElement();
	
	/**
	 * @param element to iterate
	 */
	void setElement(const UnboundedRegExpElement& element);
	
	void setElement(UnboundedRegExpElement&& element);

	virtual bool operator<(const UnboundedRegExpElement&) const;
	virtual bool operator==(const UnboundedRegExpElement&) const;
	virtual bool operator>(const UnboundedRegExpElement&) const;

	virtual bool operator<(const UnboundedRegExpIteration&) const;
	virtual bool operator==(const UnboundedRegExpIteration&) const;

	/**
	 * @copydoc UnboundedRegExpElement::operator>>() const
	 */
	virtual void operator>>(std::ostream& out) const;

	friend class RegExpOptimize;

	virtual operator std::string() const;

	virtual int selfTypeId() const {
		return typeId<UnboundedRegExpIteration>();
	}
};

} /* namespace regexp */

#endif /* UNBOUNDED_REG_EXP_ITERATION_H_ */
