/*
 * UnboundedRegExpSymbol.h
 *
 *  Created on: Nov 23, 2013
 *      Author: Martin Zak
 */

#ifndef UNBOUNDED_REG_EXP_SYMBOL_H_
#define UNBOUNDED_REG_EXP_SYMBOL_H_

#include "../../label/Label.h"
#include "UnboundedRegExpElement.h"
#include "../../alphabet/LabeledSymbol.h"

namespace regexp {

/**
 * Represents symbol in the regular expression. Contains name of the symbol.
 */
class UnboundedRegExpSymbol : public std::acceptor<UnboundedRegExpSymbol, UnboundedRegExpElement, UnboundedRegExpElement> {
protected:
	alphabet::Symbol symbol;

	/**
	 * @copydoc UnboundedRegExpElement::cloneAsFormal() const
	 */
	virtual FormalRegExpElement* cloneAsFormal() const;

	/**
	 * @copydoc UnboundedRegExpElement::testSymbol() const
	 */
	virtual bool testSymbol( const alphabet::Symbol & symbol ) const;
		
	/**
	 * @copydoc UnboundedRegExpElement::attachRegExp()
	 */
	virtual bool attachRegExp ( const UnboundedRegExp * regexp );
	
	/**
	 * @copydoc UnboundedRegExpElement::computeMinimalAlphabet()
	 */
	virtual void computeMinimalAlphabet( std::set<alphabet::Symbol>& alphabet ) const;
public:
	explicit UnboundedRegExpSymbol(int number);
	explicit UnboundedRegExpSymbol(char character);
	explicit UnboundedRegExpSymbol(const std::string& label);

	explicit UnboundedRegExpSymbol(const alphabet::Symbol& symbol);
	explicit UnboundedRegExpSymbol(alphabet::Symbol&& symbol);
	
	UnboundedRegExpSymbol(const UnboundedRegExpSymbol& other);
	UnboundedRegExpSymbol(UnboundedRegExpSymbol&& other) noexcept;
	UnboundedRegExpSymbol& operator=(const UnboundedRegExpSymbol& other);
	UnboundedRegExpSymbol& operator=(UnboundedRegExpSymbol&& other);

	/**
	 * @copydoc UnboundedRegExpElement::clone() const
	 */
	virtual UnboundedRegExpElement* clone() const;
	
	/**
	 * @copydoc UnboundedRegExpElement::plunder() const
	 */
	virtual UnboundedRegExpElement* plunder() &&;

	bool operator==(const alphabet::Symbol&) const;
	friend bool operator==(const alphabet::Symbol&, const UnboundedRegExpSymbol&);

	virtual bool operator<(const UnboundedRegExpElement&) const;
	virtual bool operator==(const UnboundedRegExpElement&) const;
	virtual bool operator>(const UnboundedRegExpElement&) const;

	virtual bool operator<(const UnboundedRegExpSymbol&) const;
	virtual bool operator==(const UnboundedRegExpSymbol&) const;
	
	/**
	 * @copydoc UnboundedRegExpElement::operator>>() const
	 */
	virtual void operator>>(std::ostream& out) const;
	
	/**
	 * Returns the string representation of RegExp Symbol.
	 */
	const alphabet::Symbol& getSymbol() const;

	virtual operator std::string() const;

	virtual int selfTypeId() const {
		return typeId<UnboundedRegExpSymbol>();
	}
};

} /* namespace regexp */

#endif /* UNBOUNDED_REG_EXP_SYMBOL_H_ */

