/*
 * Epsilon.h
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#ifndef EPSILON_H_
#define EPSILON_H_

#include <iostream>
#include <set>
#include <vector>

#include "../std/visitor.hpp"
#include "../alphabet/Symbol.h"
#include "StringBase.h"
#include "common/StringAlphabet.h"

namespace string {

/**
 * Represents epsilon. Regular expression is stored
 * as a tree of EpsilonElement.
 */
class Epsilon : public std::acceptor<Epsilon, VisitableStringBase, std::acceptor<Epsilon, alib::VisitableObjectBase, StringBase> >, public StringAlphabet {
public:
	virtual StringBase* clone() const;
	virtual StringBase* plunder() &&;

	virtual bool removeSymbolFromAlphabet(const alphabet::Symbol & symbol);

	virtual bool operator <(const alib::ObjectBase& other) const;
	virtual bool operator ==(const alib::ObjectBase& other) const;
	virtual bool operator >(const alib::ObjectBase& other) const;

	/**
	 * @return List of symbols forming string (const version).
	 */
	std::vector<alphabet::Symbol> getContent() const;

	/**
	 * @return true
	 */
	bool isEmpty() const;

	virtual bool operator<(const LinearString& other) const;
	virtual bool operator<(const CyclicString& other) const;
	virtual bool operator<(const Epsilon& other) const;

	virtual bool operator==(const LinearString& other) const;
	virtual bool operator==(const CyclicString& other) const;
	virtual bool operator==(const Epsilon& other) const;

	virtual void operator >>(std::ostream& out) const;

	virtual operator std::string() const;

	static Epsilon EPSILON;

	virtual int selfTypeId() const {
		return typeId<Epsilon>();
	}
};

} /* namespace string */

#endif /* EPSILON_H_ */

