#include "StdVisitorTest.h"
#include "std/visitor.hpp"
#include "common/base.hpp"
#include <set>
#include <string>

CPPUNIT_TEST_SUITE_REGISTRATION( StdVisitorTest );

void StdVisitorTest::setUp() {
}

void StdVisitorTest::tearDown() {
}

class Tmp1;
class Tmp2;
class Tmp3;

class TmpBase;

typedef std::acceptor_base<TmpBase,
			Tmp1, Tmp2, Tmp3
	> VisitableTmpBase;

class TmpBase :
	public alib::base<
			TmpBase,
			Tmp1, Tmp2, Tmp3
	>, public VisitableTmpBase {
};

class Tmp1 : public std::acceptor<Tmp1, VisitableTmpBase, TmpBase> {
	int data;
public:
	Tmp1(int data) : data(data) {
	
	}

	TmpBase* clone() const {
		return new Tmp1(*this);
	}

	TmpBase* plunder() && {
		return new Tmp1(*this);
	}

	virtual bool operator<(const TmpBase& other) const {
		return other > *this;
	}

	virtual bool operator==(const TmpBase& other) const {
		return other == *this;
	}

	virtual bool operator>(const TmpBase& other) const {
		return other < *this;
	}

	virtual bool operator==(const Tmp1& other) const {
		return this->data == other.data;
	}

	virtual bool operator<(const Tmp1& other) const {
		return this->data < other.data;
	}

	virtual void operator>>(std::ostream& os) const {
		os << "Tmp1(" << data <<  ")";
	}

	virtual operator std::string() const {
		return "Tmp1(" + std::to_string(data) + ")";
	}

	virtual int selfTypeId() const {
		return typeId<Tmp1>();
	}

	int getData() const {
		return data;
	}

};

class Tmp2 : public std::acceptor<Tmp2, VisitableTmpBase, TmpBase> {
	double data;
public:
	Tmp2(double data) : data(data) {
	
	}

	Tmp2(const Tmp1& other) : data(other.getData()) {

	}

	TmpBase* clone() const {
		return new Tmp2(*this);
	}

	TmpBase* plunder() && {
		return new Tmp2(*this);
	}

	virtual bool operator<(const TmpBase& other) const {
		return other > *this;
	}

	virtual bool operator==(const TmpBase& other) const {
		return other == *this;
	}

	virtual bool operator>(const TmpBase& other) const {
		return other < *this;
	}

	virtual bool operator==(const Tmp2& other) const {
		return this->data == other.data;
	}

	virtual bool operator<(const Tmp2& other) const {
		return this->data < other.data;
	}

	virtual void operator>>(std::ostream& os) const {
		os << "Tmp2(" << data <<  ")";
	}

	virtual operator std::string() const {
		return "Tmp2(" + std::to_string(data) + ")";
	}

	virtual int selfTypeId() const {
		return typeId<Tmp2>();
	}

	double getData() const {
		return data;
	}

};

class Tmp3 : public std::acceptor<Tmp3, VisitableTmpBase, TmpBase> {
	std::string data;
public:
	Tmp3(const std::string& data) : data(data) {
	
	}

	Tmp3(const Tmp1& other) : data(std::to_string(other.getData())) {

	}

	Tmp3(const Tmp2& other) : data(std::to_string(other.getData())) {

	}

	TmpBase* clone() const {
		return new Tmp3(*this);
	}

	TmpBase* plunder() && {
		return new Tmp3(*this);
	}

	virtual bool operator<(const TmpBase& other) const {
		return other > *this;
	}

	virtual bool operator==(const TmpBase& other) const {
		return other == *this;
	}

	virtual bool operator>(const TmpBase& other) const {
		return other < *this;
	}

	virtual bool operator==(const Tmp3& other) const {
		return this->data == other.data;
	}

	virtual bool operator<(const Tmp3& other) const {
		return this->data < other.data;
	}

	virtual void operator>>(std::ostream& os) const {
		os << "Tmp3(" << data <<  ")";
	}

	virtual operator std::string() const {
		return "Tmp3(" + data + ")";
	}

	virtual int selfTypeId() const {
		return typeId<Tmp3>();
	}

	const std::string& getData() const {
		return data;
	}

};

class TmpVisitor : public VisitableTmpBase::const_visitor_type {
	void Visit(void* userData, const Tmp1& first) const {
		int& data = *((int*) userData);
		data = 1;
		std::cout << first << std::endl;
	}

	void Visit(void* userData, const Tmp2& first) const {
		int& data = *((int*) userData);
		data = 2;
		std::cout << first << std::endl;
	}

	void Visit(void* userData, const Tmp3& first) const {
		int& data = *((int*) userData);
		data = 3;
		std::cout << first << std::endl;
	}
};

class TmpSameVisitor : public VisitableTmpBase::const_same_visitor_type {
	void Visit(void* userData, const Tmp1& first, const Tmp1& second) const {
		int& data = *((int*) userData);
		data = 1;
		std::cout << first << " " << second << std::endl;
	}

	void Visit(void* userData, const Tmp2& first, const Tmp2& second) const {
		int& data = *((int*) userData);
		data = 2;
		std::cout << first << " " << second << std::endl;
	}

	void Visit(void* userData, const Tmp3& first, const Tmp3& second) const {
		int& data = *((int*) userData);
		data = 3;
		std::cout << first << " " << second << std::endl;
	}
};

class TmpPromotingVisitor : public VisitableTmpBase::const_promoting_visitor_type {
	void Visit(void* userData, const Tmp1& first, const Tmp1& second) const {
		int& data = *((int*) userData);
		data = 1;
		std::cout << first << " " << second << std::endl;
	}

	void Visit(void* userData, const Tmp2& first, const Tmp2& second) const {
		int& data = *((int*) userData);
		data = 2;
		std::cout << first << " " << second << std::endl;
	}

	void Visit(void* userData, const Tmp3& first, const Tmp3& second) const {
		int& data = *((int*) userData);
		data = 3;
		std::cout << first << " " << second << std::endl;
	}
};

void StdVisitorTest::testVisitor() {
	TmpVisitor visitor;

	Tmp1 tmp1(2);

	int a = 0;
	tmp1.Accept((void*) &a, visitor);

	CPPUNIT_ASSERT(a == 1);
}

void StdVisitorTest::testSameVisitor() {
	TmpSameVisitor visitor;

	Tmp2 tmpA(2);
	Tmp2 tmpB(3);

	int a = 0;
	Accept((void*) &a, (TmpBase&) tmpA, (TmpBase&) tmpB, visitor);

	CPPUNIT_ASSERT(a == 2);
}
void StdVisitorTest::testPromoteVisitor() {
	TmpPromotingVisitor visitor;

	Tmp1 tmp1(2);
	Tmp3 tmp3("3");

	int a = 0;
	Accept((void*) &a, (TmpBase&) tmp1, (TmpBase&) tmp3, visitor);

	CPPUNIT_ASSERT(a == 3);
}
